@@ -27,7 +27,7 @@ typedef struct {
 	gchar *my_char;
 } Tconvert_table;
 
-typedef enum { 
+typedef enum {
 	tcc2i_full_match = 0,
 	tcc2i_firstchar,
 	tcc2i_mycharlen,
@@ -53,6 +53,8 @@ void refcpointer_unref(Trefcpointer *rp);
 /*gchar *filename_utf8_from_uri(GFile *uri);*/
 /*gchar *get_utf8filename_from_on_disk_encoding(const gchar *encodedname);*/
 
+gchar *mime_with_extension(const gchar *mimetype, const gchar *filename);
+
 GList *urilist_to_stringlist(GList *urilist);
 void free_urilist(GList *urilist);
 #if GTK_CHECK_VERSION(2,14,0)
@@ -67,6 +69,10 @@ gchar *filename_utf8_from_full_path_utf8(const gchar *full_path_utf8);
 gchar *get_filename_on_disk_encoding(const gchar *utf8filename);
 
 gboolean string_is_color(const gchar *color);
+
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 gchar *filemode_to_string(mode_t statmode);
 /*gchar *return_root_with_protocol(const gchar *url);*/
 void pointer_switch_addresses(gpointer *a, gpointer *b);
@@ -132,4 +138,17 @@ void callback_remove_all(GSList **slist);
 char *strcasestr(char *a, char *b);
 #endif
 
+#define BF_ELIST_HEAD gpointer next, prev
+typedef struct {
+	BF_ELIST_HEAD;
+} Telist;
+#define BF_ELIST(var) ((Telist *)var)
+#define bf_elist_next(var) (var)->next
+#define bf_elist_prev(var) (var)->prev
+
+Telist * bf_elist_prepend(gpointer cur, gpointer new);
+Telist * bf_elist_append(gpointer cur, gpointer new);
+Telist * bf_elist_first(gpointer cur);
+Telist * bf_elist_last(gpointer cur);
+Telist * bf_elist_remove(gpointer toremove);
 #endif /* __BF_LIB_H_ */
