@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bftextview2_autocomp.c
  *
- * Copyright (C) 2008,2009,2010,2011 Olivier Sessink
+ * Copyright (C) 2008,2009,2010,2011,2012,2013 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -116,6 +116,80 @@ acwin_move_selection(BluefishTextView * btv, gint keyval)
 	return FALSE;
 }
 
+static gchar *
+string_maintain_indenting(BluefishTextView * btv, gchar *string, gint *backward_cursor)
+{
+	gchar *indentstring;
+	gint i=0, len;
+	GtkTextIter iter;
+	GString *gstr;
+	/*g_print("string_maintain_indenting, check %s for newlines\n",string);*/
+	/* check if there are newlines in the string*/
+	if (!string || string[0]=='\0' || !strchr(string, '\n')) {
+		return string;
+	}
+	gtk_text_buffer_get_iter_at_mark(btv->buffer, &iter, gtk_text_buffer_get_insert(btv->buffer));
+	indentstring = get_line_indenting(btv->buffer, &iter, FALSE);
+	/*g_print("indentstring='%s' with len %d\n",indentstring,strlen(indentstring));*/
+	if (!indentstring || indentstring[0]=='\0') {
+		return string;
+	}
+	/* now do the replace of \n with \n+indentstring */
+	len = strlen(string);
+	gstr = g_string_new("");
+	while(string[i] != '\0') {
+		g_string_append_c(gstr, string[i]);
+		if (string[i] == '\n') {
+			g_string_append(gstr, indentstring);
+			if (i>=(len-*backward_cursor)) {
+				*backward_cursor += strlen(indentstring);
+			}
+		}
+		i++;
+	}
+	g_free(string);
+	string = gstr->str;
+	g_string_free(gstr, FALSE);
+	return string;
+}
+
+/* returns the number of bytes
+that are already present in the text.
+
+this is used to avoid for example inserting "return;" in a location where ';' is already the
+character directly beyond the insert position.
+*/
+static gint
+get_existing_end_len(BluefishTextView * btv, const gchar *string, gint prefix_bytelen)
+{
+	gchar *tmp;
+	GtkTextIter it1, it2;
+	gint i,len;
+	gint string_len = g_utf8_strlen(string, -1);
+
+	gtk_text_buffer_get_iter_at_mark(btv->buffer, &it1, gtk_text_buffer_get_insert(btv->buffer));
+	it2 = it1;
+	DBG_AUTOCOMP("get_existing_end_len, forward %d chars\n",string_len - prefix_bytelen);
+	gtk_text_iter_forward_chars(&it2,string_len - prefix_bytelen);
+	DBG_AUTOCOMP("get the text %d:%d\n",gtk_text_iter_get_offset(&it1),gtk_text_iter_get_offset(&it2));
+	tmp = gtk_text_buffer_get_text(btv->buffer, &it1, &it2, TRUE);
+	/*g_print("got tmp='%s'\n",tmp);*/
+	len = strlen(tmp);
+	i = len-1;
+	do {
+		DBG_AUTOCOMP("get_existing_end_len, compare %d characters of %s and %s\n",i,string+prefix_bytelen+len-i,tmp);
+		if (strncmp(string+prefix_bytelen+len-i, tmp, i)==0) {
+			DBG_AUTOCOMP("get_existing_end_len, found %d existing characters\n",i);
+			g_free(tmp);
+			return i;
+		}
+		i--;
+	} while(i>0);
+	g_free(tmp);
+	DBG_AUTOCOMP("get_existing_end_len, found no existing characters\n");
+	return 0;
+}
+
 gboolean
 acwin_check_keypress(BluefishTextView * btv, GdkEventKey * event)
 {
@@ -133,7 +207,8 @@ acwin_check_keypress(BluefishTextView * btv, GdkEventKey * event)
 			selection = gtk_tree_view_get_selection(ACWIN(btv->autocomp)->tree);
 			if (selection && gtk_tree_selection_get_selected(selection, &model, &it)) {
 				gchar *string;
-				GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(btv));
+				gint stringlen;
+				gint existing_len, prefix_len;
 				guint pattern_id;
 				gint backup_chars = 0;
 				gtk_tree_model_get(model, &it, 1, &string, -1);
@@ -161,15 +236,22 @@ acwin_check_keypress(BluefishTextView * btv, GdkEventKey * event)
 						}
 					}
 				}
+				stringlen = strlen(string);
+				prefix_len = strlen(ACWIN(btv->autocomp)->prefix);
+				existing_len = get_existing_end_len(btv, string, prefix_len);
+
 				DBG_AUTOCOMP("acwin_check_keypress: ENTER: insert %s\n",
-							 string + strlen(ACWIN(btv->autocomp)->prefix));
-				gtk_text_buffer_insert_at_cursor(buffer, string + strlen(ACWIN(btv->autocomp)->prefix), -1);
+							 string + prefix_len);
+				string[stringlen - existing_len] = '\0';
+				/* see if there are any \n characters in the autocompletion string, and add the current indenting to them */
+				string = string_maintain_indenting(btv, string, &backup_chars);
+				gtk_text_buffer_insert_at_cursor(btv->buffer, string + prefix_len, -1);
 				if (backup_chars != 0) {
 					GtkTextIter iter;
-					gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
-					if (gtk_text_iter_backward_chars(&iter, backup_chars)) {
+					gtk_text_buffer_get_iter_at_mark(btv->buffer, &iter, gtk_text_buffer_get_insert(btv->buffer));
+					if (backup_chars-existing_len > 0 && gtk_text_iter_backward_chars(&iter, backup_chars-existing_len)) {
 						DBG_AUTOCOMP("move cursor %d chars back!\n", backup_chars);
-						gtk_text_buffer_place_cursor(buffer, &iter);
+						gtk_text_buffer_place_cursor(btv->buffer, &iter);
 					}
 				}
 
@@ -516,6 +598,15 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 		return;
 	}
 
+	/* see if we have enough characters */
+	if (!user_requested && gtk_text_iter_get_offset(&cursorpos) - gtk_text_iter_get_offset(&iter) < main_v->props.autocomp_min_prefix_len) {
+		DBG_AUTOCOMP("autocomp_run, prefix len %d < autocomp_min_prefix_len (%d), abort!\n"
+					, gtk_text_iter_get_offset(&cursorpos) - gtk_text_iter_get_offset(&iter)
+					, main_v->props.autocomp_min_prefix_len);
+		acwin_cleanup(btv);
+		return;
+	}
+
 	if (g_array_index(master->bflang->st->contexts, Tcontext, contextnum).has_tagclose_from_blockstack) {
 		Tfound *found;
 		GSequenceIter *siter = NULL;
@@ -523,10 +614,9 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 		if (found) {
 			fblock =
 				found->numblockchange < 0 ? pop_blocks(found->numblockchange, found->fblock) : found->fblock;
-			if (fblock) {
-				DBG_AUTOCOMP("blockstack has pattern %d on top, with tagclose_from_blockstack=%d\n",
-						 fblock->patternum, g_array_index(master->bflang->st->matches, Tpattern,
-														  fblock->patternum).tagclose_from_blockstack);
+			if (fblock && fblock->start2_o != BF_OFFSET_UNDEFINED) {
+				g_print("abort offering closing tag: block has an end already\n");
+				fblock = NULL;
 			}
 /*		if (g_array_index(btv->bflang->st->matches, Tpattern, fblock->patternum).tagclose_from_blockstack) {
 				gchar *start;
