@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bftextview2_langmgr.c
  *
- * Copyright (C) 2008,2009,2010,2011,2012 Olivier Sessink
+ * Copyright (C) 2008,2009,2010,2011,2012,2013 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,15 +29,16 @@
 #include "bftextview2_patcompile.h"
 
 #include "bluefish.h"
+#include "bf_lib.h"
 #include "bfwin_uimanager.h"
 #include "document.h"
 #include "stringlist.h"
 
 #define UNDEFINED -1
 
-/*#undef DBG_PARSING 
+/*#undef DBG_PARSING
 #define DBG_PARSING g_print
-#undef DBG_MSG 
+#undef DBG_MSG
 #define DBG_MSG g_print*/
 
 typedef struct {
@@ -72,6 +73,7 @@ typedef struct {
 } Tautocomplete;
 
 typedef struct {
+	gboolean done_scanning;
 	GHashTable *bflang_lookup;
 	GList *bflang_list;
 	GtkTextTagTable *tagtable;	/* a GtkTextTagTable uses a hashtable internally, so lookups by name are fast */
@@ -90,9 +92,9 @@ typedef struct {
 #endif
 } Tlangmgr;
 #ifdef HAVE_LIBENCHANT
-static Tlangmgr langmgr = { NULL, NULL, NULL, 0, NULL, NULL, NULL, NULL };
+static Tlangmgr langmgr = { FALSE, NULL, NULL, NULL, 0, NULL, NULL, NULL, NULL };
 #else
-static Tlangmgr langmgr = { NULL, NULL, NULL, 0, NULL, NULL };
+static Tlangmgr langmgr = { FALSE, NULL, NULL, NULL, 0, NULL, NULL };
 #endif
 
 /* utils */
@@ -276,7 +278,7 @@ langmgr_reload_user_styles(void)
 		}
 		i++;
 	}
-	
+
 	langmgr.highlight_tags = texttag_array_from_list(highlightlist);
 #ifdef HAVE_LIBENCHANT
 	langmgr.need_spellcheck_tags = texttag_array_from_list(needlist);
@@ -293,7 +295,7 @@ langmgr_insert_user_option(gchar * lang, gchar * option, gchar * val)
 	}
 }
 
-static const gchar *
+const gchar *
 lookup_user_option(const gchar * lang, const gchar * option)
 {
 	if (lang && option) {
@@ -329,7 +331,7 @@ langmgr_reload_user_options(void)
 	}
 
 	/* loop over the languages and set some of the options */
-	
+
 	tmplist = g_list_first(langmgr.bflang_list);
 	while (tmplist) {
 		tmp = lookup_user_option(BFLANG(tmplist->data)->name, "show_in_menu");
@@ -337,7 +339,7 @@ langmgr_reload_user_options(void)
 		if (BFLANG(tmplist->data)->st) {
 			for (i = 1; i < BFLANG(tmplist->data)->st->blocks->len; i++) {
 				if (g_array_index(BFLANG(tmplist->data)->st->blocks, Tpattern_block, i).name) {
-					g_array_index(BFLANG(tmplist->data)->st->blocks, Tpattern_block, i).foldable 
+					g_array_index(BFLANG(tmplist->data)->st->blocks, Tpattern_block, i).foldable
 							= lookup_block_foldable(
 									BFLANG(tmplist->data)->name,
 									g_array_index(BFLANG(tmplist->data)->st->blocks, Tpattern_block, i).name
@@ -418,7 +420,7 @@ static gboolean
 build_lang_finished_lcb(gpointer data)
 {
 	Tbflangparsing *bfparser = data;
-	
+	DEBUG_SIG("build_lang_finished_lcb, priority=%d\n",BUILD_LANG_FINISHED_PRIORITY);
 	if (bfparser->st) {
 		bfparser->bflang->st = bfparser->st;
 		/*bfparser->bflang->line = bfparser->line;
@@ -460,8 +462,27 @@ typedef struct {
 	Tattribtype type;
 } Tattrib;
 
+static gchar *
+process_condition(Tbflang *bflang, gchar *value)
+{
+	gchar *tok1, *tok2, *lookupval;
+	tok1 = strchr(value, '?');
+	if (!tok1)
+		return value;
+	tok2 = strchr(tok1, ':');
+	if (!tok2)
+		return value;
+	*tok1 = '\0';
+	*tok2 = '\0';
+	lookupval = (gchar *) lookup_user_option(bflang->name, value);
+	if (lookupval && lookupval[0]=='1') {
+		return tok1+1;
+	}
+	return tok2+1;
+}
+
 static void
-parse_attributes(xmlTextReaderPtr reader, Tattrib *attribs, gint num_attribs)
+parse_attributes(Tbflang *bflang,xmlTextReaderPtr reader, Tattrib *attribs, gint num_attribs)
 {
 	while (xmlTextReaderMoveToNextAttribute(reader)) {
 		gint i;
@@ -469,21 +490,28 @@ parse_attributes(xmlTextReaderPtr reader, Tattrib *attribs, gint num_attribs)
 		for (i=0;i<num_attribs;i++) {
 			if (xmlStrEqual(aname, (xmlChar *)attribs[i].name)) {
 				gchar *value = (gchar *)xmlTextReaderValue(reader);
+				gchar *useval = value;
+				if (strncmp(value, "option:", 7)==0) {
+					/*g_print("lookup option %s\n",value+7);*/
+					useval = (gchar *) lookup_user_option(bflang->name, value+7);
+					/*g_print("useval=%s\n",useval);*/
+				} else if (strncmp(value, "condition:",10)==0) {
+					useval = process_condition(bflang,value+10);
+				}
 				switch (attribs[i].type) {
 					case attribtype_int:
-						*((gint *)attribs[i].var) = (gint) g_ascii_strtoll(value, NULL, 10);
-						xmlFree(value);
+						*((gint *)attribs[i].var) = GPOINTER_TO_INT(useval)?g_ascii_strtoll(useval, NULL, 10):0;
 					break;
 					case attribtype_string:
-						*((gchar **)attribs[i].var) = value;
+						*((gchar **)attribs[i].var) = g_strdup(useval?useval:"");
 					break;
 					case attribtype_boolean:
-						*((gboolean *)attribs[i].var) = (value[0]=='1');
-						xmlFree(value);
+						*((gboolean *)attribs[i].var) = useval?(useval[0]=='1'):FALSE;
 					break;
 				}
+				xmlFree(value);
 				break;
-			}	
+			}
 		}
 		xmlFree(aname);
 	}
@@ -496,16 +524,16 @@ process_header(xmlTextReaderPtr reader, Tbflang * bflang)
 	while (xmlTextReaderRead(reader) == 1) {
 		int nodetype = xmlTextReaderNodeType(reader);
 		xmlChar *name;
-		
+
 		if (nodetype == 14 /* #text */) {
 			continue;
 		}
-		
+
 		name = xmlTextReaderName(reader);
 		if (xmlStrEqual(name, (xmlChar *) "mime")) {
 			gchar *mimetype=NULL;
 			Tattrib attribs[] = {{"type", &mimetype, attribtype_string}};
-			parse_attributes(reader, attribs, 1);
+			parse_attributes(bflang,reader, attribs, 1);
 			if (mimetype) {
 				bflang->mimetypes = g_list_prepend(bflang->mimetypes, mimetype);
 			}
@@ -516,7 +544,7 @@ process_header(xmlTextReaderPtr reader, Tbflang * bflang)
 			Tattrib attribs[] = {{"name", &optionname, attribtype_string},
 					{"description", &description, attribtype_string},
 					{"default", &defaultval, attribtype_boolean}};
-			parse_attributes(reader, attribs, 3);
+			parse_attributes(bflang,reader, attribs, 3);
 			if (optionname) {
 				const gchar *val = lookup_user_option(bflang->name, optionname);
 				if (!val) {
@@ -534,7 +562,7 @@ process_header(xmlTextReaderPtr reader, Tbflang * bflang)
 						g_hash_table_insert(langmgr.option_descriptions, optionname, description);
 						free_description = FALSE;
 					}
-				
+
 				}
 				if (free_description) {
 					g_free(optionname);
@@ -545,7 +573,7 @@ process_header(xmlTextReaderPtr reader, Tbflang * bflang)
 			gchar *name = NULL, *style = NULL;
 			Tattrib attribs[] = {{"name", &name, attribtype_string},
 					{"style", &style, attribtype_string}};
-			parse_attributes(reader, attribs, 2);
+			parse_attributes(bflang,reader, attribs, 2);
 			if (name) {
 				gchar *use_textstyle = langmgr_lookup_style_for_highlight(bflang->name, name);
 				if (use_textstyle) {	/* we have a user-configured textstyle for this highlight */
@@ -577,7 +605,7 @@ process_header(xmlTextReaderPtr reader, Tbflang * bflang)
 		}
 		xmlFree(name);
 	}
-	
+
 	tmp = lookup_user_option(bflang->name, "show_in_menu");
 	bflang->in_menu = !(tmp && tmp[0] == '0');
 }
@@ -588,17 +616,17 @@ do_parse(Tbflangparsing * bfparser, const gchar *class, const gchar *notclass)
 	const gchar *haveclass=NULL, *havenotclass=NULL;
 /*
 	if class is not set and notclass is not set we return TRUE
-	
+
 	if only class is set, and haveclass is not set, we return TRUE
 	if only class is set, and haveclass is also set, we return haveclass[0] == '1'
-	
+
 	if only notclass is set, and havenotclass is not set, we return FALSE
 	if only notclass is set, and havenotclass is also set, we return haveclass[0] != '1'
-	
+
 	if both haveclass and havenotclass are both set, the havenotclass overrules
 	if both class and notclass are both set, the notclass overrules (return FALSE)
-	
-	
+
+
 */
 	if (!class && !notclass)
 		return TRUE;
@@ -610,13 +638,13 @@ do_parse(Tbflangparsing * bfparser, const gchar *class, const gchar *notclass)
 	DBG_PARSING("do_parse, class=%s, haveclass=%s, notclass=%s, havenotclass=%s\n",class, haveclass, notclass, havenotclass);
 	if (havenotclass)
 		return havenotclass[0] != '1';
-	
+
 	if (haveclass)
 		return haveclass[0] == '1';
-	
+
 	if (notclass)
 		return FALSE;
-	
+
 	return TRUE;
 }
 
@@ -648,7 +676,7 @@ add_autocomplete(Tbflangparsing * bfparser, guint16 matchnum, GSList *list, cons
 				backup_cursor = strlen(string)-1;
 				freestring=TRUE;
 			}
-			match_add_autocomp_item(bfparser->st, matchnum, 
+			match_add_autocomp_item(bfparser->st, matchnum,
 										(ac->mode == ac_mode_string) ? string : NULL,
 										(ac->mode == ac_mode_append) ? string : NULL,
 										backup_cursor);
@@ -658,8 +686,8 @@ add_autocomplete(Tbflangparsing * bfparser, guint16 matchnum, GSList *list, cons
 	}
 }
 
-static void 
-process_autocomplete(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GSList **list) 
+static void
+process_autocomplete(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GSList **list)
 {
 	gboolean enable=TRUE;
 	gchar *string=NULL, *append=NULL, *class=NULL, *notclass=NULL;
@@ -676,8 +704,8 @@ process_autocomplete(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GSList
 	};
 	if (!bfparser->load_completion)
 		return;
-	
-	parse_attributes(reader, attribs, 6);
+
+	parse_attributes(bfparser->bflang,reader, attribs, 6);
 	enabled = do_parse(bfparser, class, notclass);
 	g_free(class);
 	g_free(notclass);
@@ -750,7 +778,7 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 					{"identifier_jump", &identifier_jump, attribtype_int},
 					{"identifier_autocomp", &identifier_autocomp, attribtype_int}
 					};
-	parse_attributes(reader, attribs, 19);
+	parse_attributes(bfparser->bflang,reader, attribs, 19);
 	if (stretch_blockstart && ends_block) {
 		g_print("Error in language file, id %s / pattern %s has mutually exclusive options stretch_blockstart and ends_block both enabled\n", id?id:"-", pattern?pattern:"null");
 		stretch_blockstart = FALSE;
@@ -801,7 +829,7 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 				DBG_PARSING("got blockstartelementum %d for blockstartelement %s, ends_block=%d\n",
 							blockstartelementum, blockstartelement, ends_block);
 			}
-			matchnum = add_pattern_to_scanning_table(bfparser->st, 
+			matchnum = add_pattern_to_scanning_table(bfparser->st,
 								pattern,
 								is_regex != UNDEFINED ? is_regex : (ih_is_regex !=UNDEFINED ? ih_is_regex : FALSE),
 								case_insens != UNDEFINED ? case_insens : (ih_case_insens != UNDEFINED ? ih_case_insens :FALSE),
@@ -810,9 +838,9 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 				foldable = lookup_block_foldable(bfparser->bflang->name, block_name);
 			}
 			pattern_set_blockmatch(bfparser->st, matchnum,
-								starts_block, 
-								ends_block, 
-								blockstartelementum, 
+								starts_block,
+								ends_block,
+								blockstartelementum,
 								blockhighlight,
 								block_name,
 								foldable);
@@ -872,7 +900,7 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 	}
 	/* TODO cleanup! */
 	free_autocomplete(autocomplete);
-	
+
 	if (pattern)
 		xmlFree(pattern);
 	if (id)
@@ -895,6 +923,63 @@ stringcmp(const void *sp1, const void *sp2)
 	return (strcmp(*(char **) sp1, *(char **) sp2));
 }
 
+static guint16
+add_string(Tbflangparsing * bfparser, guint16 contexttag, const gchar *stringname, const gchar *singlematch, const gchar *fullmatch)
+{
+	static const gchar *stringhighlight = "attribute-string";
+	guint16 strcontext, endmatch=0, matchstring;
+	const gchar *val = lookup_user_option(bfparser->bflang->name, "attribute_string_is_block");
+	gchar *contextname = g_strconcat(stringname, ".context", NULL);
+	strcontext = GPOINTER_TO_INT(g_hash_table_lookup(bfparser->contexts, contextname));
+	if (strcontext == 0) {
+		strcontext = new_context(bfparser->st, 4, bfparser->bflang->name, (gchar *)singlematch, stringhighlight, FALSE, FALSE);
+		endmatch = add_pattern_to_scanning_table(bfparser->st, singlematch, FALSE, FALSE, strcontext);
+		pattern_set_runtime_properties(bfparser->st, endmatch, stringhighlight,-1,FALSE,FALSE,0,FALSE,FALSE);
+	}
+	matchstring = add_pattern_to_scanning_table(bfparser->st, singlematch, FALSE, FALSE, contexttag);
+	pattern_set_runtime_properties(bfparser->st, matchstring, stringhighlight,strcontext,FALSE,FALSE,0,FALSE,FALSE);
+	if (val && val[0]=='1') {
+		pattern_set_blockmatch(bfparser->st, matchstring,TRUE,FALSE,0,NULL,NULL,FALSE);
+		if (endmatch!=0) {
+			pattern_set_blockmatch(bfparser->st, endmatch,FALSE,TRUE,matchstring,NULL,NULL,FALSE);
+		}
+	}
+	g_hash_table_insert(bfparser->patterns, g_strdup(stringname),
+						GINT_TO_POINTER((gint) matchstring));
+	return matchstring;
+}
+
+static inline void
+add_attribute_to_tag(Tbflangparsing * bfparser, const gchar *attrstring, gint contexttag, const gchar * attribhighlight
+					, const gchar *attrib_autocomplete_append, gint attrib_autocomplete_backup_cursor)
+{
+	guint16 attrmatch;
+	if (strchr(attrstring, '=')== NULL) {
+		attrmatch = add_pattern_to_scanning_table(bfparser->st, attrstring, FALSE, TRUE, contexttag);
+		pattern_set_runtime_properties(bfparser->st, attrmatch,attribhighlight, 0, FALSE, FALSE,0, FALSE, FALSE);
+		match_add_autocomp_item(bfparser->st, attrmatch, NULL,attrib_autocomplete_append,attrib_autocomplete_backup_cursor);
+		match_autocomplete_reference(bfparser->st, attrmatch, contexttag);
+	} else {
+		/* contains a =, so split the attribute, and the context */
+		gint16 startscontext;
+		gchar *tmp;
+		gchar **splitted = g_strsplit(attrstring, "=", -1);
+		
+		/*g_print("tag attribute contains a '=', lookup context %s\n",splitted[1]);*/
+		startscontext = GPOINTER_TO_INT(g_hash_table_lookup(bfparser->contexts, splitted[1]));
+		if (startscontext == 0) {
+			g_warning("attribute %s refers to context %s which does not (yet) exist\n",splitted[0],splitted[1]);
+		}
+		tmp = g_strdup_printf("%s[ \t]*=", splitted[0]);
+		attrmatch = add_pattern_to_scanning_table(bfparser->st, tmp, TRUE, TRUE, contexttag);
+		g_free(tmp);
+		pattern_set_runtime_properties(bfparser->st, attrmatch,attribhighlight,startscontext,FALSE,FALSE,0,FALSE,FALSE);
+		match_add_autocomp_item(bfparser->st, attrmatch, splitted[0],attrib_autocomplete_append,attrib_autocomplete_backup_cursor);
+		match_autocomplete_reference(bfparser->st, attrmatch, contexttag);
+		g_strfreev(splitted);
+	}
+}
+
 static guint16
 process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16 context,
 					 GQueue * contextstack, gchar * ih_highlight,
@@ -925,7 +1010,7 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 					{"no_close", &no_close, attribtype_boolean},
 					{"attrib_autocomplete_append", &attrib_autocomplete_append, attribtype_string},
 					{"attrib_autocomplete_backup_cursor", &attrib_autocomplete_backup_cursor, attribtype_int}};
-	parse_attributes(reader, attribs, bfparser->load_completion ? 14 : 12);
+	parse_attributes(bfparser->bflang,reader, attribs, bfparser->load_completion ? 14 : 12);
 	add_tag = do_parse(bfparser, class, notclass);
 	if (add_tag) {
 		if (idref && idref[0] && !tag) {
@@ -937,7 +1022,6 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 			if (matchnum)
 				compile_existing_match(bfparser->st, matchnum, context);
 		} else if (tag && tag[0]) {
-			static const gchar *stringhighlight = "attribute-string";
 			gchar *attrib_context_id = NULL;
 			gchar **attrib_arr = NULL;
 			guint16 starttagmatch = 0, endtagmatch, matchstring;
@@ -978,12 +1062,12 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 				foldable = lookup_block_foldable(bfparser->bflang->name, block_name);
 			}
 			pattern_set_blockmatch(bfparser->st, matchnum,TRUE,FALSE,0,NULL,block_name,foldable);
-			pattern_set_runtime_properties(bfparser->st, matchnum, 
+			pattern_set_runtime_properties(bfparser->st, matchnum,
 								highlight ? highlight : ih_highlight,
 								contexttag, TRUE, FALSE,0, FALSE, FALSE);
 			DBG_PARSING("insert tag %s into hash table with matchnum %d\n", id ? id : tmp, matchnum);
 			if (g_hash_table_lookup(bfparser->patterns, id ? id : tmp) != NULL) {
-				g_print("Possible error in language file, id %s already exists\n", id ? id : tmp);
+				g_warning("Possible error in language file, pattern with id %s already exists\n", id ? id : tmp);
 			} else {
 				g_hash_table_insert(bfparser->patterns, g_strdup(id ? id : tmp),
 									GINT_TO_POINTER((gint) matchnum));
@@ -991,26 +1075,19 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 			g_free(tmp);
 
 			if (!contexttag) {
-				static const gchar *internal_tag_string_d = "__internal_tag_string_d__";
-				static const gchar *internal_tag_string_s = "__internal_tag_string_s__";
-				contexttag = new_context(bfparser->st, 8, bfparser->bflang->name, ">\"=' \t\n\r<", NULL, FALSE);
+				static const gchar *internal_tag_string_d = "__internal__.e.tag.attribute.string.d";
+				static const gchar *internal_tag_string_s = "__internal__.e.tag.attribute.string.s";
+				contexttag = new_context(bfparser->st, 8, bfparser->bflang->name, ">\"=' \t\n\r<", NULL, FALSE, FALSE);
 				match_set_nextcontext(bfparser->st, matchnum, contexttag);
 				if (attrib_arr) {
 					gchar **tmp2;
 
 					tmp2 = attrib_arr;
 					while (*tmp2) {
-						guint16 attrmatch = add_pattern_to_scanning_table(bfparser->st, *tmp2, FALSE, TRUE, contexttag);
-						pattern_set_runtime_properties(bfparser->st, attrmatch, 
-								attribhighlight ? attribhighlight : ih_attribhighlight,
-								0, FALSE, FALSE,0, FALSE, FALSE);
-						match_add_autocomp_item(bfparser->st, attrmatch, NULL,
-												attrib_autocomplete_append ? attrib_autocomplete_append :
-												ih_attrib_autocomplete_append,
-												attrib_autocomplete_backup_cursor ?
-												attrib_autocomplete_backup_cursor :
-												ih_attrib_autocomplete_backup_cursor);
-						match_autocomplete_reference(bfparser->st, attrmatch, contexttag);
+						add_attribute_to_tag(bfparser, *tmp2, contexttag
+									, attribhighlight ? attribhighlight : ih_attribhighlight
+									, attrib_autocomplete_append ? attrib_autocomplete_append:ih_attrib_autocomplete_append
+									, attrib_autocomplete_backup_cursor ? attrib_autocomplete_backup_cursor :ih_attrib_autocomplete_backup_cursor);
 						tmp2++;
 					}
 				}
@@ -1019,29 +1096,42 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 					/*g_print("adding existing tag string %s to context %d\n",internal_tag_string_d,contexttag); */
 					compile_existing_match(bfparser->st, matchstring, contexttag);
 				} else {
-					/*g_print("adding new tag string %s to context %d and highlight %s\n",internal_tag_string_d,contexttag,stringhighlight); */
-					matchstring = add_pattern_to_scanning_table(bfparser->st, "\"[^\"]*\"", TRUE, FALSE, contexttag);
-					pattern_set_runtime_properties(bfparser->st, matchstring, stringhighlight,
-								0, FALSE, FALSE,0, FALSE, FALSE);
+					add_string(bfparser, contexttag, internal_tag_string_d, "\"", "\"[^\"]*\"");
+/*					gchar *val = lookup_user_option(bfparser->bflang->name, "attribute_string_is_block");
+					if (val && val[0]=='1') {
+						guint16 strcontext, endmatch;
+						strcontext = new_context(bfparser->st, 4, bfparser->bflang->name, "\"", stringhighlight, FALSE);
+						matchstring = add_pattern_to_scanning_table(bfparser->st, "\"", FALSE, FALSE, contexttag);
+						pattern_set_runtime_properties(bfparser->st, matchstring, stringhighlight,strcontext,FALSE,FALSE,0,FALSE,FALSE);
+						pattern_set_blockmatch(bfparser->st, matchstring,TRUE,FALSE,0,NULL,NULL,FALSE);
+						endmatch = add_pattern_to_scanning_table(bfparser->st, "\"", FALSE, FALSE, strcontext);
+						pattern_set_runtime_properties(bfparser->st, endmatch, stringhighlight,-1,FALSE,FALSE,0,FALSE,FALSE);
+						pattern_set_blockmatch(bfparser->st, endmatch,FALSE,TRUE,matchstring,NULL,NULL,FALSE);
+					} else {
+						matchstring = add_pattern_to_scanning_table(bfparser->st, "\"[^\"]*\"", TRUE, FALSE, contexttag);
+						pattern_set_runtime_properties(bfparser->st, matchstring, stringhighlight,
+									0, FALSE, FALSE,0, FALSE, FALSE);
+					}
 					g_hash_table_insert(bfparser->patterns, g_strdup(internal_tag_string_d),
-										GINT_TO_POINTER((gint) matchstring));
+										GINT_TO_POINTER((gint) matchstring));*/
 				}
 				matchstring = GPOINTER_TO_INT(g_hash_table_lookup(bfparser->patterns, internal_tag_string_s));
 				if (matchstring) {
 					/*g_print("adding existing tag string %s to context %d\n",internal_tag_string_s,contexttag); */
 					compile_existing_match(bfparser->st, matchstring, contexttag);
 				} else {
-					/*g_print("adding new tag string %s to context %d and highlight %s\n",internal_tag_string_s,contexttag,stringhighlight); */
+					add_string(bfparser, contexttag, internal_tag_string_s, "'", "'[^']*'");
+/*					static const gchar *stringhighlight = "attribute-string";
 					matchstring = add_pattern_to_scanning_table(bfparser->st, "'[^']*'", TRUE, FALSE, contexttag);
 					pattern_set_runtime_properties(bfparser->st, matchstring, stringhighlight,
 								0, FALSE, FALSE,0, FALSE, FALSE);
 					g_hash_table_insert(bfparser->patterns, g_strdup(internal_tag_string_s),
-										GINT_TO_POINTER((gint) matchstring));
+										GINT_TO_POINTER((gint) matchstring));*/
 				}
 
 				if (!sgml_shorttag) {
 					guint16 tmpnum = add_pattern_to_scanning_table(bfparser->st, "/>", FALSE, FALSE, contexttag);
-					pattern_set_runtime_properties(bfparser->st, tmpnum, 
+					pattern_set_runtime_properties(bfparser->st, tmpnum,
 								highlight ? highlight : ih_highlight,
 								-1, FALSE, FALSE,0, FALSE, FALSE);
 					pattern_set_blockmatch(bfparser->st, tmpnum,
@@ -1052,7 +1142,7 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 				}
 				tmp = no_close ? NULL : g_strconcat("</", tag, ">", NULL);
 				starttagmatch = add_pattern_to_scanning_table(bfparser->st, ">", FALSE, FALSE, contexttag);
-				pattern_set_runtime_properties(bfparser->st, starttagmatch, 
+				pattern_set_runtime_properties(bfparser->st, starttagmatch,
 								highlight ? highlight : ih_highlight,
 								-1, FALSE, bfparser->stretch_tag_block,0, FALSE, FALSE);
 				pattern_set_blockmatch(bfparser->st, starttagmatch, FALSE, FALSE, matchnum /* blockstartpattern for stretch_block */, NULL, NULL, TRUE);
@@ -1078,11 +1168,11 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 				while (xmlTextReaderRead(reader) == 1) {
 					int nodetype = xmlTextReaderNodeType(reader);
 					xmlChar *name;
-					
+
 					if (nodetype == 14) {
 						continue;
 					}
-					
+
 					name = xmlTextReaderName(reader);
 					if (xmlStrEqual(name, (xmlChar *) "reference")) {
 						if (!xmlTextReaderIsEmptyElement(reader)) {
@@ -1192,7 +1282,7 @@ process_scanning_group(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gint
 					{"attribhighlight", &attribhighlight, attribtype_string},
 					{"attrib_autocomplete_append", &attrib_autocomplete_append, attribtype_string},
 					{"attrib_autocomplete_backup_cursor", &attrib_autocomplete_backup_cursor, attribtype_int}};
-	parse_attributes(reader, attribs, bfparser->load_completion ? 8 : 6);
+	parse_attributes(bfparser->bflang,reader, attribs, bfparser->load_completion ? 8 : 6);
 	add_group = do_parse(bfparser, class, notclass);
 	DBG_PARSING("add_group=%d for class=%s and notclass=%s\n",add_group, class, notclass);
 	if (!add_group) {
@@ -1203,11 +1293,11 @@ process_scanning_group(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gint
 		while (xmlTextReaderRead(reader) == 1) {
 			int nodetype = xmlTextReaderNodeType(reader);
 			xmlChar *name;
-			
+
 			if (nodetype == 14 /* #text */) {
 				continue;
-			} 
-			
+			}
+
 			name = xmlTextReaderName(reader);
 			if (xmlStrEqual(name, (xmlChar *) "element")) {
 				process_scanning_element(reader, bfparser, context, contextstack,
@@ -1299,8 +1389,9 @@ process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GQu
 {
 	gchar *symbols = NULL, *highlight = NULL, *id = NULL, *idref = NULL, *commentid_block =
 		NULL, *commentid_line = NULL;
-	gboolean autocomplete_case_insens = FALSE;
-	gint context;
+	gboolean autocomplete_case_insens = FALSE, isempty;
+	gint default_spellcheck = SPELLCHECK_INHERIT;
+	gint context, depth;
 
 	Tattrib attribs[] = {{"id", &id, attribtype_string},
 					{"idref", &idref, attribtype_string},
@@ -1308,26 +1399,47 @@ process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GQu
 					{"highlight", &highlight, attribtype_string},
 					{"commentid_block", &commentid_block, attribtype_string},
 					{"commentid_line", &commentid_line, attribtype_string},
+					{"default_spellcheck", &default_spellcheck, attribtype_boolean},
 					{"autocomplete_case_insens", &autocomplete_case_insens, attribtype_boolean}};
-	parse_attributes(reader, attribs, bfparser->load_completion ? 7 : 6);
+#ifdef HAVE_LIBENCHANT
+	if (g_queue_get_length(contextstack)==0) {
+		DBG_PARSING("top level context, set default_spellcheck to %d\n", bfparser->default_spellcheck);
+		default_spellcheck = bfparser->default_spellcheck;
+	}
+#endif
+	depth = xmlTextReaderDepth(reader);
+	isempty = xmlTextReaderIsEmptyElement(reader);
+	
+	parse_attributes(bfparser->bflang,reader, attribs, bfparser->load_completion ? 7 : 6);
 	DBG_PARSING("found <context> with id=%s, idref=%s\n", id, idref);
-	if (idref && idref[0] && !id && !symbols && !highlight && !autocomplete_case_insens) {
+	if (idref && idref[0]) {
 		DBG_PARSING("lookup context %s in hash table..\n", idref);
 		context = GPOINTER_TO_INT(g_hash_table_lookup(bfparser->contexts, idref));
 		g_free(idref);
-		return context;
+		if (context != 0 || (!id && !symbols && !highlight && !autocomplete_case_insens)) {
+			/* if the tag is not empty, we have to forward to the end of the tag now */
+			if (!isempty)
+				skip_to_end_tag(reader, depth);
+			g_free(id);
+			g_free(symbols);
+			g_free(highlight);
+			g_free(commentid_block);
+			g_free(commentid_line);
+			return context;
+		}
 	}
 	if (!symbols) {
+		g_warning("language file has context without symbols, abort.");
 		return 0;
 	}
 	/* create context */
-	DBG_PARSING("create context symbols %s and highlight %s\n", symbols, highlight);
-	context = new_context(bfparser->st, 32, bfparser->bflang->name, symbols, highlight, autocomplete_case_insens);
+	DBG_PARSING("create context %s, symbols %s and highlight %s\n", id, symbols, highlight);
+	context = new_context(bfparser->st, 32, bfparser->bflang->name, symbols, highlight, autocomplete_case_insens, default_spellcheck);
 	g_queue_push_head(contextstack, GINT_TO_POINTER(context));
 	if (id) {
 		DBG_PARSING("insert context %s into hash table as %d\n", id, context);
 		if (g_hash_table_lookup(bfparser->contexts, id) != NULL) {
-			g_print("Error in language file, context id %s already exists\n", id);
+			g_print("Error in language file, context id %s already exists. Set both id and idref if this is caused by a bfinc file that is included more than once\n", id);
 		} else {
 			g_hash_table_insert(bfparser->contexts, g_strdup(id), GINT_TO_POINTER(context));
 		}
@@ -1355,10 +1467,15 @@ process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GQu
 			process_scanning_group(reader, bfparser, context, contextstack, NULL, NULL, NULL, NULL,
 								   UNDEFINED, UNDEFINED, 0, NULL);
 		} else if (xmlStrEqual(name, (xmlChar *) "context")) {
-			xmlFree(name);
-			DBG_PARSING("parsing context, end-of-context, return context %d\n", context);
-			g_queue_pop_head(contextstack);
-			return context;
+			if (depth == xmlTextReaderDepth(reader)) {
+				xmlFree(name);
+				DBG_PARSING("parsing context, end-of-context, return context %d\n", context);
+				g_queue_pop_head(contextstack);
+				return context;
+			} else {
+				/* a context inside a context is usually a context that is referred to by various other elements or tags */
+				process_scanning_context(reader, bfparser, contextstack);
+			}
 		} else {
 			DBG_PARSING("parsing context, found %s\n", name);
 		}
@@ -1443,7 +1560,7 @@ build_lang_thread(gpointer data)
 				if (xmlStrEqual(name2, (xmlChar *) "context")) {
 					GQueue *contextstack = g_queue_new();
 					process_scanning_context(reader, bfparser, contextstack);
-					/* TODO: free contextstack, memory leak ???? */
+					g_queue_free(contextstack);
 				} else if (xmlStrEqual(name2, (xmlChar *) "definition")) {
 					xmlFree(name2);
 					break;
@@ -1463,12 +1580,12 @@ build_lang_thread(gpointer data)
 							{"end", &com.eo, attribtype_string},
 							{"type", &typestr, attribtype_string},
 							{"id", &id, attribtype_string}};
-					parse_attributes(reader, attribs, 4);
+					parse_attributes(bflang,reader, attribs, 4);
 					com.type = g_strcmp0(typestr, "block") == 0 ? comment_type_block : comment_type_line;
 					if (com.so && (com.type == comment_type_line || com.eo)) {
 						g_array_append_val(bfparser->st->comments, com);
 						if (id)
-							/* we insert one position higher than the actual index in the hashtable, because index 0 would 
+							/* we insert one position higher than the actual index in the hashtable, because index 0 would
 							otherwise produce pointer NULL in the hashtable. set_comment() should thus decrement by 1) */
 							g_hash_table_insert(bfparser->commentid_table, id,
 												GINT_TO_POINTER(bfparser->st->comments->len));
@@ -1480,15 +1597,15 @@ build_lang_thread(gpointer data)
 					xmlFree(typestr);
 				} else if (xmlStrEqual(name2, (xmlChar *) "smartindent")) {
 					Tattrib attribs[] = {{"characters", &bfparser->smartindentchars, attribtype_string}};
-					parse_attributes(reader, attribs, 1);
+					parse_attributes(bflang,reader, attribs, 1);
 				} else if (xmlStrEqual(name2, (xmlChar *) "smartoutdent")) {
 					Tattrib attribs[] = {{"characters", &bfparser->smartoutdentchars, attribtype_string}};
-					parse_attributes(reader, attribs, 1);
+					parse_attributes(bflang,reader, attribs, 1);
 #ifdef HAVE_LIBENCHANT
 				} else if (xmlStrEqual(name2, (xmlChar *) "default_spellcheck")) {
 					Tattrib attribs[] = {{"enabled", &bfparser->default_spellcheck, attribtype_boolean},
 							{"spell_decode_entities", &bfparser->spell_decode_entities, attribtype_boolean}};
-					parse_attributes(reader, attribs, 2);
+					parse_attributes(bflang,reader, attribs, 2);
 #endif
 				} else if (xmlStrEqual(name2, (xmlChar *) "properties")) {
 					xmlFree(name2);
@@ -1549,14 +1666,10 @@ langmgr_get_bflang(const gchar * mimetype, const gchar * filename)
 		return NULL;
 	DBG_MSG("langmgr_get_bflang, request for mimetype %s filename %s\n",mimetype, filename);
 	if (filename && strchr(mimetype, '?') == NULL) {
-		gchar *key, *tmp;
-		tmp = strrchr(filename, '.');
-		if (tmp) {
-			key = g_strconcat(mimetype, "?", tmp + 1, NULL);
-/*			g_print("langmgr_get_bflang, search for key %s\n",key);*/
-			bflang = g_hash_table_lookup(langmgr.bflang_lookup, key);
-			g_free(key);
-		}
+		gchar *key = mime_with_extension(mimetype, filename);
+/*		g_print("search for key=%s\n",key);*/
+		bflang = g_hash_table_lookup(langmgr.bflang_lookup, key);
+		g_free(key);
 	}
 	if (!bflang) {
 		bflang = g_hash_table_lookup(langmgr.bflang_lookup, mimetype);
@@ -1610,7 +1723,7 @@ parse_bflang2_header(const gchar * filename)
 					{"table", &bflang->size_table, attribtype_int},
 					{"matches", &bflang->size_matches, attribtype_int},
 					{"contexts", &bflang->size_contexts, attribtype_int}};
-			parse_attributes(reader, attribs, 5);
+			parse_attributes(bflang,reader, attribs, 5);
 		} else if (xmlStrEqual(name, (xmlChar *) "header")) {
 			process_header(reader, bflang);
 			xmlFree(name);
@@ -1619,7 +1732,7 @@ parse_bflang2_header(const gchar * filename)
 		xmlFree(name);
 	}
 	xmlFreeTextReader(reader);
-	
+
 	if (bflang->name == NULL) {
 		g_print("Language file %s has no name.. abort..\n", filename);
 		failed=TRUE;
@@ -1627,7 +1740,7 @@ parse_bflang2_header(const gchar * filename)
 	if (g_strcmp0(bflangversion, CURRENT_BFLANG2_VERSION)!=0) {
 		g_print("Language file %s has incompatible version %s (need version "CURRENT_BFLANG2_VERSION"), abort..\n", filename, bflangversion);
 		failed=TRUE;
-	} 
+	}
 	g_free(bflangversion);
 	if (failed) {
 		g_free(bflang->name);
@@ -1678,7 +1791,9 @@ langmgr_need_spellcheck_tags(void)
 GList *
 langmgr_get_languages(void)
 {
-	return g_list_copy(langmgr.bflang_list);
+	if (langmgr.done_scanning)
+		return g_list_copy(langmgr.bflang_list);
+	return NULL;
 }
 
 static gint
@@ -1758,7 +1873,7 @@ langmgr_cleanup(void)
 		tmplist = g_list_next(tmplist);
 	}
 	g_list_free(langmgr.bflang_list);
-	
+
 	g_object_unref(G_OBJECT(langmgr.tagtable));
 	g_hash_table_destroy(langmgr.configured_styles);
 	g_hash_table_destroy(langmgr.bflang_options);
@@ -1772,13 +1887,30 @@ langmgr_cleanup(void)
 }
 #endif
 
+void
+langmgr_add_custom_mime(Tbflang *bflang, const gchar *mime)
+{
+	main_v->globses.custombflangmime = g_list_prepend(main_v->globses.custombflangmime, array_from_arglist(bflang->name,mime,NULL));
+	g_hash_table_insert(langmgr.bflang_lookup, g_strdup(mime), bflang);
+}
+
 static void
 register_bflanguage(Tbflang * bflang)
 {
 	if (bflang) {
 		GList *tmplist;
 		gboolean registered = FALSE;
-		DBG_PARSING("register_bflanguage, register bflang %p %s\n",bflang,bflang->name); 
+		DBG_PARSING("register_bflanguage, register bflang %p %s\n",bflang,bflang->name);
+
+		/* see if the user has additional mime types */
+		for (tmplist = g_list_last(main_v->globses.custombflangmime); tmplist; tmplist = tmplist->prev) {
+			gchar **arr = (gchar **) tmplist->data;
+			if (g_strv_length(arr) == 2 && strcmp(arr[0],bflang->name)==0) {
+				DEBUG_MSG("adding additional user mime %s to %s\n",arr[1], arr[0]);
+				bflang->mimetypes = g_list_prepend(bflang->mimetypes, g_strdup(arr[1]));
+			}
+		}
+
 		tmplist = g_list_first(bflang->mimetypes);
 		while (tmplist) {
 			if (!g_hash_table_lookup(langmgr.bflang_lookup, (gchar *) tmplist->data)) {
@@ -1823,17 +1955,19 @@ static gboolean
 bflang2scan_finished_lcb(gpointer data)
 {
 	GList *tmplist;
+	DEBUG_SIG("bflang2scan_finished_lcb, priority=%d\n",BFLANGSCAN_FINISHED_PRIORITY);
+	langmgr.done_scanning = TRUE;
 	/* now add the languages once the GUI if the GUI has been loaded */
 	DBG_MSG("bflang2scan_finished_lcb\n");
 	for (tmplist = g_list_first(main_v->bfwinlist); tmplist; tmplist = g_list_next(tmplist)) {
 		GList *tmplist2;
-		DBG_MSG("bflang2scan_finished, create menu in bfwin %p\n", tmplist->data);
+		DBG_MSG("bflang2scan_finished, call lang_mode_menu_create() for bfwin %p\n", tmplist->data);
 		lang_mode_menu_create(BFWIN(tmplist->data));
 		for (tmplist2 = g_list_first(BFWIN(tmplist->data)->documentlist); tmplist2; tmplist2 = g_list_next(tmplist2)) {
 			DBG_MSG("bflang2scan_finished, request bflang for document %p\n", tmplist2->data);
 			if (DOCUMENT(tmplist2->data)->info || DOCUMENT(tmplist2->data)->uri) {
-				bluefish_text_view_select_language(BLUEFISH_TEXT_VIEW(DOCUMENT(tmplist2->data)->view), 
-						DOCUMENT(tmplist2->data)->info ? g_file_info_get_content_type(DOCUMENT(tmplist2->data)->info) : NULL, 
+				bluefish_text_view_select_language(BLUEFISH_TEXT_VIEW(DOCUMENT(tmplist2->data)->view),
+						DOCUMENT(tmplist2->data)->info ? g_file_info_get_content_type(DOCUMENT(tmplist2->data)->info) : NULL,
 						DOCUMENT(tmplist2->data)->uri ? g_file_get_path(DOCUMENT(tmplist2->data)->uri): NULL);
 			}
 		}
