@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bftextview2_patcompile.c
  *
- * Copyright (C) 2008,2009,2011 Olivier Sessink
+ * Copyright (C) 2008,2009,2011,2013 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,6 +54,10 @@ the character list [a-z]
 
 */
 
+#ifdef ENABLE_PRINT_DFA
+void print_DFA_subset(Tscantable * st, gint16 context, char *chars);
+#endif
+
 static gint
 pointersort_compare(gconstpointer a, gconstpointer b)
 {
@@ -199,24 +203,24 @@ create_state_tables(Tscantable * st, gint16 context, gchar * characters, gboolea
 	gint newstate = -1;			/* if all characters can follow existing states we don't need any newstate
 								   and thus newstate==-1 but if one or more characters in one or more states need a new state
 								   it will be >0 */
-	DBG_PATCOMPILE("create_state_tables started with get_length(positions)=%d states, pointtoself=%d\n",
-				   g_queue_get_length(positions), pointtoself);
+	DBG_PATCOMPILE("create_state_tables, context=%d, started with get_length(positions)=%d states, pointtoself=%d, end_is_symbol=%d\n",
+				   (gint)context,g_queue_get_length(positions), pointtoself,end_is_symbol);
 	while (g_queue_get_length(positions)) {
 		pos = GPOINTER_TO_INT(g_queue_pop_head(positions));
 		/*DBG_PATCOMPILE("working on position %d, identstate=%d\n", pos, identstate);*/
 		for (c = 0; c < NUMSCANCHARS; c++) {
 			if (characters[c] == 1) {
-				DBG_PATCOMPILE("running for position %d char %c\n",pos,c);
+				DBG_PATCOMPILE("running for position %d char %c (=%d)\n",pos,c,c);
 				/* there is a bug in the pattern compiler that may compile a buggy language file without a warning:
 				if a new keyword is added, and all characters already exist there are no new states created: this function
 				will simply follow the existing states. BUT IT DOES NOT CHECK IF THERE ARE MULTIPLE WAYS TO GET TO THESE
 				EXISTING STATES. So if those states are created by a regex pattern that allows jumping to these states via
 				various other characters, you will have an overlapping pattern WITHOUT WARNING.
-				
-				You can avoid this bug by having regex patterns alwats in the end of a context.  
+
+				You can avoid this bug by having regex patterns alwats in the end of a context.
 				 */
-				
-				
+
+
 				if (get_tablerow(st, context, pos).row[c] != 0
 					&& get_tablerow(st, context, pos).row[c] != identstate) {
 					if (get_tablerow(st, context, pos).row[c] == pos) {
@@ -429,10 +433,10 @@ inputpositions, and returns all valid outputstates.
 regexpart_ends_regex is a boolean that is used to set the state for the last character of the regex.
 if the last character of a pattern is a symbol, the states are different. because this function is
 called recursively this is only relevant for the first call.
-BUG: this is not true, for a pattern like <(a|b) the a and b are both end-patterns, but they are handled in a 
-subpattern  
+BUG: this is not true, for a pattern like <(a|b) the a and b are both end-patterns, but they are handled in a
+subpattern
+
 
- 
 */
 static GQueue *
 process_regex_part(Tscantable * st, gchar * regexpart, gint16 context, gboolean caseinsensitive,
@@ -506,14 +510,14 @@ process_regex_part(Tscantable * st, gchar * regexpart, gint16 context, gboolean
 						characters[j - 32] = characters[j];
 					}
 				}
-				/* BUG: the following code misses a lot of cases where the next state could be a possible 
+				/* BUG: the following code misses a lot of cases where the next state could be a possible
 				   end-state already. For example <%[=@]? does not work */
 				if (regexpart_ends_regex && regexpart[i] != '\0' && regexpart[i + 1] == '\0') {
 					gboolean only_symbols = TRUE;
 					/* check if the last character of the regex is a symbol, if so the last state should not
 					   refer to the identstate for all non-symbols */
 					gint j;
-					for (j = 0; j <= NUMSCANCHARS; j++) {
+					for (j = 0; j < NUMSCANCHARS; j++) {
 						if (characters[j] == 1
 							&& !character_is_symbol(st, context, j)) {
 							only_symbols = FALSE;
@@ -605,12 +609,18 @@ compile_limitedregex_to_DFA(Tscantable * st, gchar * input, gboolean caseinsensi
 	g_queue_free(positions);
 	g_queue_free(newpositions);
 	g_free(lregex);
+
+#ifdef ENABLE_PRINT_DFA
+	if (context == 3) {
+		print_DFA_subset(st, context, "#a \n\r");
+	}
+#endif
 }
 
 /* this function cannot do any regex style patterns
 just full keywords */
 static void
-compile_keyword_to_DFA(Tscantable * st, gchar * keyword, guint16 matchnum, gint16 context,
+compile_keyword_to_DFA(Tscantable * st, const gchar * keyword, guint16 matchnum, gint16 context,
 					   gboolean case_insens)
 {
 	gint i, len;
@@ -633,7 +643,7 @@ compile_keyword_to_DFA(Tscantable * st, gchar * keyword, guint16 matchnum, gint1
 		pattern = g_strdup(keyword);
 	}
 
-	DBG_PATCOMPILE("in context %d we start with position %d\n", context, (gint) g_queue_peek_head(positions));
+	DBG_PATCOMPILE("in context %d we start with position %d\n", (gint)context, GPOINTER_TO_INT(g_queue_peek_head(positions)));
 	len = strlen(pattern);
 
 	end_is_symbol =
@@ -677,21 +687,22 @@ compile_keyword_to_DFA(Tscantable * st, gchar * keyword, guint16 matchnum, gint1
 }
 
 gint16
-new_context(Tscantable * st, guint expected_size, const gchar * lang, gchar * symbols, const gchar * contexthighlight,
-			gboolean autocomplete_case_insens)
+new_context(Tscantable * st, guint expected_size, const gchar * lang, const gchar * symbols, const gchar * contexthighlight,
+			gboolean autocomplete_case_insens, gboolean default_spellcheck)
 {
 	gint16 context;
 	gint i;
-	gchar *tmp;
+	const gchar *tmp;
 	GArray *tmptable;
 
 	context = st->contexts->len;
 	g_array_set_size(st->contexts, st->contexts->len + 1);
 
 	g_array_index(st->contexts, Tcontext, context).autocomplete_case_insens = autocomplete_case_insens;
+	g_array_index(st->contexts, Tcontext, context).default_spellcheck = default_spellcheck;
 	g_array_index(st->contexts, Tcontext, context).contexthighlight = (gchar *) contexthighlight;
 	tmptable = g_array_index(st->contexts, Tcontext, context).table = g_array_sized_new(TRUE, TRUE, sizeof(Ttablerow), expected_size);
-	g_array_set_size(g_array_index(st->contexts, Tcontext, context).table, 2); /* first two states are the startstate (0) and the identstate (1) */ 
+	g_array_set_size(g_array_index(st->contexts, Tcontext, context).table, 2); /* first two states are the startstate (0) and the identstate (1) */
 	/* identstate refers to itself for all characters except the symbols. we cannot use memset
 	   because an guint16 occupies 2 bytes */
 	for (i = 0; i < NUMSCANCHARS; i++)
@@ -842,22 +853,22 @@ compile_existing_match(Tscantable * st, guint16 matchnum, gint16 context)
 }
 
 void
-pattern_set_blockmatch(Tscantable * st, guint16 matchnum, 
+pattern_set_blockmatch(Tscantable * st, guint16 matchnum,
 							gboolean starts_block,
-							gboolean ends_block, 
+							gboolean ends_block,
 							guint blockstartpattern,
 							const gchar *blockhighlight,
 							const gchar *blockname,
-							gboolean foldable) 
+							gboolean foldable)
 {
 	if (starts_block == TRUE && ends_block == TRUE) {
-		g_warning("Error in language file or Bluefish bug: pattern %s both starts and ends a block\n", 
+		g_warning("Error in language file or Bluefish bug: pattern %s both starts and ends a block\n",
 						g_array_index(st->matches, Tpattern, matchnum).pattern);
 	}
 	if (blockname && !starts_block) {
 		g_warning("Error in language file or Bluefish bug: block_name %s can only be set on a block start\n", blockname);
 	}
-	
+
 	if (starts_block) {
 		guint16 blocknum = 0;
 		if (blockname || blockhighlight) { /* only create a block if we need it */
@@ -877,7 +888,7 @@ pattern_set_blockmatch(Tscantable * st, guint16 matchnum,
 }
 
 void
-pattern_set_runtime_properties(Tscantable * st, guint16 matchnum, 
+pattern_set_runtime_properties(Tscantable * st, guint16 matchnum,
 								const gchar * selfhighlight,
 								gint16 nextcontext,
 								gboolean tagclose_from_blockstack,
@@ -897,7 +908,7 @@ pattern_set_runtime_properties(Tscantable * st, guint16 matchnum,
 }
 
 guint16
-add_pattern_to_scanning_table(Tscantable * st, gchar * pattern,
+add_pattern_to_scanning_table(Tscantable * st, const gchar * pattern,
 								gboolean is_regex,
 								gboolean case_insens,
 								gint16 context)
@@ -908,28 +919,31 @@ add_pattern_to_scanning_table(Tscantable * st, gchar * pattern,
 		return 0;
 	}
 	matchnum = st->matches->len;
+	if (matchnum == G_MAXUINT16) {
+		g_warning("Language file has too many patterns, this will very likely result in a crash!\n");
+	}
 	g_array_set_size(st->matches, st->matches->len + 1);
 	g_array_index(st->matches, Tpattern, matchnum).pattern = g_strdup(pattern);
 	g_array_index(st->matches, Tpattern, matchnum).case_insens = case_insens;
 	g_array_index(st->matches, Tpattern, matchnum).is_regex = is_regex;
 	DBG_PATCOMPILE("add_pattern_to_scanning_table,pattern=%s for context=%d got matchnum %d\n",pattern, context, matchnum);
 	if (is_regex) {
-		compile_limitedregex_to_DFA(st, pattern, case_insens, matchnum, context);
+		compile_limitedregex_to_DFA(st, g_array_index(st->matches, Tpattern, matchnum).pattern, case_insens, matchnum, context);
 	} else {
-		compile_keyword_to_DFA(st, pattern, matchnum, context, case_insens);
+		compile_keyword_to_DFA(st, g_array_index(st->matches, Tpattern, matchnum).pattern, matchnum, context, case_insens);
 	}
 	/*if (g_strcmp0(pattern, "rem ")==0 || g_strcmp0(pattern, "\\.?[a-zA-Z][a-zA-Z_0-9]*[\\$%]?")==0) {
 		print_DFA_subset(st, context, "rem var");
 	}*/
 	return matchnum;
-} 
+}
 
 
 /*guint16
 add_keyword_to_scanning_table(Tscantable * st, gchar * pattern, const gchar * lang,
 							  const gchar * selfhighlight, const gchar * blockhighlight, gboolean is_regex,
 							  gboolean case_insens, gint16 context, gint16 nextcontext, gboolean starts_block,
-							  gboolean ends_block, guint blockstartpattern, 
+							  gboolean ends_block, guint blockstartpattern,
 							  gboolean tagclose_from_blockstack, gboolean stretch_blockstart,
 							  guint8 identmode, gboolean identjump, gboolean identautocomp)
 {
@@ -939,7 +953,7 @@ add_keyword_to_scanning_table(Tscantable * st, gchar * pattern, const gchar * la
 		g_print("CORRUPT LANGUAGE FILE: empty pattern/tag/keyword\n");
 		return 0;
 	}
-	
+
 	if (context == nextcontext)
 		DBG_PATCOMPILE("context=nextcontext=%d for %s\n", context, pattern);
 
@@ -957,7 +971,7 @@ add_keyword_to_scanning_table(Tscantable * st, gchar * pattern, const gchar * la
 	g_array_index(st->matches, Tpattern, matchnum).case_insens = case_insens;
 	g_array_index(st->matches, Tpattern, matchnum).is_regex = is_regex;
 	g_array_index(st->matches, Tpattern, matchnum).selfhighlight = (gchar *) selfhighlight;
-	
+
 	g_array_index(st->matches, Tpattern, matchnum).tagclose_from_blockstack = tagclose_from_blockstack;
 	g_array_index(st->matches, Tpattern, matchnum).stretch_blockstart = stretch_blockstart;
 #ifdef IDENTSTORING
@@ -985,9 +999,16 @@ print_DFA_subset(Tscantable * st, gint16 context, char *chars)
 {
 	gint i, j, len;
 	len = strlen(chars);
-	g_print("     ");
+	g_print("***************** print subset of DFA table for context %d\n",(gint)context);
+	g_print("       ");
 	for (j = 0; j <= len; j++) {
-		g_print("  %c  ", chars[j]);
+		if (chars[j] == '\n') {
+			g_print(" \\n   ");
+		} else if (chars[j] == '\r') {
+			g_print(" \\r   ");
+		} else {
+			g_print(" '%c' ", chars[j]);
+		}
 	}
 	g_print(": match\n");
 	for (i = 0; i < g_array_index(st->contexts, Tcontext, context).table->len; i++) {
@@ -1015,7 +1036,7 @@ print_DFA_subset(Tscantable * st, gint16 context, char *chars)
 		}
 		g_print("\n");
 	}
-
+	g_print("*****************\n");
 }
 
 void
@@ -1032,7 +1053,7 @@ print_DFA(Tscantable * st, gint16 context, char start, char end)
 		for (j = start; j <= end; j++) {
 			g_print("%3d ", g_array_index(g_array_index(st->contexts, Tcontext, context).table, Ttablerow, i).row[j]);
 		}
-		g_print(": %3d (%s)\n", g_array_index(g_array_index(st->contexts, Tcontext, context).table, Ttablerow, i).match, 
+		g_print(": %3d (%s)\n", g_array_index(g_array_index(st->contexts, Tcontext, context).table, Ttablerow, i).match,
 					g_array_index(st->matches, Tpattern,g_array_index(g_array_index(st->contexts, Tcontext, context).table, Ttablerow, i).match).pattern);
 	}
 
