@@ -32,9 +32,11 @@ extern void g_none(char *first, ...);
 
 #define CURRENT_BFLANG2_VERSION "2.0"
 
-#define BF2_OFFSET_UNDEFINED G_MAXUINT32
+#define BF_OFFSET_UNDEFINED G_MAXINT32
+#define BF_POSITION_UNDEFINED G_MAXINT32
 
 /*#define DUMP_SCANCACHE*/
+/*#define DUMP_SCANCACHE_UPDATE_OFFSET*/
 /*#define CHECK_CONSISTENCY*/
 
 #define DBG_MSG DBG_NONE
@@ -51,6 +53,17 @@ extern void g_none(char *first, ...);
 #define DBG_TOOLTIP DBG_NONE
 #define DBG_SPELL DBG_NONE
 #define DBG_IDENTIFIER DBG_NONE
+#define DBG_MARKREGION DBG_NONE
+
+#ifdef UPDATE_OFFSET_DELAYED
+#include "bf_lib.h"
+typedef struct {
+	BF_ELIST_HEAD;
+	guint32 startpos;
+	gint32 offset;
+} Toffsetupdate;
+#define OFFSETUPDATE(var) ((Toffsetupdate *)var)
+#endif /*UPDATE_OFFSET_DELAYED*/
 
 #define NUMSCANCHARS 127		/* 128 is ascii, but the last character is never scanned (DEL)
 								   and the Ttablerow has one more 16bit value. By setting this to 127 instead of 128
@@ -61,30 +74,27 @@ extern void g_none(char *first, ...);
 /*****************************************************************/
 #define COMMENT_INDEX_INHERIT 255
 #define COMMENT_INDEX_NONE 254
-	typedef struct {
-	GArray *table; /* a pointer to the DFA table for this context */ 
+#define SPELLCHECK_INHERIT 255
+#define SPELLCHECK_ENABLED 1
+#define SPELLCHECK_DISABLED 0
+typedef struct {
+	GArray *table; /* a pointer to the DFA table for this context */
 	GCompletion *ac;			/* autocompletion items in this context */
-	/*GHashTable *reference; *//* reference help for each autocompletion item */
 	GHashTable *patternhash;	/* a hash table where the pattern and its autocompletion string are the keys, and an integer to the ID of the pattern is the value */
 	GtkTextTag *contexttag;		/* if the context area itself needs some kind of style (to implement a string context for example) */
 	gchar *contexthighlight;	/* the string that has the id for the highlight */
-	/*guint16 startstate;*/			/* refers to the row number in scantable->table that is the start state for this context */
-	/*guint16 identstate;*/			/* refers to the row number in scantable->table that is the identifier-state
-								   for this context. The identifier state is a state that refers to itself for all characters
-								   except the characters (symbols) thay may be the begin or end of an identifier such
-								   as whitespace, ();[]{}*+-/ etc. */
 	guint8 has_tagclose_from_blockstack;	/* this context has xml end patterns that need autoclosing for generix xml tags, based on the tag that is on top of the blockstack */
-	guint8 comment_block;		/* block comment index in array scantable->comments 
-								   or COMMENT_INDEX_INHERIT (which means inherit) 
+	guint8 comment_block;		/* block comment index in array scantable->comments
+								   or COMMENT_INDEX_INHERIT (which means inherit)
 								   or COMMENT_INDEX_NONE if there is no block comment  */
 	guint8 comment_line;		/* index in array scantable->comments for line comments; see comment_block */
 	guint8 autocomplete_case_insens;
+	guint8 default_spellcheck;
 } Tcontext;
 
 typedef struct {
 	gchar *autocomplete_string;
 	guint8 autocomplete_backup_cursor;	/* number of characters to backup the cursor after autocompletion (max 256) */
-	/* guint8 autocomplete;whether or not this pattern should be added to the autocompletion; stored in the Tpattern so we can re-use it in another context */
 } Tpattern_autocomplete;
 
 typedef struct {
@@ -97,16 +107,15 @@ typedef struct {
 
 typedef struct {
 	GtkTextTag *selftag;		/* the tag used to highlight this pattern */
-	/*GtkTextTag *blocktag;*/		/* if this pattern ends a context or a block, we can highlight
-								   the region within the start and end pattern with this tag */
 	gchar *reference;			/* the reference data, or NULL. may be inserted in hash tables for multiple keys in multiple contexts */
 	gchar *pattern;				/* the pattern itself. stored in the Tpattern so we can re-use it in another context */
-	GSList *autocomp_items;
-	gchar *selfhighlight;		/* a string with the highlight for this pattern. used when re-linking highlights and textstyles 
+	GSList *autocomp_items; /* a list of Tpattern_autocomplete - a pattern can autocomplete in multiple ways, for 
+										example with and without closing tag, or with and without function arguments.
+										to be able to recompile a pattern in multiple contexts we need this information in Tpattern */
+	gchar *selfhighlight;		/* a string with the highlight for this pattern. used when re-linking highlights and textstyles
 								   if the user changed any of these in the preferences */
-	/*gchar *blockhighlight;*/		/* a string for the highlight corresponding to the  blocktag */
-	guint16 block; 
-	gint16 blockstartpattern;	/* the number of the pattern that may start this block, or -1 to end the last started block */
+	guint16 block;			/* this is 0 for most blocks, only blocks that need a tag have this set, refers to a a position in an array of Tpattern_block*/
+	gint16 blockstartpattern;	/* the number of the pattern that may start this block, or -1 to end the last started block, also used for stretch block */
 	gint16 nextcontext;			/* 0, or if this pattern starts a new context the number of the context, or -1 or -2 etc.
 								   to pop a context of the stack */
 #ifdef IDENTSTORING
@@ -123,9 +132,9 @@ typedef struct {
 	guint8 identmode :1;
 #endif							/* IDENTSTORING */
 } Tpattern;
-/* 
-32bit size = 5*32 + 3*16 + 1*2 + 8*1 = 217 + 7 padding = 28 bytes
-64bit size = 5*64 + 3*16 + 1*2 + 8*1 = 361 + 7 padding = 48 bytes
+/*
+32bit size = 5*32 + 3*16 + 1*2 + 8*1 = 217 + 7 padding bits = 28 bytes
+64bit size = 5*64 + 3*16 + 1*2 + 8*1 = 361 + 7 padding bits = 48 bytes
 */
 
 typedef struct {
@@ -137,7 +146,8 @@ typedef struct {
 } Ttablerow;					/* a row in the DFA, right now exactly 256 bytes */
 
 
-#define character_is_symbol(st,context,c) (g_array_index((GArray *)g_array_index(st->contexts, Tcontext, context).table, Ttablerow, 1).row[c] != 1)
+/*#define character_is_symbol(st,context,c) (g_array_index((GArray *)g_array_index(st->contexts, Tcontext, context).table, Ttablerow, 1).row[c] != 1)*/
+gboolean character_is_symbol(Tscantable *st,guint16 context, gunichar uc);
 
 #define get_table(scantable, context) ((GArray *)g_array_index(scantable->contexts, Tcontext, context).table)
 
@@ -152,7 +162,7 @@ typedef struct {
 	guint32 end1_o;
 	guint32 start2_o;
 	guint32 end2_o;
-	gint16 patternum;			/* which pattern (number of the array element in scantable->matches) */
+	gint16 patternum;			/* which pattern (number of the array element in scantable->matches) started the block */
 	guint8 folded;
 	guint8 foldable;			/* FALSE on a single line */
 } Tfoundblock;					/* Once a start-of-block is found start1 and end1 are set
@@ -187,11 +197,11 @@ typedef struct {
 typedef struct {
 	Tfoundcontext *fcontext;	/* if numcontextchange == 0 this points to the current active context
 								   if numcontextchange > 0 this points to the pushed context, which also happens to be the current context
-								   if numcontextchange < 0 this points to the top of the stack at this position, to get the current position 
+								   if numcontextchange < 0 this points to the top of the stack at this position, to get the current position
 								   you'll have to pop N items (where N is -1 * numcontextchange). */
 	Tfoundblock *fblock;		/* if numblockchange == 0 this points to the current active block
 								   if numblockchange > 0 this points to the pushed block, which also happens to be the current block
-								   if numblockchange < 0 this points to the top of the stack at this position, to get the current position 
+								   if numblockchange < 0 this points to the top of the stack at this position, to get the current position
 								   you'll have to pop N items (where N is -1 * numblockchange). */
 	guint32 charoffset_o;
 	gint16 numblockchange;		/* there are files that have > 127 pops in a single position
