@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bftextview2_spell.c
  *
- * Copyright (C) 2009-2011 Olivier Sessink
+ * Copyright (C) 2009-2013 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,9 @@
 #include "bftextview2_scanner.h"
 #include "bftextview2_langmgr.h"
 #include "bftextview2_spell.h"
+#ifdef MARKREGION
+#include "bftextview2_markregion.h"
+#endif
 #include "document.h"
 #include "xml_entity.h"
 
@@ -44,14 +47,28 @@
 static EnchantBroker *eb;
 static guint loops_per_timer = 1000;
 
+#ifdef MARKREGION
 static gboolean
-bftextview2_find_region2spellcheck(BluefishTextView * btv, GtkTextBuffer * buffer, GtkTextIter * start,
-								   GtkTextIter * end)
+markregion_find_region2spellcheck(BluefishTextView * btv, GtkTextIter * sit, GtkTextIter * eit)
 {
-	GtkTextTag *misspelled;
+	guint start,end;
+	markregion_get_region(&btv->spellcheck, NULL, &start, &end);
+	if (start == BF_OFFSET_UNDEFINED) {
+		return FALSE;
+	}
+	gtk_text_buffer_get_iter_at_offset(btv->buffer, sit, start);
+	gtk_text_buffer_get_iter_at_offset(btv->buffer, eit, end);
+	DBG_MARKREGION("markregion_find_region2spellcheck, tried iters at %u:%u, got iters at %u:%u\n",start,end,gtk_text_iter_get_offset(sit),gtk_text_iter_get_offset(eit));
+	return TRUE;
+}
+#endif
 
+#ifdef NEEDSCANNING
+static gboolean
+needscanning_find_region2spellcheck(BluefishTextView * btv, GtkTextIter * start, GtkTextIter * end)
+{
 	/* first find a region that needs a spellcheck */
-	gtk_text_buffer_get_start_iter(buffer, start);
+	gtk_text_buffer_get_start_iter(btv->buffer, start);
 	if (!gtk_text_iter_begins_tag(start, btv->needspellcheck)) {
 		DBG_SPELL("iter %d does not begins tag needspellcheck %p, needscanning(%p)=%d\n",
 				  gtk_text_iter_get_offset(start), btv->needspellcheck, btv->needscanning,
@@ -71,8 +88,47 @@ bftextview2_find_region2spellcheck(BluefishTextView * btv, GtkTextBuffer * buffe
 			return FALSE;
 		}
 	}
+	DBG_MARKREGION("needscanning_find_region2spellcheck, return iters at %d:%d\n",gtk_text_iter_get_offset(start),gtk_text_iter_get_offset(end));
+	return TRUE;
+}
+#endif
+
+static gboolean
+bftextview2_find_region2spellcheck(BluefishTextView * btv, GtkTextIter * start,GtkTextIter * end)
+{
+	GtkTextTag *misspelled;
+	gboolean ret;
+#ifdef MARKREGION
+	ret = markregion_find_region2spellcheck(btv, start, end);
+#ifdef NEEDSCANNING
+	gboolean mrret = ret;
+	GtkTextIter mrits=*start, mrite=*end;
+#endif
+#endif
+#ifdef NEEDSCANNING
+	ret = needscanning_find_region2spellcheck(btv, start, end);
+#endif
+
+#ifdef MARKREGION
+#ifdef NEEDSCANNING
+	if (mrret != ret) {
+		g_print("ABORT: find_region2spellcheck, markregion returned %d, needscanning returned %d\n",mrret,ret);
+		g_assert_not_reached();
+	}
+
+	if (ret && (!gtk_text_iter_equal(&mrits, start) || !gtk_text_iter_equal(&mrite, end))) {
+		g_print("ABORT: find_region2spellcheck, markregion (%d:%d) and needscanning code(%d:%d) have different regions!!\n",
+				gtk_text_iter_get_offset(&mrits),gtk_text_iter_get_offset(&mrite),
+				gtk_text_iter_get_offset(start),gtk_text_iter_get_offset(end));
+		g_assert_not_reached();
+	}
+#endif
+#endif
+	if (!ret) {
+		return FALSE;
+	}
 	/* if the region is within a misspelled word, enlarge it to the total misspelled word
-	   (this fixes the situation where you add a space in the middle of "forgotthespace" and only 
+	   (this fixes the situation where you add a space in the middle of "forgotthespace" and only
 	   the space is scanned again) */
 	misspelled = gtk_text_tag_table_lookup(langmgr_get_tagtable(), "_spellerror_");
 	DBG_SPELL("find_region2spellcheck, start at %d has misspelled=%d, ends_mispelled=%d\n"
@@ -210,6 +266,45 @@ has_tags(GSList * tags, GtkTextTag ** tagarr)
 	return FALSE;
 }
 
+static gboolean
+get_spellcheck_from_context_at_position(BluefishTextView *btv, GtkTextIter *iter)
+{
+	Tfoundcontext *tmpfcontext=NULL;
+	Tfound *found;
+	gint changecounter;
+
+	if (!btv || !btv->bflang)
+		return TRUE;
+
+	found = get_foundcache_at_offset(btv, gtk_text_iter_get_offset(iter), NULL);
+
+	if (!found) {
+		return btv->bflang->default_spellcheck;
+	}
+
+	tmpfcontext = found->fcontext;
+	changecounter = found->numcontextchange;
+
+	while (changecounter < 0) {
+		tmpfcontext = (Tfoundcontext *) tmpfcontext->parentfcontext;
+		changecounter++;
+	}
+	DBG_SPELL("get_spellcheck_from_context_at_position, got context %d with default_spellcheck=%d\n"
+					,tmpfcontext ? tmpfcontext->context: 0
+					, tmpfcontext ?g_array_index(btv->bflang->st->contexts, Tcontext, tmpfcontext->context).default_spellcheck: -1);
+	while (tmpfcontext && g_array_index(btv->bflang->st->contexts, Tcontext, tmpfcontext->context).default_spellcheck == SPELLCHECK_INHERIT) {
+		tmpfcontext = (Tfoundcontext *) tmpfcontext->parentfcontext;
+		DBG_SPELL("get_spellcheck_from_context_at_position, get parent, got context %d with default_spellcheck=%d\n"
+					,tmpfcontext ? tmpfcontext->context: 0
+					, tmpfcontext ?g_array_index(btv->bflang->st->contexts, Tcontext, tmpfcontext->context).default_spellcheck: -1);
+	}
+	if (tmpfcontext) {
+		return g_array_index(btv->bflang->st->contexts, Tcontext, tmpfcontext->context).default_spellcheck;
+	}
+	DBG_SPELL("return bflang->default_spellcheck=%d\n",btv->bflang->default_spellcheck);
+	return btv->bflang->default_spellcheck;
+}
+
 static gboolean
 get_next_region(BluefishTextView * btv, GtkTextIter * so, GtkTextIter * eo)
 {
@@ -228,8 +323,10 @@ get_next_region(BluefishTextView * btv, GtkTextIter * so, GtkTextIter * eo)
 		} else if (has_tags(tmpslist, langmgr_no_spellcheck_tags())) {
 			/* do not scan */
 			DBG_SPELL("found tag not to scan, skip at %d\n", gtk_text_iter_get_offset(&iter));
-		} else {
+		} else if (btv->bflang && btv->bflang->st) {
 			/* scan depending on the settings of the language if it needs spell checking in default area's */
+			fso = get_spellcheck_from_context_at_position(btv, &iter);
+		} else {
 			if (btv->bflang->default_spellcheck) {
 				DBG_SPELL("no tags: scan at %d\n", gtk_text_iter_get_offset(&iter));
 				fso = TRUE;
@@ -357,16 +454,111 @@ text_iter_next_word_bounds(GtkTextIter * soword, GtkTextIter * eoword, gboolean
 	return TRUE;
 }
 
+#ifdef SPELL_PROFILING
+	static guint profile_words;
+#endif
+
+static gboolean
+spellcheck_region(BluefishTextView * btv, GTimer *timer, GtkTextIter *itcursor, GtkTextIter *so, GtkTextIter *eo)
+{
+	GtkTextIter iter;
+	gboolean cont=TRUE;
+	gint loop=0;
+
+#ifdef SPELL_PROFILING
+	gdouble time_at_start = g_timer_elapsed(timer, NULL);
+	guint words_at_start = profile_words;
+#endif
+
+	iter = *so;
+	DBG_SPELL("spellcheck_region, in bfwin=%p, bfwin->ed=%p loop1 from %d to %d\n",
+			  DOCUMENT(btv->doc)->bfwin, BFWIN(DOCUMENT(btv->doc)->bfwin)->ed, gtk_text_iter_get_offset(so),
+			  gtk_text_iter_get_offset(eo));
+
+	gtk_text_buffer_remove_tag_by_name(btv->buffer, "_spellerror_", so, eo);
+	/* we have two loops inside each other:
+		* loop1 loops over the entire region that was marked with 'needspellcheck'
+		   within this loop1 html tags and such are found, these are not to be scanned
+	   * loop2 loops over the elements, keywords, function names etc. that are found
+	     within this loop only the remaining text is scanned
+	*/
+	do {
+		GtkTextIter eo2 = *eo;
+		gboolean cont2 = TRUE;
+		if (btv->bflang->st) {
+			cont2 = get_next_region(btv, &iter, &eo2);
+			if (!cont2) {
+				iter = *eo;
+			}
+		} else {				/* no scantable */
+			eo2 = *eo;
+		}
+		DBG_SPELL("spellcheck_region, loop2 from %d to %d\n", gtk_text_iter_get_offset(&iter),
+				  gtk_text_iter_get_offset(&eo2));
+		while (cont2 && (loop % loops_per_timer != 0 || g_timer_elapsed(timer, NULL) < MAX_CONTINUOUS_SPELLCHECK_INTERVAL)) {	/* loop from iter to eo2 */
+			GtkTextIter wordstart = iter;
+			loop++;
+			DBG_SPELL("spellcheck_region, iter at %d, now forward to word end\n", gtk_text_iter_get_offset(&iter));
+			if (text_iter_next_word_bounds(&wordstart, &iter, btv->bflang->spell_decode_entities)
+				&& gtk_text_iter_compare(&iter, &eo2) <= 0) {
+				DBG_SPELL("spellcheck_region, iter at %d, backward wordstart at %d\n", gtk_text_iter_get_offset(&iter),
+						  gtk_text_iter_get_offset(&wordstart));
+
+				/* check word */
+#ifdef SPELL_PROFILING
+				profile_words++;
+#endif
+				spellcheck_word(btv, btv->buffer, &wordstart, &iter);
+			} else {
+				DBG_SPELL("spellcheck_region, no word end within region\n");
+				iter = eo2;
+				cont2 = FALSE;
+			}
+			DBG_SPELL("spellcheck_region, iter=%d, eo=%d, eo2=%d\n", gtk_text_iter_get_offset(&iter),
+					  gtk_text_iter_get_offset(eo), gtk_text_iter_get_offset(&eo2));
+			if (cont2 && gtk_text_iter_compare(&iter, &eo2) >= 0)
+				cont2 = FALSE;
+		}
+
+		if (gtk_text_iter_compare(&iter, eo) >= 0) {
+			DBG_SPELL
+				("spellcheck_region, iter (%d) equals eo, finished this area for spell checking\n",
+				 gtk_text_iter_get_offset(&iter));
+			cont = FALSE;
+		}
+
+	} while (cont
+			 && (loop % loops_per_timer != 0
+				 || g_timer_elapsed(timer, NULL) < MAX_CONTINUOUS_SPELLCHECK_INTERVAL));
+	DBG_SPELL("spellcheck_region, loop finished, cont=%d, timer_elapsed=%d ms\n",cont,(gint) (1000.0 * g_timer_elapsed(timer, NULL)));
+	if (cont) {
+		loops_per_timer = MAX(loop / 10, 100);
+	}
+#ifdef SPELL_PROFILING
+	g_print("%d ms spell run (loop=%d, loops_per_timer=%d) from %d to %d checked %d words\n",loop, loops_per_timer,
+			(gint) (1000.0 * (g_timer_elapsed(timer, NULL)-time_at_start)), gtk_text_iter_get_offset(so),
+			gtk_text_iter_get_offset(&iter), profile_words-words_at_start);
+#endif
+#ifdef NEEDSCANNING
+	DBG_SPELL("spellcheck_region, remove needspellcheck from start %d to iter at %d\n",
+			  gtk_text_iter_get_offset(so), gtk_text_iter_get_offset(&iter));
+	gtk_text_buffer_remove_tag(btv->buffer, btv->needspellcheck, so, &iter);
+#endif
+#ifdef MARKREGION
+	markregion_region_done(&btv->spellcheck, gtk_text_iter_get_offset(&iter));
+#endif
+	return (!gtk_text_iter_is_end(&iter));
+}
+
 gboolean
 bftextview2_run_spellcheck(BluefishTextView * btv)
 {
-	GtkTextIter so, eo, iter, itcursor;
-	GtkTextBuffer *buffer;
+	GtkTextIter so, eo, itcursor;
 	GTimer *timer;
-	gint loop = 0;
 	gboolean cont = TRUE;
+
 #ifdef SPELL_PROFILING
-	guint profile_words = 0;
+	profile_words = 0;
 #endif
 
 	if (!btv->spell_check)
@@ -377,108 +569,37 @@ bftextview2_run_spellcheck(BluefishTextView * btv)
 		return FALSE;
 	}
 
-	buffer = btv->buffer;
-	if (!bftextview2_find_region2spellcheck(btv, buffer, &so, &eo)) {
-		DBG_SPELL("bftextview2_run_spellcheck, no region to spellcheck found... return FALSE\n");
-		DBG_DELAYSCANNING("bftextview2_run_spellcheck, nothing to spellcheck..\n");
-		return FALSE;
-	}
 	timer = g_timer_new();
-	iter = so;
+
 	gtk_text_buffer_get_iter_at_mark(btv->buffer, &itcursor, gtk_text_buffer_get_insert(btv->buffer));
-	/* if we start at the cursor, that might be an indication that the previous word was 
+	/* if we start at the cursor, that might be an indication that the previous word was
 	skipped because it ended at the cursor, so lets skip back one word */
-	
-	
-	
-	DBG_SPELL("bftextview2_run_spellcheck, in bfwin=%p, bfwin->ed=%p loop1 from %d to %d\n",
-			  DOCUMENT(btv->doc)->bfwin, BFWIN(DOCUMENT(btv->doc)->bfwin)->ed, gtk_text_iter_get_offset(&so),
-			  gtk_text_iter_get_offset(&eo));
+
 	do {
-		GtkTextIter eo2 = eo;
-		gboolean cont2 = TRUE;
-		if (btv->bflang->st) {
-			cont2 = get_next_region(btv, &iter, &eo2);
-			if (!cont2) {
-				iter = eo;
-			}
-		} else {				/* no scantable */
-			eo2 = eo;
-		}
-		gtk_text_buffer_remove_tag_by_name(gtk_text_view_get_buffer(GTK_TEXT_VIEW(btv)),
-										   "_spellerror_", &iter, &eo2);
-		DBG_SPELL("bftextview2_run_spellcheck, loop2 from %d to %d\n", gtk_text_iter_get_offset(&iter),
-				  gtk_text_iter_get_offset(&eo2));
-		while (cont2 && (loop % loops_per_timer != 0 || g_timer_elapsed(timer, NULL) < MAX_CONTINUOUS_SPELLCHECK_INTERVAL)) {	/* loop from iter to eo2 */
-			GtkTextIter wordstart = iter;
-			loop++;
-			DBG_SPELL("iter at %d, now forward to word end\n", gtk_text_iter_get_offset(&iter));
-			if (text_iter_next_word_bounds(&wordstart, &iter, btv->bflang->spell_decode_entities)
-				&& gtk_text_iter_compare(&iter, &eo2) <= 0) {
-				DBG_SPELL("iter at %d, backward wordstart at %d\n", gtk_text_iter_get_offset(&iter),
-						  gtk_text_iter_get_offset(&wordstart));
-
-/*
-				if (gtk_text_iter_get_char(&iter) == '\'') {
-					DBG_SPELL("handle apostrophe, forward to next word end\n");
-					if (gtk_text_iter_forward_char(&iter)) {
-						if (g_unichar_isalpha(gtk_text_iter_get_char(&iter))) {
-							gtk_text_iter_forward_word_end(&iter);
-						}
-					}
-				} else if (gtk_text_iter_get_char(&iter) == '&' && forward_to_end_of_entity(&iter)) { / * handle entities gracefully * /
-					DBG_SPELL("handle entity, forward to next word end\n");
-					if (gtk_text_iter_forward_char(&iter)) {
-						if (g_unichar_isalpha(gtk_text_iter_get_char(&iter))) {
-							gtk_text_iter_forward_word_end(&iter);
-						}
-					}
-				}*/
-				/* check word */
+		if (!bftextview2_find_region2spellcheck(btv, &so, &eo)) {
+			DBG_SPELL("bftextview2_run_spellcheck, no region to spellcheck found... return FALSE\n");
+			DBG_DELAYSCANNING("bftextview2_run_spellcheck, nothing to spellcheck..\n");
+			g_timer_destroy(timer);
 #ifdef SPELL_PROFILING
-				profile_words++;
+			g_print("no more region to spellcheck, %d ms spell run\n",
+					(gint) (1000.0 * g_timer_elapsed(timer, NULL)));
 #endif
-/*				if (gtk_text_iter_equal(&itcursor, &iter)) {
-					g_print("skip spellcheck for %d:%d, cursor is at %d\n",gtk_text_iter_get_offset(&wordstart),
-						  gtk_text_iter_get_offset(&iter), gtk_text_iter_get_offset(&itcursor));
-				} else {*/
-					spellcheck_word(btv, buffer, &wordstart, &iter);
-/*				}*/
-			} else {
-				DBG_SPELL("bftextview2_run_spellcheck, no word end within region\n");
-				iter = eo2;
-				cont2 = FALSE;
-			}
-			DBG_SPELL("iter=%d, eo=%d, eo2=%d\n", gtk_text_iter_get_offset(&iter),
-					  gtk_text_iter_get_offset(&eo), gtk_text_iter_get_offset(&eo2));
-			if (cont2 && gtk_text_iter_compare(&iter, &eo2) >= 0)
-				cont2 = FALSE;
+			return FALSE;
 		}
+		DBG_SPELL("bftextview2_run_spellcheck, call spellcheck_region(%d:%d)\n",gtk_text_iter_get_offset(&so), gtk_text_iter_get_offset(&eo));
+		cont = spellcheck_region(btv, timer, &itcursor, &so, &eo);
 
-		if (gtk_text_iter_compare(&iter, &eo) >= 0) {
-			DBG_SPELL
-				("bftextview2_run_spellcheck, iter (%d) equals eo, finished this area for spell checking\n",
-				 gtk_text_iter_get_offset(&iter));
-			cont = FALSE;
-		}
 
-	} while (cont
-			 && (loop % loops_per_timer != 0
-				 || g_timer_elapsed(timer, NULL) < MAX_CONTINUOUS_SPELLCHECK_INTERVAL));
-	if (cont) {
-		loops_per_timer = MAX(loop / 10, 100);
-	}
+
+
+	} while(cont && g_timer_elapsed(timer, NULL) < MAX_CONTINUOUS_SPELLCHECK_INTERVAL);
 #ifdef SPELL_PROFILING
-	g_print("%d ms spell run from %d to %d checked %d words\n",
-			(gint) (1000.0 * g_timer_elapsed(timer, NULL)), gtk_text_iter_get_offset(&so),
-			gtk_text_iter_get_offset(&iter), profile_words);
+	g_print("%d ms spell run, checked %d words, not yet finished\n",
+			(gint) (1000.0 * g_timer_elapsed(timer, NULL)), profile_words);
 #endif
-	DBG_SPELL("bftextview2_run_spellcheck, remove needspellcheck from start %d to iter at %d\n",
-			  gtk_text_iter_get_offset(&so), gtk_text_iter_get_offset(&iter));
-	gtk_text_buffer_remove_tag(buffer, btv->needspellcheck, &so, &iter);
+
 	g_timer_destroy(timer);
-
-	return (!gtk_text_iter_is_end(&iter));
+	return cont;
 }
 
 void
@@ -501,9 +622,16 @@ bftextview2_spell_cleanup(void)
 static void
 recheck_document(Tdocument * doc)
 {
+#ifdef NEEDSCANNING
 	GtkTextIter start, end;
 	gtk_text_buffer_get_bounds(doc->buffer, &start, &end);
 	gtk_text_buffer_apply_tag(doc->buffer, BLUEFISH_TEXT_VIEW(doc->view)->needspellcheck, &start, &end);
+#endif
+#ifdef MARKREGION
+	GtkTextIter ite;
+	gtk_text_buffer_get_end_iter(BLUEFISH_TEXT_VIEW(doc->view)->buffer, &ite);
+	markregion_nochange(&BLUEFISH_TEXT_VIEW(doc->view)->spellcheck, 0, gtk_text_iter_get_offset(&ite));
+#endif
 }
 
 static void
@@ -604,10 +732,7 @@ mark_all_docs_needspelling(Tbfwin * bfwin)
 	GList *tmplist;
 	/* now mark all documents in this window with 'need_spellcheck' */
 	for (tmplist = g_list_first(bfwin->documentlist); tmplist; tmplist = g_list_next(tmplist)) {
-		Tdocument *doc = DOCUMENT(tmplist->data);
-		GtkTextIter start, end;
-		gtk_text_buffer_get_bounds(doc->buffer, &start, &end);
-		gtk_text_buffer_apply_tag(doc->buffer, BLUEFISH_TEXT_VIEW(doc->view)->needspellcheck, &start, &end);
+		recheck_document(DOCUMENT(tmplist->data));
 	}
 	if (bfwin->current_document)
 		bluefish_text_view_rescan(BLUEFISH_TEXT_VIEW(bfwin->current_document->view));
@@ -652,7 +777,7 @@ list_dicts_lcb(const char *const lang_tag, const char *const provider_name, cons
 {
 	Tdictlist *dl = data;
 	GtkWidget *menuitem;
-	DBG_SPELL("lang_tag=%s, provider_name=%s, provider_desc=%s, provider_file=%s\n",lang_tag,provider_name,provider_desc,provider_file); 
+	DBG_SPELL("lang_tag=%s, provider_name=%s, provider_desc=%s, provider_file=%s\n",lang_tag,provider_name,provider_desc,provider_file);
 	menuitem = gtk_radio_menu_item_new_with_label(dl->group, lang_tag);
 	if (!dl->group)
 		dl->group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menuitem));
@@ -787,7 +912,7 @@ static void bftextview2_preferences_menu_enable_lcb(GtkWidget *widget, gpointer
 void bftextview2_populate_preferences_popup(GtkMenu *menu, Tdocument *doc) {
 	GtkWidget *menuitem, *submenu;
 	Tdictlist dl;
-	
+
 
 	menuitem = gtk_check_menu_item_new_with_label(_("Enable spell check"));
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), BFWIN(doc->bfwin)->session->spell_enable);
