@@ -2,7 +2,7 @@
  * Bluefish HTML Editor
  * bfwin.c
  *
- * Copyright (C) 2002-2012 Olivier Sessink
+ * Copyright (C) 2002-2013 Olivier Sessink
  * Copyright (C) 2011 James Hayward
  *
  * This program is free software; you can redistribute it and/or modify
@@ -61,7 +61,6 @@ enum {
 	TARGET_STRING
 } Tdnd_types;
 
-
 void
 bfwin_fullscreen_toggle(Tbfwin * bfwin, gboolean active)
 {
@@ -194,6 +193,11 @@ side_panel_notify_position(GObject * object, GParamSpec * pspec, gpointer data)
 		}
 		DEBUG_MSG("side_panel_notify_position, side_panel_width=%d\n", main_v->globses.left_panel_width);
 	}
+	if (main_v->props.wrap_on_right_margin) {
+		/* all docs need to recalculate the amount of whitespace on the right */
+		g_print("side_panel_notify_position, recalc right margin\n");
+		bfwin_alldoc_recalc_right_margin(bfwin);
+	}
 }
 
 static void
@@ -223,6 +227,7 @@ side_panel_build(Tbfwin * bfwin)
 	DEBUG_MSG("side_panel_build, building side panel for bfwin %p\n", bfwin);
 	bmarks = bmark_gui(bfwin);
 	fb2g = fb2_init(bfwin);
+	fb2_update_settings_from_session(bfwin, NULL);
 	gtk_notebook_append_page_menu(GTK_NOTEBOOK(bfwin->leftpanel_notebook), fb2g, new_pixmap(105),
 								  gtk_label_new(_("Filebrowser")));
 	gtk_notebook_append_page_menu(GTK_NOTEBOOK(bfwin->leftpanel_notebook), bmarks, new_pixmap(104),
@@ -367,7 +372,7 @@ bfwin_statusbar_show(Tbfwin * bfwin, gboolean active)
 }
 
 void
-bfwin_apply_session(Tbfwin * bfwin)
+bfwin_apply_session(Tbfwin * bfwin, Tdocument *active_doc)
 {
 	bfwin_set_main_toolbar_visible(bfwin, bfwin->session->view_main_toolbar, TRUE);
 
@@ -380,7 +385,7 @@ bfwin_apply_session(Tbfwin * bfwin)
 
 	bfwin_statusbar_show_hide_toggle(bfwin, bfwin->session->view_statusbar, TRUE);
 
-	fb2_update_settings_from_session(bfwin);
+	fb2_update_settings_from_session(bfwin, active_doc);
 	bfwin_recent_menu_create(bfwin, TRUE);
 
 	if (bfwin->simplesearch_combo) {
@@ -412,32 +417,27 @@ bfwin_apply_settings(Tbfwin * bfwin)
 	}
 }
 
-static gboolean
-bfwin_configure_event(GtkWidget * widget, GdkEvent * revent, Tbfwin * bfwin)
+static gboolean bfwin_window_state_event_idle_cb(gpointer data) {
+	bfwin_alldoc_recalc_right_margin(data);
+	return FALSE;
+}
+
+static gboolean bfwin_window_state_event(GtkWidget * widget, GdkEvent * revent, Tbfwin * bfwin)
 {
 	if (main_v->props.restore_dimensions) {
-		if (revent->type == GDK_CONFIGURE) {
-			GdkEventConfigure *event = (GdkEventConfigure *) revent;
-			DEBUG_MSG("bfwin_configure_event, configure event\n");
-			if (main_v->globses.main_window_w > 0) {
-				main_v->globses.main_window_w = event->width;
-				main_v->globses.main_window_h = event->height;
-				DEBUG_MSG("bfwin_configure_event, not maximized, setting width=%d, height=%d\n",
-						  main_v->globses.main_window_w, main_v->globses.main_window_h);
-			}
-		} else if (revent->type == GDK_WINDOW_STATE && gtk_widget_get_visible(bfwin->main_window)) {
+		if (gtk_widget_get_visible(bfwin->main_window)) {
 			GdkEventWindowState *event = (GdkEventWindowState *) revent;
 			DEBUG_MSG
-				("bfwin_configure_event, window state event, state=%d, globses_main_window_w=%d\n",
+				("bfwin_window_state_event, window state event, state=%d, globses_main_window_w=%d\n",
 				 event->new_window_state, main_v->globses.main_window_w);
 			if ((event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED) && main_v->globses.main_window_w > 0) {
 				main_v->globses.main_window_w = -1 * main_v->globses.main_window_w;	/* negative means it is maximized !! */
-				DEBUG_MSG("bfwin_configure_event, maximized!! setting width=%d\n",
+				DEBUG_MSG("bfwin_window_state_event, maximized!! storing width=%d\n",
 						  main_v->globses.main_window_w);
 			} else if (!(event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED)
 					   && main_v->globses.main_window_w < 0) {
 				main_v->globses.main_window_w = -1 * main_v->globses.main_window_w;	/* make it positive again */
-				DEBUG_MSG("bfwin_configure_event, NOT-maximized, setting width=%d\n",
+				DEBUG_MSG("bfwin_window_state_event, NOT-maximized, storing width=%d\n",
 						  main_v->globses.main_window_w);
 			}
 			if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN) {
@@ -451,6 +451,31 @@ bfwin_configure_event(GtkWidget * widget, GdkEvent * revent, Tbfwin * bfwin)
 		}
 	}
 
+	if (main_v->props.wrap_on_right_margin && ((revent->type == GDK_WINDOW_STATE && (((GdkEventWindowState *)revent)->changed_mask & (GDK_WINDOW_STATE_FULLSCREEN | GDK_WINDOW_STATE_MAXIMIZED))))) {
+		/* all docs need to recalculate the amount of whitespace on the right, but the width is not yet set,
+		so do this in an idle callback */
+		g_idle_add(bfwin_window_state_event_idle_cb, bfwin);
+	}
+	return FALSE;
+}
+
+static gboolean
+bfwin_configure_event(GtkWidget * widget, GdkEvent * revent, Tbfwin * bfwin)
+{
+	if (main_v->props.restore_dimensions) {
+		GdkEventConfigure *event = (GdkEventConfigure *) revent;
+		DEBUG_MSG("bfwin_configure_event, configure event, got event->width=%d, event->height=%d\n",event->width,event->height);
+		if (main_v->globses.main_window_w > 0) {
+			main_v->globses.main_window_w = event->width;
+			main_v->globses.main_window_h = event->height;
+			DEBUG_MSG("bfwin_configure_event, not maximized, storing width=%d, height=%d\n",
+					  main_v->globses.main_window_w, main_v->globses.main_window_h);
+		}
+	}
+	if (main_v->props.wrap_on_right_margin) {
+		/* all docs need to recalculate the amount of whitespace on the right */
+		bfwin_alldoc_recalc_right_margin(bfwin);
+	}
 	return FALSE;
 }
 
@@ -601,6 +626,39 @@ bfwin_delete_event(GtkWidget * widget, GdkEvent * event, Tbfwin * bfwin)
 	}
 
 }
+#ifdef MAC_INTEGRATION
+gboolean
+bfwin_osx_terminate_event(GtkWidget * widget, GdkEvent * event, Tbfwin * bfwin)
+{
+	/*
+	   On OSX we need this simplified function since showing dialog window on NSApplicationBlockTermination callback
+	   causes bluefish to crash. Probably this is bug in gtk-mac-integration. So, we save all modified documents
+	   by default without asking if user really wants to do it.
+	 */
+
+	DEBUG_MSG("bfwin_osx_terminate_event, started for bfwin %p\n", bfwin);
+	if (!bfwin->documentlist) {
+		g_print("bfwin_osx_terminate_event, document list is empty\n");
+		bfwin_destroy_and_cleanup(bfwin);
+		return TRUE;
+	}
+
+	if (have_modified_documents(bfwin->documentlist)) {
+			DEBUG_MSG("bfwin_osx_terminate_event, have modified documents\n");
+			if (bfwin->project && bfwin->project->uri) {
+			project_save_and_mark_closed(bfwin);
+			}
+			doc_save_all_close(bfwin);
+	} else {
+		DEBUG_MSG("bfwin_osx_terminate_event, nothing modified, close all\n");
+		if (bfwin->project && bfwin->project->uri) {
+		project_save_and_mark_closed(bfwin);
+		}
+		doc_close_multiple_backend(bfwin, TRUE, close_mode_close_all);
+	}
+	return TRUE;
+}
+#endif
 
 void
 bfwin_gotoline_from_clipboard(Tbfwin * bfwin)
@@ -644,12 +702,13 @@ bfwin_simplesearch_show(Tbfwin *bfwin)
 			gchar *tmpstr = gtk_text_buffer_get_text(bfwin->current_document->buffer,&itstart,&itend,TRUE);
 			gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))),tmpstr);
 			g_free(tmpstr);
-			gtk_editable_select_region(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))),0,-1);
 			/* TODO: mark the current selection as the 'current' search result */
 		}
 	}
+
 	simplesearch_combo_entry_activated(bfwin->simplesearch_regex, bfwin);
 	gtk_widget_grab_focus(bfwin->simplesearch_combo);
+	gtk_editable_select_region(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))),0,-1);
 }
 
 void
@@ -683,6 +742,9 @@ gotoline_entry_changed(GtkEditable * editable, Tbfwin * bfwin)
 
 	if (!bfwin->current_document)
 		return;
+	if (!gtk_widget_get_visible(bfwin->gotoline_frame))
+		return;
+
 	DEBUG_MSG("gotoline_entry_changed, called!\n");
 	linestr = gtk_editable_get_chars(editable, 0, -1);
 	linenum = get_int_from_string(linestr);
@@ -731,17 +793,19 @@ gotoline_entry_insert_text(GtkEditable * editable, gchar * text, gint length, gi
 }
 
 void
-bfwin_gotoline_search_bar_close(Tbfwin *bfwin)
+bfwin_gotoline_search_bar_close(Tbfwin *bfwin, gboolean clean_entry_only)
 {
-	DEBUG_MSG("bfwin_gotoline_search_bar_close, called!\n");
-	gtk_widget_hide(bfwin->gotoline_frame);
+	if (!clean_entry_only) {
+		DEBUG_MSG("bfwin_gotoline_search_bar_close, called!\n");
+		gtk_widget_hide(bfwin->gotoline_frame);
 
-	if (bfwin->simplesearch_snr3run) {
-		DEBUG_MSG("free simple search run %p\n", bfwin->simplesearch_snr3run);
-		snr3run_free(bfwin->simplesearch_snr3run);
-		bfwin->simplesearch_snr3run = NULL;
+		if (bfwin->simplesearch_snr3run) {
+			DEBUG_MSG("free simple search run %p\n", bfwin->simplesearch_snr3run);
+			snr3run_free(bfwin->simplesearch_snr3run, TRUE);
+			bfwin->simplesearch_snr3run = NULL;
+		}
 	}
-
+	/*gtk_editable_delete_text(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))), 0, -1);*/
 	g_signal_handlers_block_matched(bfwin->gotoline_entry,
 									G_SIGNAL_MATCH_FUNC, 0, 0, NULL, gotoline_entry_changed, NULL);
 
@@ -755,7 +819,7 @@ static void
 gotoline_close_button_clicked(GtkButton * button, Tbfwin * bfwin)
 {
 	DEBUG_MSG("gotoline_close_button_clicked, called!\n");
-	bfwin_gotoline_search_bar_close(bfwin);
+	bfwin_gotoline_search_bar_close(bfwin, FALSE);
 	if (bfwin->current_document)
 		gtk_widget_grab_focus(bfwin->current_document->view);
 }
@@ -784,7 +848,7 @@ simplesearch_start(Tbfwin *bfwin, gboolean allow_single_char_search) {
 	tmpstr = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))));
 	if (bfwin->simplesearch_snr3run) {
 		DEBUG_MSG("free simple search run %p\n", bfwin->simplesearch_snr3run);
-		snr3run_free(bfwin->simplesearch_snr3run);
+		snr3run_free(bfwin->simplesearch_snr3run, TRUE);
 		bfwin->simplesearch_snr3run=NULL;
 	}
 	if (tmpstr && tmpstr[0]!='\0' && (allow_single_char_search || tmpstr[1] != '\0')) {
@@ -833,6 +897,8 @@ simplesearch_back_clicked(GtkButton * button, Tbfwin * bfwin)
 static void
 simplesearch_combo_entry_changed(gpointer widget, Tbfwin * bfwin)
 {
+	if (!gtk_widget_get_visible(bfwin->gotoline_frame))
+		return;
 	DEBUG_MSG("simplesearch_combo_entry_changed, call simplesearch_start()\n");
 	simplesearch_start(bfwin, FALSE);
 }
@@ -840,6 +906,8 @@ simplesearch_combo_entry_changed(gpointer widget, Tbfwin * bfwin)
 static void
 simplesearch_combo_entry_activated(gpointer widget, Tbfwin * bfwin)
 {
+	if (!gtk_widget_get_visible(bfwin->gotoline_frame))
+		return;
 	DEBUG_MSG("simplesearch_combo_entry_activated, call simplesearch_start()\n");
 	simplesearch_forward_clicked(NULL, bfwin);
 }
@@ -873,6 +941,8 @@ gotoline_entries_key_press_event(GtkWidget * widget, GdkEventKey * event, Tbfwin
 	}
 	if (event->keyval == GDK_KEY_Return && widget == bfwin->gotoline_entry) {
 		gotoline_entry_changed(GTK_EDITABLE(widget), bfwin);
+		/* the close button clicked function automatically switches the focus to the text widget */
+		gotoline_close_button_clicked(NULL, bfwin);
 		return TRUE;
 	}
 	return FALSE;
@@ -955,11 +1025,25 @@ bfwin_destroy_event(GtkWidget * widget, Tbfwin * bfwin)
 	}
 	main_v->bfwinlist = g_list_remove(main_v->bfwinlist, bfwin);
 	DEBUG_MSG("bfwin_destroy_event, bfwin(%p) is removed from bfwinlist\n", bfwin);
-
+#ifndef MAC_INTEGRATION
 	if (NULL == main_v->bfwinlist) {
 		rcfile_save_global_session();
 		gtk_main_quit();
 	}
+#else
+	if (NULL == main_v->bfwinlist) {
+		if (main_v->osx_status == 0 ) {
+			main_v->osx_status = 2;
+			bfwin_window_new();
+			return;
+		}
+		if (main_v->osx_status == 1 ) {
+			rcfile_save_global_session();
+			gtk_main_quit();
+			return;
+		}
+	}
+#endif
 }
 
 static void
@@ -1024,7 +1108,13 @@ static void
 notebook_set_tab_accels(Tbfwin * bfwin)
 {
 	GtkAccelGroup *tab_accels;
-
+	gboolean create_signals = TRUE;
+#ifdef MAC_INTEGRATION /* We need to create these signals just once, at startup */
+	if (main_v->osx_status != 0 || main_v->bfwinlist != NULL) {
+	create_signals = FALSE;
+	}
+#endif
+	if (create_signals) {
 	DEBUG_MSG("notebook_set_tab_accels, g_signal_new for <Alt>X triggered events\n");
 	g_signal_new("tab-last", gtk_widget_get_type(), G_SIGNAL_ACTION, 0, NULL, NULL,
 				 g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
@@ -1046,7 +1136,7 @@ notebook_set_tab_accels(Tbfwin * bfwin)
 				 g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 	g_signal_new("tab9", gtk_widget_get_type(), G_SIGNAL_ACTION, 0, NULL, NULL,
 				 g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-
+	}
 	tab_accels = gtk_accel_group_new();
 	DEBUG_MSG("notebook_set_tab_accels, gtk_window_add_accel_group\n");
 	gtk_window_add_accel_group(GTK_WINDOW(bfwin->main_window), tab_accels);
@@ -1068,6 +1158,7 @@ notebook_set_tab_accels(Tbfwin * bfwin)
 static gboolean
 notebook_changed_activate_current_document(gpointer data)
 {
+	DEBUG_SIG("notebook_changed_activate_current_document, priority=%d\n",NOTEBOOKCHANGED_DOCACTIVATE_PRIORITY);
 	DEBUG_MSG("notebook_changed_activate_current_document, current_document=%p\n",BFWIN(data)->current_document);
 	if (BFWIN(data)->current_document)
 		doc_activate(BFWIN(data)->current_document);
@@ -1247,6 +1338,39 @@ notebook_reordered(GtkNotebook * notebook, GtkWidget * child, guint page_num, gp
 	DEBUG_MSG("notebook_reordered, done\n");
 }
 
+/* Restores tab scrolling feature for gtk+3 builds */
+#if GTK_CHECK_VERSION (3, 0, 0)
+static gboolean
+notebook_scroll_event_lcb (GtkNotebook * notebook, GdkEventScroll *event, gpointer user_data)
+{
+	DEBUG_MSG("notebook_scroll_event, started for notebook=%p\n", notebook);
+
+	/* Some sanity check, which is required when document does not have scroll bar and scroll-events are not consumed by textview*/
+	GtkWidget *child, *event_widget;
+	child = gtk_notebook_get_nth_page (notebook, gtk_notebook_get_current_page (notebook));
+	if (child == NULL)
+		return FALSE;
+	event_widget = gtk_get_event_widget ((GdkEvent *) event);
+	if (event_widget == NULL || event_widget == child || gtk_widget_is_ancestor (event_widget, child))
+		return FALSE; 
+
+	switch (event->direction) {
+		case GDK_SCROLL_RIGHT:
+		case GDK_SCROLL_DOWN:
+			gtk_notebook_next_page (notebook);
+			break;
+		case GDK_SCROLL_LEFT:
+		case GDK_SCROLL_UP:
+			gtk_notebook_prev_page (notebook);
+			break;
+		default:
+			break;
+	}
+
+	return TRUE;
+}
+#endif
+
 static void
 notebook_switch_page(GtkWidget * notebook, gpointer * page, gint page_num, Tbfwin * bfwin)
 {
@@ -1343,7 +1467,9 @@ bfwin_create_main(Tbfwin * bfwin)
 		window_full2(_("New Bluefish Window"), GTK_WIN_POS_CENTER, 0, G_CALLBACK(bfwin_destroy_event),
 					 bfwin, FALSE, NULL);
 	gtk_window_set_role(GTK_WINDOW(bfwin->main_window), "bluefish");
-	gtk_widget_realize(bfwin->main_window);
+#ifndef MAC_INTEGRATION
+	gtk_widget_realize(bfwin->main_window); /* Causes half baked window entry to appear on OSX WIndow menu. Probably this is not required also on other systems */
+#endif
 	if (!main_v->props.leave_to_window_manager) {
 		if (main_v->globses.main_window_w > 0) {
 			gtk_window_set_default_size(GTK_WINDOW(bfwin->main_window), main_v->globses.main_window_w,
@@ -1358,18 +1484,23 @@ bfwin_create_main(Tbfwin * bfwin)
 	g_signal_connect(G_OBJECT(bfwin->main_window), "configure-event",
 					 G_CALLBACK(bfwin_configure_event), bfwin);
 	g_signal_connect(G_OBJECT(bfwin->main_window), "window-state-event",
-					 G_CALLBACK(bfwin_configure_event), bfwin);
+					 G_CALLBACK(bfwin_window_state_event), bfwin);
 	g_signal_connect(G_OBJECT(bfwin->main_window), "key-press-event", G_CALLBACK(bfwin_key_press_event), bfwin);
 	g_signal_connect(G_OBJECT(bfwin->main_window), "key-release-event", G_CALLBACK(bfwin_key_release_event), bfwin);
 
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(bfwin->main_window), vbox);
 	gtk_widget_show(vbox);
-
+#if GTK_CHECK_VERSION(3,4,0)
+	bfwin->toolbarbox = gtk_grid_new();
+	gtk_orientable_set_orientation(GTK_ORIENTABLE(bfwin->toolbarbox), GTK_ORIENTATION_VERTICAL);
+	gtk_box_pack_start(GTK_BOX(vbox), bfwin->toolbarbox, FALSE, FALSE, 0);
+	gtk_widget_show(bfwin->toolbarbox);
+#else
 	bfwin->toolbarbox = gtk_vbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), bfwin->toolbarbox, FALSE, FALSE, 0);
 	gtk_widget_show(bfwin->toolbarbox);
-
+#endif
 	/* first a menubar */
 	bfwin->uimanager = gtk_ui_manager_new();
 #if !GTK_CHECK_VERSION(3,4,0)
@@ -1459,6 +1590,10 @@ bfwin_create_main(Tbfwin * bfwin)
 	}
 	notebook_connect_signals(bfwin);
 	g_signal_connect(G_OBJECT(bfwin->notebook), "page-reordered", G_CALLBACK(notebook_reordered), bfwin);
+#if GTK_CHECK_VERSION (3, 0, 0) /* Restores tab scrolling feature for gtk+3 builds */
+	gtk_widget_add_events (bfwin->notebook, GDK_SCROLL_MASK);
+	g_signal_connect (G_OBJECT(bfwin->notebook), "scroll-event", G_CALLBACK (notebook_scroll_event_lcb), bfwin);
+#endif
 
 	if (main_v->props.switch_tabs_by_altx) {
 		notebook_set_tab_accels(bfwin);
@@ -1522,23 +1657,35 @@ bfwin_show_main(Tbfwin * bfwin)
 {
 #ifdef MAC_INTEGRATION
 	GtkWidget *menuitem;
-	GtkOSXApplication *theApp = g_object_new(GTK_TYPE_OSX_APPLICATION, NULL);
+	GtkosxApplication *theApp = g_object_new(GTKOSX_TYPE_APPLICATION, NULL);
 	gtk_widget_hide(bfwin->menubar);
-	gtk_osxapplication_set_menu_bar(theApp, GTK_MENU_SHELL(bfwin->menubar));
-	g_print("hide gtk menubar, set gtkosxapplication menubar\n");
+	gtkosx_application_set_menu_bar(theApp, GTK_MENU_SHELL(bfwin->menubar));
+	DEBUG_MSG("hide gtk menubar, set gtkosxapplication menubar\n");
+/*This arrangement gives more mackish ordering of menu. TODO Window menu does not track opened toplevels correctly; see bug #701571; for 2.2.5 temporal hack applied to the gdk/quartz/gdkeventloop-quartz.c  */
+	gint menupos = 0;
+	if (bfwin_action_group_is_available(bfwin->uimanager, "AboutActions")) { /*Since it is plugin, user can disable it, and this leads to bf crash */
+		menuitem = gtk_ui_manager_get_widget(bfwin->uimanager, "/MainMenu/HelpMenu");
+		gtkosx_application_set_help_menu (theApp, menuitem);
+		menuitem = gtk_ui_manager_get_widget(bfwin->uimanager, "/MainMenu/HelpMenu/HelpAbout");
+		gtkosx_application_insert_app_menu_item(theApp, menuitem, 0);
+		gtkosx_application_insert_app_menu_item (theApp, g_object_ref(gtk_separator_menu_item_new ()), 1);
+		menupos = 2;
+	}
+	gtkosx_application_set_window_menu (theApp, NULL);
 
 	menuitem = gtk_ui_manager_get_widget(bfwin->uimanager, "/MainMenu/EditMenu/EditPreferences");
-	gtk_osxapplication_insert_app_menu_item(theApp, menuitem,5);
-
-	menuitem = gtk_ui_manager_get_widget(bfwin->uimanager, "/MainMenu/HelpMenu/HelpAbout");
-	gtk_osxapplication_insert_app_menu_item(theApp, menuitem,5);
+	gtkosx_application_insert_app_menu_item(theApp, menuitem, menupos);
 
 	menuitem = gtk_ui_manager_get_widget(bfwin->uimanager, "/MainMenu/FileMenu/FileQuit");
 	gtk_widget_hide(menuitem);
 
-	gtk_accel_map_foreach_unfiltered(NULL, osx_accel_map_foreach_controltometa_lcb);
-	gtk_accel_map_foreach_unfiltered(NULL, osx_accel_map_foreach_mod1tocontrol_lcb);
+	gtkosx_application_set_use_quartz_accelerators (theApp, FALSE); /* If set to TRUE then widgets does not respond to bindings like Cmd+X|C|V TODO: find the way to make MacOSX shortcuts Cmd+M and Cmd+H work; for 2.2.5 hack applied to gdk/quartz/gdkkeys-quartz.c see bug #711019*/
 
+	if (main_v->osx_status == 0 && g_list_length(main_v->bfwinlist) == 1) { /* Accelarators should be moved just once, at the startup */
+		DEBUG_MSG("bfwin_show_main, configuring accelerators for OSX\n");
+		gtk_accel_map_foreach_unfiltered(NULL, osx_accel_map_foreach_controltometa_lcb);
+		gtk_accel_map_foreach_unfiltered(NULL, osx_accel_map_foreach_mod1tocontrol_lcb);
+	}
 
 /*
 IgeMacMenuGroup *group;
@@ -1560,6 +1707,13 @@ IgeMacMenuGroup *group;
 
 	/* MACTODO: add focus in and focus out event so we can sync the menu
 	   when we switch to a different bluefish window */
+
+	   if (main_v->osx_status == 2) {
+
+	   	bfwin_action_groups_set_sensitive(bfwin, FALSE);
+
+		return;
+	   }
 #endif
 	DEBUG_MSG("bfwin_show_main, before show\n");
 	gtk_widget_show(bfwin->main_window);
