@@ -32,7 +32,7 @@ typedef void (*DocInsertTextCallback) (Tdocument *doc, const gchar *string, GtkT
 #define bfwin_exists(bfwin) (g_list_index(main_v->bfwinlist, bfwin)!=-1)
 
 void bfwin_fullscreen_toggle(Tbfwin * bfwin, gboolean active);
-void bfwin_gotoline_search_bar_close(Tbfwin *bfwin);
+void bfwin_gotoline_search_bar_close(Tbfwin *bfwin, gboolean clean_entry_only);
 void bfwin_gotoline_from_clipboard(Tbfwin * bfwin);
 void bfwin_gotoline_frame_show(Tbfwin * bfwin);
 void bfwin_simplesearch_show(Tbfwin *bfwin);
@@ -65,10 +65,13 @@ void bfwin_side_panel_rebuild(Tbfwin * bfwin);
 
 void bfwin_statusbar_show(Tbfwin * bfwin, gboolean active);
 
-void bfwin_apply_session(Tbfwin * bfwin);
+void bfwin_apply_session(Tbfwin * bfwin, Tdocument *active_doc);
 void bfwin_apply_settings(Tbfwin * bfwin);
 void bfwin_destroy_and_cleanup(Tbfwin *bfwin);
 gboolean bfwin_delete_event(GtkWidget * widget, GdkEvent * event, Tbfwin * bfwin);
+#ifdef MAC_INTEGRATION
+gboolean bfwin_osx_terminate_event(GtkWidget * widget, GdkEvent * event, Tbfwin * bfwin);
+#endif
 
 void bfwin_create_main(Tbfwin * bfwin);
 void bfwin_show_main(Tbfwin * bfwin);
