@@ -21,7 +21,7 @@
 /*#define DEBUG */
 
 #include <gtk/gtk.h>
-#include <stdlib.h>				/* exit() on Solaris */
+#include <stdlib.h>				/* exit() on Solaris and unsetenv()  */
 #include <time.h>				/* nanosleep */
 
 #ifndef WIN32
@@ -67,16 +67,13 @@
 #include "file_autosave.h"
 #include "languages.h"
 
+/*#define STARTUP_PROFILING */
+
 /*********************************************/
 /* this var is global for all bluefish files */
 /*********************************************/
 Tmain *main_v;
 
-/********************************/
-/* functions used in bluefish.c */
-/********************************/
-/*#define STARTUP_PROFILING*/
-
 #ifdef STARTUP_PROFILING
 static GTimer *startuptimer;
 #endif
@@ -132,27 +129,63 @@ static void handle_signals(void) {
 #endif
 
 #ifdef MAC_INTEGRATION
-static gboolean osx_open_file_cb(GtkOSXApplication *app, gchar *path, gpointer user_data) {
-	GFile *file;
+static gboolean osx_open_file_cb(GtkosxApplication *app, gchar *path, gpointer user_data) {
+	GFile *uri = g_file_new_for_path(path);
 	Tbfwin *bfwin = BFWIN(g_list_last(main_v->bfwinlist)->data);
 	g_print("osx_open_file_cb, open %s\n",path);
-	doc_new_from_input(bfwin,path,FALSE,FALSE,-1);
+	file_handle(uri, bfwin , NULL, TRUE);
+	g_object_unref(uri);
 	return TRUE;
 }
-#endif
 
-/*********************/
-/* the main function */
-/*********************/
+static gboolean osx_block_termination_cb(GtkosxApplication *app, gpointer user_data) {
+	GList *tmplist, *duplist;
+	main_v->osx_status = 1;
+	DEBUG_MSG("osx_block_termination, started\n");
+	duplist = g_list_copy(main_v->bfwinlist); /* Copy bfwinlist first, since using just tmplist causes unexpected behavior as bfwin is destroyed */
+	tmplist = g_list_first(duplist);
+	while (tmplist) {
+		Tbfwin *bfwin = BFWIN(tmplist->data);
+		bfwin_osx_terminate_event(NULL,NULL,bfwin);
+		tmplist = g_list_next(tmplist);
+	}
+	g_list_free(duplist);
+	return TRUE;
+}
+
+static gboolean osx_will_terminate_cb(GtkosxApplication *app, gpointer user_data) {
+	/* Connect final shutdown routine here */
+	g_print("gtkosx terminating application");
+	flush_queue();
+	rcfile_save_global_session();
+	gtk_main_quit();
+}
+
+
+#endif
 
 typedef struct {
 	Tbfwin *firstbfwin;
 	GList *filenames;
 	guint state;
+#ifdef MAC_INTEGRATION
+	GMainLoop *startup_main_loop;
+#endif
 } Tstartup;
 
+#ifdef MAC_INTEGRATION /* quits startup loop and releases pointers */
+void startup_finished_cb(gpointer data) {
+	Tstartup *startup=data;
+	g_main_loop_quit (startup->startup_main_loop);
+	g_main_loop_unref (startup->startup_main_loop);
+	g_free(startup);
+}
+#endif
+
+
 static gboolean startup_in_idle(gpointer data) {
 	Tstartup *startup=data;
+	DEBUG_SIG("startup_in_idle, started, priority %d, state=%d\n",BLUEFISH_STARTUP_IN_IDLE_PRIORITY, startup->state);
 	DEBUG_MSG("startup_in_idle, started state=%d, elapsed=\n",startup->state);
 #ifdef STARTUP_PROFILING
 	g_print("startup_in_idle, state=%d, elapsed=%d\n",startup->state,(gint)(g_timer_elapsed(startuptimer,NULL)*1000.0));
@@ -185,7 +218,7 @@ static gboolean startup_in_idle(gpointer data) {
 			startup->firstbfwin = g_new0(Tbfwin, 1);
 			startup->firstbfwin->session = main_v->session;
 			startup->firstbfwin->bmarkdata = main_v->bmarkdata;
-			main_v->bfwinlist = g_list_append(NULL, startup->firstbfwin);
+			main_v->bfwinlist = NULL;
 #ifdef WITH_MSG_QUEUE
 			if (main_v->props.open_in_running_bluefish) {
 				msg_queue_check_server(FALSE);
@@ -194,6 +227,7 @@ static gboolean startup_in_idle(gpointer data) {
 		break;
 		case 1:
 			bfwin_create_main(startup->firstbfwin);
+			main_v->bfwinlist = g_list_append(NULL, startup->firstbfwin); /* Moved it here, so it is handled the same way as in bfwin_window_new() */
 #ifdef WITH_MSG_QUEUE
 			if (main_v->props.open_in_running_bluefish) {
 				msg_queue_check_server(FALSE);
@@ -208,8 +242,10 @@ static gboolean startup_in_idle(gpointer data) {
 				startup->firstbfwin->focus_next_new_doc = TRUE;
 				while (tmplist) {
 					file_handle((GFile *)tmplist->data, startup->firstbfwin, NULL, TRUE);
+					g_object_unref((GFile *)tmplist->data);
 					tmplist = g_list_next(tmplist);
 				}
+				g_list_free(startup->filenames);
 			}
 			if (startup->firstbfwin->session == main_v->session)
 				bmark_reload(startup->firstbfwin); /* do not reload bookmarks for a project */
@@ -228,22 +264,19 @@ static gboolean startup_in_idle(gpointer data) {
 		break;
 		case 3:
 			bfwin_show_main(startup->firstbfwin);
-#ifdef MAC_INTEGRATION
-			{
-				GtkOSXApplication *theApp = g_object_new(GTK_TYPE_OSX_APPLICATION, NULL);
-				gtk_osxapplication_ready(theApp);
-				g_signal_connect(theApp, "NSApplicationOpenFile", osx_open_file_cb, NULL);
-			}
-#endif
 		break;
 		case 4:
 			main_v->recentm = gtk_recent_manager_get_default();
-			doc_scroll_to_cursor(BFWIN(startup->firstbfwin)->current_document);
+			if (BFWIN(startup->firstbfwin)->current_document)
+				doc_scroll_to_cursor(BFWIN(startup->firstbfwin)->current_document);
 			modified_on_disk_check_init();
-#ifndef WIN32            
+#ifndef WIN32
 			handle_signals();
-#endif            
-			g_free(startup);
+#endif
+#ifndef MAC_INTEGRATION /*We need startup to stop the startup_main_loop running*/
+			g_free(startup); 
+#endif
+			
 			return FALSE;
 		break;
 	}
@@ -251,7 +284,7 @@ static gboolean startup_in_idle(gpointer data) {
 	g_print("startup_in_idle, end of state=%d, elapsed=%d\n",startup->state,(gint)(g_timer_elapsed(startuptimer,NULL)*1000.0));
 #endif
 	startup->state++;
-	return TRUE;	
+	return TRUE;
 }
 
 int main(int argc, char *argv[])
@@ -259,6 +292,17 @@ int main(int argc, char *argv[])
 	static gboolean arg_curwindow = FALSE, arg_newwindow=FALSE;
 	static gchar **files = NULL;
 	Tstartup *startup;
+#ifdef MAC_INTEGRATION
+	GPollFunc orig_poll_func;
+	GPollFunc gdk_poll_func;
+#endif
+
+#ifndef WIN32
+	/* the unity appmenu cannot handle an application with two menu's
+	for now I didn't find a way to make it honor the main menu and ignore the snippets menu, so
+	I'll ask to avoid the appmenu at all. */
+	unsetenv("UBUNTU_MENUPROXY");
+#endif
 
 #ifdef WIN32
 	gchar *path;
@@ -269,7 +313,7 @@ int main(int argc, char *argv[])
     HKEY hPython;
     DWORD dwSize;
     DWORD dwError;
-    
+
     DEBUG_MSG("main, about to add Python to PATH");
     // Open HKLM registry key, if that fails try to open HKCU registry key
     if((dwError = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\Python\\PythonCore\\2.7\\InstallPath", 0, KEY_QUERY_VALUE, &hPython)) != ERROR_SUCCESS)
@@ -322,10 +366,11 @@ int main(int argc, char *argv[])
 		 "Special option that collects any remaining arguments for us", NULL},
 		{NULL}
 	};
-
+#if !GLIB_CHECK_VERSION(2, 32, 0)
+	/* g_thread_init() is called by default starting at glib 2.32 */
 	if (!g_thread_supported())
 		g_thread_init(NULL);
-/*  gdk_threads_init ();*/
+#endif
 
 #ifdef STARTUP_PROFILING
 	startuptimer = g_timer_new();
@@ -339,6 +384,10 @@ int main(int argc, char *argv[])
 	textdomain(PACKAGE);
 #endif							/* ENABLE_NLS */
 
+#ifdef MAC_INTEGRATION /* get virgin polling function */
+	orig_poll_func = g_main_context_get_poll_func (NULL);
+#endif
+
 	context = g_option_context_new(_(" [FILE(S)]"));
 #ifdef ENABLE_NLS
 	g_option_context_add_main_entries(context, options, PACKAGE);
@@ -373,6 +422,7 @@ int main(int argc, char *argv[])
 	xmlInitParser();
 	startup = g_new0(Tstartup,1);
 	main_v = g_new0(Tmain, 1);
+	main_v->alldochash = g_hash_table_new(g_file_hash, (GEqualFunc) g_file_equal);
 	DEBUG_MSG("main, main_v is at %p\n", main_v);
 
 	if (files != NULL) {
@@ -431,7 +481,7 @@ int main(int argc, char *argv[])
 #endif /* ENABLE_NLS */
 	if (main_v->props.open_in_running_bluefish) {
 #ifdef WITH_MSG_QUEUE
-		/* start message queue as early as possible so a running bluefish process has a lot of 
+		/* start message queue as early as possible so a running bluefish process has a lot of
 		time to respond to our request-alive request */
 		msg_queue_start(startup->filenames, (arg_newwindow || (main_v->props.open_in_new_window && !arg_curwindow) ) );
 #else
@@ -442,11 +492,27 @@ int main(int argc, char *argv[])
 #endif /* WITH_MSG_QUEUE */
 	}
 #ifdef MAC_INTEGRATION
-	{
-	GtkOSXApplication *theApp = g_object_new(GTK_TYPE_OSX_APPLICATION, NULL);
-	}
-#endif
+	main_v->osx_status = 0; /*normal operation */
+	GtkosxApplication *theApp = g_object_new(GTKOSX_TYPE_APPLICATION, NULL);
+	g_signal_connect(theApp, "NSApplicationOpenFile", G_CALLBACK (osx_open_file_cb), NULL);
+	g_signal_connect (theApp,
+	                  "NSApplicationWillTerminate",
+	                  G_CALLBACK (osx_will_terminate_cb),
+	                  NULL);
+	g_signal_connect (theApp,
+	                  "NSApplicationBlockTermination",
+	                  G_CALLBACK (osx_block_termination_cb),
+	                  NULL); 
+	gdk_poll_func = g_main_context_get_poll_func (NULL);
+	g_main_context_set_poll_func (NULL, orig_poll_func); /* use virgin polling funtion for startup loop */
+	startup->startup_main_loop = g_main_loop_new (NULL, FALSE);
+	g_idle_add_full(G_PRIORITY_DEFAULT_IDLE-50, startup_in_idle, startup, (GDestroyNotify)startup_finished_cb);
+	g_main_loop_run (startup->startup_main_loop);
+	g_main_context_set_poll_func (NULL, gdk_poll_func); /* return to NORMAL polling function */
+	gtkosx_application_ready(theApp);
+#else
 	g_idle_add_full(G_PRIORITY_DEFAULT_IDLE-50, startup_in_idle, startup, NULL);
+#endif
 	DEBUG_MSG("main, before gtk_main()\n");
 	gtk_main();
 	DEBUG_MSG("main, after gtk_main()\n");
@@ -468,9 +534,9 @@ int main(int argc, char *argv[])
 	bluefish_cleanup_plugins();
 	langmgr_cleanup();
 	xmlCleanupParser();
-	
+
 	/*cairo_debug_reset_static_data();
-	FcFini();*/ 
+	FcFini();*/
 	DEBUG_MSG("Bluefish: exiting cleanly\n");
 #else
 	exit(0);
