@@ -37,6 +37,7 @@
 #include "gtk_easy.h"
 #include "stringlist.h"
 #include "async_queue.h"
+#include "undo_redo.h"
 
 #undef OAD_MEMCOUNT
 #ifdef OAD_MEMCOUNT
@@ -75,10 +76,10 @@ static Tasyncqueue ffdqueue;
 void
 file_static_queues_init(void)
 {
-	queue_init(&ofqueue, 16, openfile_run);
-	queue_init(&ffdqueue, 16, findfiles_rundir);
-	queue_init(&fiqueue, 16, fill_fileinfo_run);
-	queue_init(&sfqueue, 16, file_checkNsave_run);
+	queue_init(&ofqueue, 8, openfile_run);
+	queue_init(&ffdqueue, 8, findfiles_rundir);
+	queue_init(&fiqueue, 8, fill_fileinfo_run);
+	queue_init(&sfqueue, 8, file_checkNsave_run);
 }
 
 /********************************** wait for mount functions **********************************/
@@ -144,6 +145,7 @@ static gboolean
 delete_async_finished_lcb(gpointer data)
 {
 	Tdelete *del = data;
+	DEBUG_SIG("delete_async_finished_lcb, started, priority %d\n",G_PRIORITY_LOW);
 	if (del->callback) {
 		del->callback(del->callback_data);
 	}
@@ -207,6 +209,7 @@ delete_async(GIOSchedulerJob * job, GCancellable * cancellable, gpointer user_da
 {
 	Tdelete *del = user_data;
 	GError *error = NULL;
+	DEBUG_SIG("delete_async, started, priority %d\n",G_PRIORITY_LOW);
 	g_file_delete(del->uri, NULL, &error);
 	if (error) {
 		if (error->code == G_IO_ERROR_NOT_EMPTY && del->recursive) {
@@ -628,6 +631,7 @@ static gboolean
 fileintodoc_finished_idle_lcb(gpointer data)
 {
 	Tfileintodoc *fid = data;
+	DEBUG_SIG("file2doc_finished_idle_lcb, started, priority %d\n",FILEINTODOC_PRIORITY);
 	DEBUG_MSG("fileintodoc_finished_idle_lcb, loading the data for doc %p\n",fid->doc);
 	if (fid->isTemplate || fid->untiledRecovery) {
 		doc_buffer_to_textbox(fid->doc, fid->buffer->data, fid->buflen, FALSE, TRUE);
@@ -764,17 +768,27 @@ file2doc_cancel(gpointer f2d)
 }
 
 static gboolean
-file2doc_goto_idle_cb(Tfile2doc * f2d)
+file2doc_goto_idle_cb(gpointer data)
 {
+	Tfile2doc *f2d = data;
+	DEBUG_MSG("file2doc_goto_idle_cb, goto_line=%d, goto_offset=%d, cursor_offset=%d\n", f2d->doc->goto_line,  f2d->doc->goto_offset,  f2d->doc->cursor_offset);
 	if (f2d->doc->goto_line >= 0) {
-		DEBUG_MSG("file2doc_lcb, goto_line=%d\n", f2d->doc->goto_line);
+		DEBUG_MSG("file2doc_goto_idle_cb, goto_line=%d\n", f2d->doc->goto_line);
 		doc_select_line(f2d->doc, f2d->doc->goto_line, TRUE);
-	} else if (f2d->doc->goto_offset >= 0) {
-		DEBUG_MSG("file2doc_lcb, goto_offset=%d\n", f2d->doc->goto_offset);
-		doc_select_line_by_offset(f2d->doc, f2d->doc->goto_offset, TRUE);
+	} else {
+		if (f2d->doc->goto_offset >= 0) {
+			DEBUG_MSG("file2doc_goto_idle_cb, goto_offset=%d, align_center=%d\n", f2d->doc->goto_offset, f2d->doc->align_center);
+			doc_select_line_by_offset(f2d->doc, f2d->doc->goto_offset, TRUE, f2d->doc->align_center);
+		}
+		if (f2d->doc->cursor_offset >= 0) {
+			DEBUG_MSG("file2doc_goto_idle_cb, cursor_offset=%d\n", f2d->doc->cursor_offset);
+			doc_set_cursor_position(f2d->doc, f2d->doc->cursor_offset);
+		}
 	}
 	f2d->doc->goto_line = -1;
+	f2d->doc->cursor_offset = -1;
 	f2d->doc->goto_offset = -1;
+	f2d->doc->align_center = TRUE;
 	f2d->doc->load = NULL;
 	file2doc_cleanup(f2d);
 	return FALSE;
@@ -788,17 +802,29 @@ file2doc_finished_idle_lcb(gpointer data)
 {
 	Tfile2doc * f2d=data;
 	Trefcpointer *refp = f2d->buffer;
+	DEBUG_SIG("file2doc_finished_idle_lcb, started, priority %d\n",FILE2DOC_PRIORITY);
 	DEBUG_MSG("file2doc_finished_idle_lcb started for doc %p\n",f2d->doc);
 	if (f2d->recovery_status == 1) {
 		GtkTextIter itstart, itend;
 
 		f2d->recovery_status = 2;
 		doc_buffer_to_textbox(f2d->doc, f2d->buffer->data, f2d->buflen, FALSE, TRUE);
+		g_print("file2doc_finished_idle_lcb, recovery of existing file, inserted original file\n");
+		f2d->doc->block_undo_reg = TRUE;
+		doc_unre_new_group(f2d->doc);
+		doc_unre_add(f2d->doc, f2d->buffer->data, 0, g_utf8_strlen(f2d->buffer->data, f2d->buflen), UndoDelete);
 		gtk_text_buffer_get_bounds(f2d->doc->buffer, &itstart, &itend);
 		gtk_text_buffer_delete(f2d->doc->buffer, &itstart, &itend);
+		DEBUG_MSG("file2doc_finished_idle_lcb, recovery of existing file, deleted contents, load autosaved file\n");
 		f2d->of = file_openfile_uri_async(f2d->recover_uri, f2d->bfwin, file2doc_lcb, f2d);
 	} else if (f2d->recovery_status == 2) {
 		doc_buffer_to_textbox(f2d->doc, f2d->buffer->data, f2d->buflen, FALSE, TRUE);
+		doc_unre_add(f2d->doc, f2d->buffer->data, 0, g_utf8_strlen(f2d->buffer->data, f2d->buflen), UndoInsert);
+		f2d->doc->block_undo_reg = FALSE;
+		doc_unre_new_group(f2d->doc);
+		DEBUG_MSG("file2doc_finished_idle_lcb, inserted loaded file\n");
+		doc_reset_filetype(f2d->doc, f2d->doc->uri, f2d->buffer->data, f2d->buflen);
+		doc_set_tooltip(f2d->doc);
 		f2d->doc->autosave_uri = f2d->recover_uri;
 		f2d->doc->autosaved = register_autosave_journal(f2d->recover_uri, f2d->doc->uri, NULL);
 		doc_set_status(f2d->doc, DOC_STATUS_COMPLETE);
@@ -832,18 +858,26 @@ file2doc_finished_idle_lcb(gpointer data)
 											   BFWIN(f2d->bfwin)->num_docs_not_completed),
 									  BFWIN(f2d->bfwin)->num_docs_not_completed, utf8uri);
 			} else {
-				tmp = g_strdup_printf(_("All files loaded, finished %s"), utf8uri);
+				gint doclistlen = g_list_length(BFWIN(f2d->bfwin)->documentlist);
+				tmp = g_strdup_printf(ngettext("All files loaded, finished %s, %d document open", "All files loaded, finished %s, %d documents open", doclistlen), utf8uri, doclistlen);
 			}
 			bfwin_statusbar_message(f2d->doc->bfwin, tmp, 3);
 			g_free(tmp);
 			g_free(utf8uri);
 		}
 		add_filename_to_recentlist(BFWIN(f2d->doc->bfwin), f2d->doc->uri);
-		if (f2d->doc->goto_line >= 0 || f2d->doc->goto_offset >= 0) {
-			g_idle_add(((GSourceFunc)file2doc_goto_idle_cb), f2d);
+		DEBUG_MSG("file2doc_finished_idle_lcb, goto_line=%d, goto_offset=%d, cursor_offset=%d\n", f2d->doc->goto_line,  f2d->doc->goto_offset,  f2d->doc->cursor_offset);
+		if (f2d->doc->goto_line > 0 || f2d->doc->goto_offset > 0 || f2d->doc->cursor_offset > 0) {
+			if (f2d->doc->load_first) {
+				g_idle_add_full(FILE2DOC_PRIORITY-2,file2doc_goto_idle_cb, f2d, NULL);
+			} else {
+				g_idle_add_full(FILE2DOC_PRIORITY-1,file2doc_goto_idle_cb, f2d, NULL);
+			}
 		} else {
 			f2d->doc->goto_line = -1;
+			f2d->doc->cursor_offset = -1;
 			f2d->doc->goto_offset = -1;
+			f2d->doc->align_center = TRUE;
 			f2d->doc->load = NULL;
 			file2doc_cleanup(data);
 		}
@@ -864,7 +898,11 @@ file2doc_lcb(Topenfile_status status, GError * gerror, Trefcpointer * buffer, go
 		f2d->buffer = buffer;
 		f2d->buflen = buflen;
 		refcpointer_ref(buffer);
-		g_idle_add_full(FILE2DOC_PRIORITY, file2doc_finished_idle_lcb, f2d, NULL);
+		if (f2d->doc->load_first) {
+			g_idle_add_full(FILE2DOC_PRIORITY-1, file2doc_finished_idle_lcb, f2d, NULL);
+		} else {
+			g_idle_add_full(FILE2DOC_PRIORITY, file2doc_finished_idle_lcb, f2d, NULL);
+		}
 		break;
 	case OPENFILE_CHANNEL_OPENED:
 		/* do nothing */
@@ -1050,7 +1088,7 @@ file_doc_fill_from_uri(Tdocument * doc, GFile * uri, GFileInfo * finfo, gint got
 /* this funcion is usually used to load documents */
 void
 file_doc_from_uri(Tbfwin * bfwin, GFile * uri, GFile * recover_uri, GFileInfo * finfo, gint goto_line,
-				  gint goto_offset, gboolean readonly)
+				  gint goto_offset, gboolean readonly, gint cursor_offset, gboolean align_center, gboolean load_first)
 {
 	Tfile2doc *f2d;
 	f2d = g_slice_new0(Tfile2doc);
@@ -1065,7 +1103,10 @@ file_doc_from_uri(Tbfwin * bfwin, GFile * uri, GFile * recover_uri, GFileInfo *
 	f2d->doc = doc_new_loading_in_background(bfwin, uri, finfo, readonly);
 	f2d->doc->load = f2d;
 	f2d->doc->goto_line = goto_line;
+	f2d->doc->cursor_offset = cursor_offset;
 	f2d->doc->goto_offset = goto_offset;
+	f2d->doc->align_center = align_center;
+	f2d->doc->load_first = load_first;
 	DEBUG_MSG("file_doc_from_uri, got doc %p\n", f2d->doc);
 	if (finfo == NULL) {
 		/* get the fileinfo also async */
@@ -1344,7 +1385,10 @@ open_advanced_cleanup(Topenadvanced *oa) {
 static void open_advanced_unref(Topenadvanced *oa) {
 	oa->refcount--;
 	if (oa->refcount == 0) {
-		bfwin_statusbar_message(oa->bfwin, _("Finished advanced open"), 2);
+		gint doclistlen = g_list_length(oa->bfwin->documentlist);
+		gchar *tmp = g_strdup_printf(ngettext("Finished advanced open, %d document open.", "Finished advanced open, %d documents open.", doclistlen), doclistlen);
+		bfwin_statusbar_message(oa->bfwin, tmp, 2);
+		g_free(tmp);
 		open_advanced_cleanup(oa);
 	}
 }
@@ -1443,7 +1487,7 @@ open_advanced_filematch_cb(Topenadvanced *oa, GFile *uri, GFileInfo *finfo) {
 	if (oa->content_filter) {	/* do we need content filtering */
 		openadv_content_filter_file(oa, uri, finfo);
 	} else {	/* open this file as document */
-		doc_new_from_uri(oa->bfwin, uri, finfo, TRUE, FALSE, -1, -1);
+		doc_new_from_uri(oa->bfwin, uri, finfo, TRUE, FALSE, -1, -1, -1, TRUE, FALSE);
 	}
 }
 
@@ -1538,7 +1582,7 @@ copy_async_lcb(GObject * source_object, GAsyncResult * res, gpointer user_data)
 	g_object_unref(cf->curdest);
 
 	if (!copy_uris_process_queue(cf)) {
-		fb2_refresh_dir_from_uri(cf->destdir);
+		filetreemodel_refresh_uri_async(FB2CONFIG(main_v->fb2config)->ftm, cf->destdir);
 		g_object_unref(cf->destdir);
 		g_free(cf);
 	}
@@ -1771,6 +1815,7 @@ static gboolean
 do_update_push(gpointer data)
 {
 	Tsync_update *su = data;
+	DEBUG_SIG("do_update_push, started, priority %d\n",G_PRIORITY_DEFAULT_IDLE);
 	queue_push(&su->sync->queue_update, su);
 	return FALSE;
 }
@@ -1992,6 +2037,7 @@ static gboolean
 walk_local_directory_push(gpointer data)
 {
 	Tsync_walkdir *swd = data;
+	DEBUG_SIG("walk_local_directory_push, started, priority %d\n",G_PRIORITY_DEFAULT_IDLE);
 	queue_push(&swd->sync->queue_walkdir_local, swd);
 	return FALSE;
 }
@@ -2107,7 +2153,7 @@ file_handle(GFile * uri, Tbfwin * bfwin, gchar * mimetype, gboolean external_inp
 			}
 		}
 	} else {
-		doc_new_from_uri(bfwin, uri, NULL, FALSE, FALSE, -1, -1);
+		doc_new_from_uri(bfwin, uri, NULL, FALSE, FALSE, -1, -1, -1, TRUE, FALSE);
 	}
 #ifdef WIN32
 	if (!mimetype)
