@@ -81,9 +81,9 @@ files_advanced_win_ok_clicked(Tfiles_advanced * tfs)
 	baseuri = g_file_new_for_uri(basedir);
 	content_filter = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(tfs->grep_pattern));
 	if (content_filter && content_filter[0]!='\0')
-		tfs->bfwin->session->searchlist = add_to_history_stringlist(tfs->bfwin->session->searchlist, content_filter, FALSE, TRUE);
+		tfs->bfwin->session->searchlist = add_to_history_stringlist(tfs->bfwin->session->searchlist, content_filter, TRUE);
 	if (extension_filter && extension_filter[0] != '\0')
-		tfs->bfwin->session->filegloblist = add_to_history_stringlist(tfs->bfwin->session->filegloblist, extension_filter,FALSE, TRUE);
+		tfs->bfwin->session->filegloblist = add_to_history_stringlist(tfs->bfwin->session->filegloblist, extension_filter, TRUE);
 
 	retval =
 		open_advanced(tfs->bfwin, baseuri, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tfs->recursive))
@@ -269,7 +269,7 @@ file_open_ok_lcb(GtkDialog * dialog, gint response, Tbfwin * bfwin)
 
 		tmpslist = slist = gtk_file_chooser_get_files(GTK_FILE_CHOOSER(dialog));
 		while (tmpslist) {
-			doc_new_from_uri(bfwin, (GFile *) tmpslist->data, NULL, (slist->next != NULL), FALSE, -1, -1);
+			doc_new_from_uri(bfwin, (GFile *) tmpslist->data, NULL, (slist->next != NULL), FALSE, -1, -1, -1, TRUE, FALSE);
 			g_object_unref((GFile *) tmpslist->data);
 			tmpslist = tmpslist->next;
 		}
@@ -449,7 +449,7 @@ doc_checkNsave_lcb(TcheckNsave_status status, GError * gerror, gpointer data)
 	Tdocsavebackend *dsb = data;
 	Tdocument *doc = dsb->doc;
 	gchar *errmessage;
-	DEBUG_MSG("doc_checkNsave_lcb, doc=%p, status=%d, doc->uri=%p\n", doc, status, doc->uri);
+	DEBUG_MSG("doc_checkNsave_lcb, doc=%p, status=%d, doc->uri=%p, simplesearch_snr3run=%p\n", doc, status, doc->uri, BFWIN(doc->bfwin)->simplesearch_snr3run);
 	switch (status) {
 	case CHECKANDSAVE_ERROR_NOBACKUP:
 		if (main_v->props.backup_abort_action == 0) {
@@ -568,7 +568,7 @@ gtk_label_get_text(GTK_LABEL(doc->tab_label)));
 				parent2 = g_file_get_parent(dsb->fbrefresh_uri);
 				if (!g_file_equal(parent1, parent2)) {
 					/* if they are equal, the directory will be refreshed by the unlink callback */
-					fb2_refresh_dir_from_uri(parent2);
+					filetreemodel_refresh_uri_async(FB2CONFIG(main_v->fb2config)->ftm, parent2);
 				}
 				g_object_unref(parent1);
 				g_object_unref(parent2);
@@ -780,7 +780,7 @@ doc_save_backend(Tdocument * doc, Tdocsave_mode savemode, gboolean close_doc,
 		gchar *newfilename, *curi, *dialogtext;
 		const gchar *gui_name = gtk_label_get_text(GTK_LABEL(doc->tab_label));
 		DEBUG_MSG("doc_save_backend, no uri (doc->uri=%p), or saveas/copy/move (savemode=%d)\n", doc->uri, savemode);
-		curi = doc->uri ? g_file_get_uri(doc->uri) : NULL;
+		curi = doc->uri ? g_file_get_uri(doc->uri) : g_strdup(gui_name); /* SaveFile dialog shows weird behavior if NULL is passed as suggested filename */
 		if (savemode == docsave_normal) {
 			dialogtext = g_strdup(_("Save"));
 		} else if (savemode == docsave_saveas){
@@ -806,10 +806,8 @@ doc_save_backend(Tdocument * doc, Tdocsave_mode savemode, gboolean close_doc,
 					dsb->unlink_uri = doc->uri;	/* unlink and refresh this uri later */
 					g_object_ref(dsb->unlink_uri);
 				}
-				g_object_unref(doc->uri);
 			}
-			doc->uri = dest_uri;
-			g_object_ref(doc->uri);
+			doc_set_uri(doc, dest_uri, FALSE);
 			dest_finfo = doc->fileinfo;
 		}
 		DEBUG_MSG("doc_save_backend, newfilename=%s, dest_uri=%p, doc->uri=%p\n",newfilename, dest_uri, doc->uri);
@@ -821,9 +819,6 @@ doc_save_backend(Tdocument * doc, Tdocsave_mode savemode, gboolean close_doc,
 		g_object_ref(dest_uri);
 	}
 	DEBUG_MSG("doc_save_backend, dest_uri=%p\n",dest_uri);
-	if (doc->uri && (savemode == docsave_move || savemode==docsave_saveas)) {
-		bmark_doc_renamed(BFWIN(doc->bfwin), doc);
-	}
 	if ((firstsave || savemode != docsave_normal)&& dest_uri) {
 		dsb->fbrefresh_uri = dest_uri;	/* refresh this uri later */
 		g_object_ref(dest_uri);
@@ -859,7 +854,7 @@ glib_major_version, glib_minor_version, glib_micro_version);
 	if (firstsave || savemode == docsave_saveas || savemode == docsave_move) {
 		doc->readonly = FALSE;
 		doc_reset_filetype(doc, doc->uri, buffer->data, strlen(buffer->data));
-		doc_set_title(doc);
+		doc_set_title(doc, NULL);
 		doc_force_activate(doc);
 	}
 	g_object_unref(dest_uri);
@@ -955,18 +950,6 @@ file_save_all_cb(GtkWidget * widget, Tbfwin * bfwin)
 	}
 }
 
-void
-doc_save_all_close(Tbfwin * bfwin)
-{
-	GList *tmplist = g_list_first(bfwin->documentlist);
-	while (tmplist) {
-		Tdocument *tmpdoc = (Tdocument *) tmplist->data;
-		doc_save_backend(tmpdoc, docsave_normal, TRUE, TRUE);
-		tmplist = g_list_next(tmplist);
-	}
-}
-
-
 gint
 doc_modified_dialog(Tdocument * doc)
 {
@@ -981,6 +964,21 @@ doc_modified_dialog(Tdocument * doc)
 	return retval;
 }
 
+gint
+project_not_found_dialog(Tbfwin * bfwin, GFile * uri)
+{
+	gchar *tmpstr;
+	gint retval;
+	const gchar *buttons[] = { GTK_STOCK_CANCEL, _("_Reopen"), NULL };
+	gchar *path = g_file_get_parse_name(uri);
+	tmpstr = g_strdup_printf(_("Project \"%s\" failed to load"), path);
+	retval =	message_dialog_new_multi(bfwin->main_window, GTK_MESSAGE_WARNING, buttons, tmpstr,
+										 _("Do you want to reopen project from another location?"));
+	g_free(tmpstr);
+	g_free(path);
+	return retval;
+}
+
 Tclose_mode
 multiple_files_modified_dialog(Tbfwin * bfwin)
 {
@@ -1085,6 +1083,32 @@ doc_close_single_backend(Tdocument * doc, gboolean delay_activate, gboolean clos
 	return TRUE;
 }
 
+void
+doc_save_all_close(Tbfwin * bfwin)
+{
+	GList *tmplist, *duplist;
+	Tdocument *tmpdoc;
+	duplist = g_list_copy(bfwin->documentlist); /* Copy ducumentlist first, since using just tmplist causes unexpected behavior as docs are destroyed */
+	tmplist = g_list_first(duplist);
+	while (tmplist) {
+		Tdocument *tmpdoc = (Tdocument *) tmplist->data;
+#ifdef MAC_INTEGRATION
+		if (tmpdoc->uri == NULL && main_v->osx_status == 1 ) { /* if osx app is terminating we do not save untitled files that does not have uri */
+		DEBUG_MSG("bfwin_osx_terminate_event, closing untitled document\n");
+		tmpdoc->modified = FALSE;
+		doc_close_single_backend(tmpdoc, TRUE, TRUE); }
+		else {
+		DEBUG_MSG("bfwin_osx_terminate_event, saving document\n");
+		doc_save_backend(tmpdoc, docsave_normal, TRUE, TRUE);
+		}
+#else
+		doc_save_backend(tmpdoc, docsave_normal, TRUE, TRUE);
+#endif
+		tmplist = g_list_next(tmplist);
+	}
+	g_list_free(duplist);
+}
+
 /**
  * file_close_cb:
  * @widget: unused #GtkWidget
