@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * gtk_easy.c
  *
- * Copyright (C) 1999-2011 Olivier Sessink
+ * Copyright (C) 1999-2012 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* #define DEBUG */
+/*#define DEBUG*/
 
 #include <gtk/gtk.h>
 #include <stdlib.h>				/* strtod() */
@@ -30,7 +30,7 @@
 #include "bfwin.h"
 #include "pixmap.h"
 #include "dialog_utils.h"
-
+#include "filefilter.h"
 
 #ifdef WIN32
 #define DIRSTR "\\"
@@ -42,9 +42,9 @@
 
 /**
  * flush_queue:
- * 
+ *
  * handles the gtk and gdk queue
- * 
+ *
  * Return value: void
  **/
 void
@@ -58,9 +58,9 @@ flush_queue(void)
 /*
  * this function creates a GtkListStore with numcols columns, where numcols >= 2
  * every row will be filled from the data pointer of the list, which should be an array
- * every column is filled with the entries in the array, except the last one, that 
+ * every column is filled with the entries in the array, except the last one, that
  * will contain the array itself.
- * 
+ *
  * the last column will thus always contain a pointer to the array
  *
  */
@@ -116,9 +116,19 @@ widget_get_string_size(GtkWidget * widget, gchar * string)
 	gint retval = -1;
 	layout = gtk_widget_create_pango_layout(widget, string);
 	if (layout != NULL) {
+/*		PangoFontDescription *pfd = pango_layout_get_font_description(layout);
+		if (pfd) {
+			gchar *tmp = pango_font_description_to_string(pfd);
+			g_print("widget_get_string_size, widget=%p, use font %s\n",widget, tmp);
+			g_free(tmp);
+		} else {
+			g_print("widget_get_string_size, widget=%p has a NULL font description\n",widget);
+		}*/
 		pango_layout_get_pixel_size(layout, &retval, NULL);
 		g_object_unref(G_OBJECT(layout));
-	}
+	}/* else {
+		g_print("widget_get_string_size, failed to get a pango layout for widget %p\n",widget);
+	}*/
 	return retval;
 }
 
@@ -135,9 +145,9 @@ widget_set_visible(GtkWidget * widget, gboolean visible)
 /**
  * window_destroy:
  * @windowname: a #GtkWidget pointer to the window
- * 
+ *
  * Remove grab and signals and then destroy window
- * 
+ *
  * Return value: void
  **/
 void
@@ -190,7 +200,7 @@ window_close_by_data_cb(GtkWidget * widget, gpointer data)
  * 	@entry: the button #GtkWidget * to read
  *
  * 	free the old string in **config_var, and replaces it with the new string from button label
- *   I assume that button is destroyed, so I create copy of a label 
+ *   I assume that button is destroyed, so I create copy of a label
  *
  * Return value:	void
  */
@@ -245,7 +255,7 @@ string_apply(gchar ** config_var, GtkWidget * widget)
 		/*tmpstring = gdk_color_to_string(&color);*/
 	}
 #if GTK_CHECK_VERSION(2,24,0)
-	else if (GTK_IS_COMBO_BOX_TEXT(widget)) 
+	else if (GTK_IS_COMBO_BOX_TEXT(widget))
 #else
 	else if (GTK_IS_COMBO_BOX(widget))
 #endif
@@ -319,41 +329,69 @@ boxed_combobox_with_popdown(const gchar * setstring, const GList * which_list, g
 	gtk_box_pack_start(GTK_BOX(box), returnwidget, TRUE, TRUE, 3);
 	return returnwidget;
 }
+void
+combobox_empty(GtkWidget *combobox)
+{
+#if GTK_CHECK_VERSION(3,0,0)
+	gtk_combo_box_text_remove_all(GTK_COMBO_BOX_TEXT(combobox));
+#else
+	gint num;
+	GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(combobox));
+	if (!model)
+		return;
+	num = gtk_tree_model_iter_n_children(model, NULL);
+	while (num > 0) {
+		gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(combobox), 0);
+		num--;
+	}
+#endif
+}
+
+void
+combobox_fill(GtkWidget *combobox, const gchar * setstring, const GList * which_list)
+{
+	GList *tmplist;
+	gint activenum = -1, i = 0;
+	for (tmplist = g_list_last((GList *)which_list); tmplist; tmplist = g_list_previous(tmplist)) {
+		if (tmplist->data) {
+			DEBUG_MSG("combobox_fill, prepend %s\n",(gchar *)tmplist->data);
+			gtk_combo_box_text_prepend_text(GTK_COMBO_BOX_TEXT(combobox), tmplist->data);
+			if (setstring && g_strcmp0(tmplist->data, setstring) == 0) {
+				activenum = i;
+				DEBUG_MSG("combobox_fill, %s has got activenum=%d\n",(gchar *)tmplist->data, activenum);
+			}
+			i++;
+		}
+	}
+	/* if we PREpend, the activenum is the number starting from the end!! */
+	if (activenum != -1)
+		activenum=(i-1)-activenum;
+	if (setstring) {
+		DEBUG_MSG("combobox_fill, setstring=%s, activenum=%d\n",setstring, activenum);
+		if (activenum == -1) {
+			gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combobox), setstring);
+			gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), i);
+		} else {
+			gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), activenum);
+		}
+	} else {
+		/* for append this should be set to i-1, for prepend set to 0 */
+		gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), 0);
+	}
+}
 
 /* if setstring is NULL the first item will be set as default, if setstring is "" an empty item will be set active */
 GtkWidget *
 combobox_with_popdown(const gchar * setstring, const GList * which_list, gboolean editable)
 {
-	GtkWidget *child, *returnwidget;
-	GList *tmplist;
-	gint activenum = -1, index = 0;
+	GtkWidget *returnwidget;
 	if (editable)
 		returnwidget = gtk_combo_box_text_new_with_entry();
 	else
 		returnwidget = gtk_combo_box_text_new();
-	for (tmplist = g_list_last((GList *)which_list); tmplist; tmplist = g_list_previous(tmplist)) {
-		if (tmplist->data) {
-			gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(returnwidget), tmplist->data);
-			if (setstring && g_strcmp0(tmplist->data, setstring) == 0) {
-				activenum = index;
-			}
-			index++;
-		}
-	}
-	/*g_print("setstring='%s', activenum=%d, index=%d\n",setstring,activenum,index);*/
-	if (setstring) {
-		if (activenum == -1) {
-			gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(returnwidget), setstring);
-			gtk_combo_box_set_active(GTK_COMBO_BOX(returnwidget), index);
-		} else {
-			gtk_combo_box_set_active(GTK_COMBO_BOX(returnwidget), activenum);
-		}
-	} else {
-		gtk_combo_box_set_active(GTK_COMBO_BOX(returnwidget), index-1);
-	}
+	combobox_fill(returnwidget, setstring, which_list);
 	if (editable) {
-		child = gtk_bin_get_child(GTK_BIN(returnwidget));
-		gtk_entry_set_activates_default(GTK_ENTRY(child), TRUE);
+		gtk_entry_set_activates_default(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(returnwidget))), TRUE);
 	}
 	return returnwidget;
 }
@@ -403,7 +441,7 @@ boxed_checkbut_with_value(gchar * labeltext, gint which_config_int, GtkWidget *
  * 	@enabled: #gint if the radiobutton is enabled or not
  * @prevbut: #GtkRadioButton* pointer to the first button or NULL if none
  *
- * 	Create new radiobutton with some value, and if there is a first button 
+ * 	Create new radiobutton with some value, and if there is a first button
  * for this group, add this button to that group
  *
  * Return value: #GtkWidget* pointer to the new radiobutton widget
@@ -429,7 +467,7 @@ radiobut_with_value(gchar * labeltext, gint enabled, GtkRadioButton * prevbut)
  * @prevbut: #GtkRadioButton* pointer to the first button or NULL if none
  * @box: the #GtkWidget* box widget to add the entry to
  *
- * 	Create new radiobutton with some value, and if there is a first button 
+ * 	Create new radiobutton with some value, and if there is a first button
  * for this group, add this button to that group, and this button to box
  *
  * Return value: #GtkWidget* pointer to the new radiobutton widget
@@ -469,7 +507,7 @@ is_int(gfloat testval)
  *
  * Return value: #GtkWidget* pointer to the new spinbutton widget
  */
- 
+
 static gboolean
 spinbut_output(GtkSpinButton *spin,gpointer data)
 {
@@ -512,7 +550,7 @@ spinbut_with_value(gchar * value, gfloat lower, gfloat upper, gfloat step_increm
 	if (!value || value[0]=='\0') {
 		gtk_entry_set_text(GTK_ENTRY(returnwidget), value);
 	}*/
-	
+
 	GtkWidget *returnwidget;
 	gchar *endptr;
 	double testval;
@@ -564,7 +602,7 @@ window_with_title(const gchar * title, GtkWindowPosition position, gint borderwi
 	return returnwidget;
 }
 
-/* GtkWindowPosition can be 
+/* GtkWindowPosition can be
 GTK_WIN_POS_NONE
 GTK_WIN_POS_CENTER
 GTK_WIN_POS_MOUSE */
@@ -604,7 +642,7 @@ window_delete_on_escape(GtkWindow * window)
  * and if needed set a callback so the window will be closed on escape press
  * and set it if needed transient for another window
  *
- * the close_func should have the form 
+ * the close_func should have the form
  * void close_func(GtkObject *object,gpointer user_data);
  *
  * Return value: #GtkWidget* pointer to created window
@@ -647,7 +685,7 @@ window_full2(const gchar * title, GtkWindowPosition position, gint borderwidth,
  * 	@contents: #gchar* the initial contents of the textbox
  * 	@wrapmode: #GtkWrapMode the wrapmode for the TextView GTK_WRAP_NONE,GTK_WRAP_CHAR,GTK_WRAP_WORD
  *
- * creates a textbuffer, a textview widget (stored in the textview pointer), 
+ * creates a textbuffer, a textview widget (stored in the textview pointer),
  * puts that inside a scrolwin, adds the content and sets the size and returns
  * the scrolwin
  *
@@ -697,7 +735,12 @@ apply_font_style(GtkWidget * this_widget, gchar * fontstring)
 	if (fontstring && fontstring[0] != '\0') {
 		PangoFontDescription *font_desc;
 		font_desc = pango_font_description_from_string(fontstring);
+		DEBUG_MSG("apply_font_style, set font %s (desc=%p) to widget %p\n",fontstring,font_desc,this_widget);
+#if GTK_CHECK_VERSION(3,0,0)
+		gtk_widget_override_font(this_widget, font_desc);
+#else
 		gtk_widget_modify_font(this_widget, font_desc);
+#endif
 		pango_font_description_free(font_desc);
 	}
 	return this_widget;
@@ -747,7 +790,7 @@ bf_label_with_markup(const gchar * labeltext)
  * @left_attach: #guint column number to attach the left side of the label to
  * @right_atach: #guint column number to attach the right side of a label to
  * @top_attach: #guint row number to attach the top of a label to
- * @bottom_attach: #guint row number to attach the bottom of a label to 	
+ * @bottom_attach: #guint row number to attach the bottom of a label to
  *
  *	create a label with markup, align it, and attach it to a table
  * using the table defaults
@@ -1073,7 +1116,7 @@ file_but_clicked_lcb(GtkWidget * widget, Tfilebut * fb)
 
 	tmpstring = run_file_select_dialog(GTK_WINDOW(gtk_widget_get_toplevel(fb->entry))
 							, setfile
-							, relativeto, 
+							, relativeto,
 							fb->chooseraction);
 	if (tmpstring) {
 		gtk_entry_set_text(GTK_ENTRY(fb->entry), tmpstring);
@@ -1105,7 +1148,7 @@ file_but_clicked_lcb(GtkWidget * widget, Tfilebut * fb)
 		if (!fb->fullpath && fb->bfwin && fb->bfwin->current_document) {
 			if (fb->bfwin->current_document->uri != NULL) {
 				gchar *doc_curi;
-				/ * the function g_file_get_relative_path cannot create links that don't 
+				/ * the function g_file_get_relative_path cannot create links that don't
 				   have the same prefix (relative links with ../../) * /
 				doc_curi = g_file_get_uri(fb->bfwin->current_document->uri);
 				tmp2string = create_relative_link_to(doc_curi, tmpstring);
@@ -1122,7 +1165,7 @@ file_but_clicked_lcb(GtkWidget * widget, Tfilebut * fb)
 		gtk_entry_set_text(GTK_ENTRY(fb->entry), tmpstring);
 
 	}*/
-	
+
 }
 
 static void
@@ -1211,22 +1254,28 @@ is_separator(GtkTreeModel * model, GtkTreeIter * iter, gpointer data)
 static gboolean
 file_chooser_custom_filter_func(GtkFileFilterInfo * filter_info, gpointer data)
 {
-	gint ret;
+	/*gint ret;*/
 	Tfchooser_filter *cf = data;
-	if (!filter_info->display_name || filter_info->display_name[0] == '\0')
+	if (!filter_info->display_name || filter_info->display_name[0] == '\0') {
+		DEBUG_MSG("file_chooser_custom_filter_func, no display_name, don't display, return FALSE\n");
 		return FALSE;			/* error condition ?? */
+	}
 	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cf->show_backup))) {
 		gint namelen = strlen(filter_info->display_name);
-		if (filter_info->display_name[namelen - 1] == '~')
+		if (filter_info->display_name[namelen - 1] == '~') {
+			DEBUG_MSG("file_chooser_custom_filter_func, %s is backup file, return FALSE\n", filter_info->display_name);
 			return FALSE;
+		}
 	}
-	if (cf->filter == NULL)
+	/*if (cf->filter == NULL)
 		return TRUE;
 	if (!cf->filter->filetypes)
-		return !cf->filter->mode;
-
-	ret = GPOINTER_TO_INT(g_hash_table_lookup(cf->filter->filetypes, filter_info->mime_type));
-	return (ret ? cf->filter->mode : !cf->filter->mode);
+		return !cf->filter->mode;*/
+	DEBUG_MSG("file_chooser_custom_filter_func, calling file_visible_in_filter for %s with mime=%s and filename=%s\n", filter_info->display_name, filter_info->mime_type, filter_info->filename);
+	/* for some reason the GtkFileFilterInfo often has a NULL filename, so we use the display name to filter on */
+	return file_visible_in_filter(cf->filter, filter_info->mime_type, filter_info->display_name);
+	/*ret = GPOINTER_TO_INT(g_hash_table_lookup(cf->filter->filetypes, filter_info->mime_type));
+	return (ret ? cf->filter->mode : !cf->filter->mode);*/
 }
 
 GtkWidget *
@@ -1242,9 +1291,13 @@ file_chooser_dialog(Tbfwin * bfwin, const gchar * title, GtkFileChooserAction ac
 										 GTK_RESPONSE_ACCEPT, NULL);
 	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
 	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), localonly);
-	if (set && strlen(set)) {
+	if (set && set[0] != '\0') {
 		DEBUG_MSG("file_chooser_dialog, set=%s,localonly=%d\n", set, localonly);
-		gtk_file_chooser_set_uri(GTK_FILE_CHOOSER(dialog), set);
+		if (set[0] == '/' || strchr(set, ':')!=NULL) {
+			gtk_file_chooser_set_uri(GTK_FILE_CHOOSER(dialog), set);
+		} else {
+			gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), set);
+		}
 	} else if (!localonly && bfwin) {	/* localonly is used for the project files */
 		if (action == GTK_FILE_CHOOSER_ACTION_SAVE) {
 			DEBUG_MSG("file_chooser_dialog, opendir=%s, savedir=%s\n", bfwin->session->opendir,
@@ -1561,3 +1614,4 @@ color_but_new2(GtkWidget * entry)
 	color_entry_changed_lcb(entry, cbut);
 	return cbut;
 }
+
