@@ -1,11 +1,11 @@
 /* Bluefish HTML Editor
  * msg_queue.h - message queue handling
  *
- * Copyright (C) 2003 Olivier Sessink
+ * Copyright (C) 2003-2009 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,9 +14,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-void msg_queue_start(GList * filenames, GList *projectfiles, gboolean in_new_window);
+#ifndef __MSG_QUEUE_H_
+#define __MSG_QUEUE_H_
+
+void msg_queue_start(GList * filenames, gboolean in_new_window);
+void msg_queue_check_server(gboolean last_check);
 void msg_queue_cleanup(void);
+
+#endif /* __MSG_QUEUE_H_ */
