@@ -47,6 +47,14 @@ bluefish_url_show(const gchar * url)
 	if ((int) ShellExecute(NULL, "open", url, NULL, NULL, SW_SHOWNORMAL) <= 32) {
 		g_warning("Failed trying to launch URL in about dialog");
 	}
+#else
+#ifdef MAC_INTEGRATION
+	GString *string;
+	string = g_string_new("open ");
+	string = g_string_append(string, url);
+	g_print("bluefish_url_show, launching uri=%s\n", string->str);
+	system(string->str);
+	g_string_free(string, TRUE);
 #else
 	GError *error = NULL;
 
@@ -56,6 +64,7 @@ bluefish_url_show(const gchar * url)
 		g_error_free(error);
 	}
 #endif
+#endif
 }
 
 static void
@@ -162,6 +171,17 @@ about_activate_url(GtkAboutDialog * about, const gchar * url, gpointer data)
 }
 #endif
 
+#ifdef MAC_INTEGRATION
+static gboolean
+activate_link_lcb (GtkWidget   *dialog,
+               const gchar *url,
+               gpointer     data)
+{
+  bluefish_url_show(url);
+  return TRUE;
+}
+#endif
+
 static void
 about_dialog_create(GtkAction * action, gpointer user_data)
 {
@@ -217,7 +237,7 @@ about_dialog_create(GtkAction * action, gpointer user_data)
 		NULL
 	};
 
-	const gchar *copyright = "Copyright \xc2\xa9 1998-2012 Olivier Sessink and others.\n";
+	const gchar *copyright = "Copyright \xc2\xa9 1998-2013 Olivier Sessink and others.\n";
 
 	/* wrap the license here,
 	 * the "wrap-license" property is only available with GTK >= 2.8
@@ -234,7 +254,7 @@ about_dialog_create(GtkAction * action, gpointer user_data)
 		"GNU General Public License for more details.\n"
 		"\n"
 		"You should have received a copy of the GNU General Public License "
-		"along with this program.  If not, see <http://www.gnu.org/licenses/>.";
+		"along with this program.  If not, see http://www.gnu.org/licenses/ .";
 
 	const gchar *comments =
 		_
@@ -262,8 +282,8 @@ about_dialog_create(GtkAction * action, gpointer user_data)
 #if !GTK_CHECK_VERSION(3, 0, 0)
 	gtk_about_dialog_set_url_hook(about_activate_url, NULL, NULL);
 #endif /* gtk3 */
-
-	gtk_show_about_dialog(GTK_WINDOW(bfwin->main_window), "logo", logo, "name", PACKAGE,
+#ifndef MAC_INTEGRATION
+	gtk_show_about_dialog(GTK_WINDOW(bfwin->main_window), "logo", logo, "name", PACKAGE, 
 #ifdef SVN_REVISION
 						  "version", VERSION " rev" SVN_REVISION,
 #else	/* SVN_REVISION */
@@ -282,6 +302,38 @@ about_dialog_create(GtkAction * action, gpointer user_data)
 
 	if (logo)
 		g_object_unref(logo);
+#else
+/* gtk_show_about_dialog hides window when it is closed (no other choices). On OSX this hidden dead window can be accessed from WIndow menu, so we have 
+to construct about dialog manually and destroy it after use */
+	GtkWidget *dialog;
+	dialog = gtk_about_dialog_new();
+	gtk_window_set_transient_for(GTK_WINDOW( dialog ), GTK_WINDOW(bfwin->main_window));
+	gtk_window_set_destroy_with_parent (GTK_WINDOW(dialog), TRUE);
+	g_signal_connect (dialog, "activate-link", G_CALLBACK (activate_link_lcb), NULL);
+         /* Set it's properties */
+	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(dialog), "Bluefish" );
+	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dialog), 
+#ifdef SVN_REVISION
+						  VERSION " rev" SVN_REVISION
+#else	/* SVN_REVISION */
+					  VERSION
+#endif	/* SVN_REVISION */	
+	 );
+	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(dialog), copyright);
+	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(dialog), "http://bluefish.openoffice.nl" );
+	gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(dialog), authors);
+	gtk_about_dialog_set_artists(GTK_ABOUT_DIALOG(dialog), artists);
+	gtk_about_dialog_set_documenters(GTK_ABOUT_DIALOG(dialog), documenters);
+	gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(dialog), license);
+	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(dialog), comments);
+	gtk_about_dialog_set_translator_credits(GTK_ABOUT_DIALOG(dialog), translator_credits);
+	gtk_about_dialog_set_wrap_license(GTK_ABOUT_DIALOG(dialog), TRUE);
+	gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(dialog), logo);
+	g_object_unref(logo), logo=NULL;
+    /* Run dialog and destroy it after it returns. */
+    gtk_dialog_run( GTK_DIALOG(dialog) );
+    gtk_widget_destroy(GTK_WIDGET(dialog));
+#endif /* MAC_INTEGRATION */
 }
 
 static void
