@@ -24,6 +24,7 @@
 #include <gmodule.h>
 #include "../plugins.h"
 #include "../rcfile.h"
+#include "../bfwin_uimanager.h" /* bfwin_set_menu_toggle_item() */
 #include "../document.h"
 #include "../gtk_easy.h"
 #include "../stringlist.h"
@@ -72,7 +73,7 @@ htmlbar_doc_view_button_press(GtkWidget * widget, GdkEventButton * bevent, Tdocu
 static void
 htmlbar_sidepanel_initgui(Tbfwin *bfwin)
 {
-	
+
 	if (htmlbar_v.in_sidepanel && bfwin->leftpanel_notebook) {
 		Thtmlbarwin *hbw;
 		Thtmlbarsession *hbs;
@@ -85,18 +86,21 @@ htmlbar_sidepanel_initgui(Tbfwin *bfwin)
 			g_warning("htmlbar, no hbw/hbs when creating sidebar, please report");
 			return;
 		}
+
+		image = gtk_image_new_from_stock(BF_STOCK_BROWSER_PREVIEW, GTK_ICON_SIZE_LARGE_TOOLBAR);
+		html_notebook = htmlbar_toolbar_create(hbw, hbs);
+		DEBUG_MSG("htmlbar_sidepanel_initgui, html_notebook=%p\n",html_notebook);
+		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(html_notebook), GTK_POS_LEFT);
+		gtk_widget_show_all(html_notebook);
+		DEBUG_MSG("htmlbar_sidepanel_initgui, append html_notebook=%p to leftpanel_notebook %p and image %p\n",html_notebook, bfwin->leftpanel_notebook, image);
+		gtk_notebook_append_page_menu(GTK_NOTEBOOK(bfwin->leftpanel_notebook),html_notebook,image,gtk_label_new(_("Htmlbar")));
 		if (hbw->handlebox) {
+			DEBUG_MSG("htmlbar_sidepanel_initgui, destroy hbw->handlebox %p\n",hbw->handlebox);
 			gtk_widget_destroy(hbw->handlebox);
 			hbw->handlebox = NULL;
 		}
-		
-		image = gtk_image_new_from_stock(BF_STOCK_BROWSER_PREVIEW, GTK_ICON_SIZE_LARGE_TOOLBAR);
-		html_notebook = htmlbar_toolbar_create(hbw, hbs);
-		
-		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(html_notebook), GTK_POS_LEFT);
-		gtk_widget_show_all(html_notebook);
-		gtk_notebook_append_page_menu(GTK_NOTEBOOK(bfwin->leftpanel_notebook),html_notebook,image,gtk_label_new(_("Htmlbar")));
-	} 
+
+	}
 }
 
 /*static void
@@ -183,6 +187,7 @@ htmlbar_enforce_session(Tbfwin * bfwin)
 	}
 	if (!htmlbar_v.in_sidepanel) {
 		htmlbar_toolbar_show(hbw, hbs, hbs->view_htmlbar);
+		bfwin_set_menu_toggle_item(hbw->actiongroup, "ViewHTMLToolbar", hbs->view_htmlbar);
 	}
 }
 
