@@ -18,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-
+/*#define DEBUG*/
 
 #include "htmlbar_uimanager.h"
 #include "../bluefish.h"
@@ -38,8 +38,6 @@
 #include "wizards.h"
 
 
-/*#define DEBUG*/
-
 #define HTMLBAR_PIXMAP_DIR PKGDATADIR"/plugins/htmlbar/pixmaps/"
 #define HTMLBAR_MENU_UI	   PKGDATADIR"/plugins/htmlbar/ui/htmlbar_menu_ui.xml"
 /*#define HTMLBAR_TOOLBAR_UI PKGDATADIR"/plugins/htmlbar/ui/htmlbar_toolbar_ui.xml"*/ /* menu and toolbar are merged into one file now */
@@ -366,7 +364,7 @@ static void
 htmlbar_insert_break_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml == 1 ? cap("<br />") : cap("<br>"), NULL);
+						   get_curlang_option_value(BFWIN(user_data), self_close_singleton_tags) ? cap("<br />") : cap("<br>"), NULL);
 }
 
 static void
@@ -388,7 +386,7 @@ static void
 htmlbar_insert_clearall_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml == 1 ? cap("<BR CLEAR=\"ALL\" />") : cap("<BR CLEAR=\"ALL\">"),
+						   get_curlang_option_value(BFWIN(user_data), self_close_singleton_tags) ? cap("<BR CLEAR=\"ALL\" />") : cap("<BR CLEAR=\"ALL\">"),
 						   NULL);
 }
 
@@ -446,7 +444,7 @@ htmlbar_insert_font_minus_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
 						   main_v->props.xhtml ==
-						   1 ? cap("<SPAN STYLE=\"font-size: larger;\">") : cap("<FONT SIZE=\"+1\">")
+						   1 ? cap("<SPAN STYLE=\"font-size: smaller;\">") : cap("<FONT SIZE=\"-1\">")
 						   , main_v->props.xhtml == 1 ? cap("</SPAN>") : cap("</FONT>"));
 }
 
@@ -455,7 +453,7 @@ htmlbar_insert_font_plus_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
 						   main_v->props.xhtml ==
-						   1 ? cap("<SPAN STYLE=\"font-size: smaller;\">") : cap("<FONT SIZE=\"-1\">")
+						   1 ? cap("<SPAN STYLE=\"font-size: larger;\">") : cap("<FONT SIZE=\"+1\">")
 						   , main_v->props.xhtml == 1 ? cap("</SPAN>") : cap("</FONT>"));
 }
 
@@ -487,7 +485,7 @@ static void
 htmlbar_insert_form_input_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml == 1 ? cap("<INPUT />") : cap("<INPUT>"), NULL);
+						   get_curlang_option_value(BFWIN(user_data), self_close_singleton_tags) ? cap("<INPUT />") : cap("<INPUT>"), NULL);
 }
 
 static void
@@ -530,7 +528,7 @@ static void
 htmlbar_insert_form_keygen_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml == 1 ? cap("<KEYGEN />") : cap("<KEYGEN>"), NULL);
+						   get_curlang_option_value(BFWIN(user_data), self_close_singleton_tags) ? cap("<KEYGEN />") : cap("<KEYGEN>"), NULL);
 }
 
 static void
@@ -555,7 +553,7 @@ static void
 htmlbar_insert_frame_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml == 1 ? cap("<FRAME />") : cap("<FRAME>"), NULL);
+						   get_curlang_option_value(BFWIN(user_data), self_close_singleton_tags) ? cap("<FRAME />") : cap("<FRAME>"), NULL);
 }
 
 static void
@@ -569,7 +567,7 @@ htmlbar_insert_generator_meta_tag(GtkAction * action, gpointer user_data)
 {
 	gchar *tmp = g_strconcat(cap("<META NAME=\"Generator\" CONTENT=\""), "Bluefish ", VERSION,
 							 " http://bluefish.openoffice.nl/",
-							 main_v->props.xhtml == 1 ? "\" />\n" : "\">\n", NULL);
+							 get_curlang_option_value(BFWIN(user_data), self_close_singleton_tags) ? "\" />\n" : "\">\n", NULL);
 	doc_insert_two_strings(BFWIN(user_data)->current_document, tmp, NULL);
 	g_free(tmp);
 }
@@ -614,7 +612,7 @@ static void
 htmlbar_insert_image_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml == 1 ? cap("<IMG />") : cap("<IMG>"), NULL);
+						   get_curlang_option_value(BFWIN(user_data), self_close_singleton_tags) ? cap("<IMG />") : cap("<IMG>"), NULL);
 }
 
 static void
@@ -639,7 +637,7 @@ static void
 htmlbar_insert_link_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml == 1 ? cap("<LINK />") : cap("<LINK>"), NULL);
+						   get_curlang_option_value(BFWIN(user_data), self_close_singleton_tags) ? cap("<LINK />") : cap("<LINK>"), NULL);
 }
 
 static void
@@ -772,8 +770,7 @@ static void
 htmlbar_insert_target_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml ==
-						   1 ? cap("<BASE TARGET=\"\" />") : cap("<BASE TARGET=\"\">"), NULL);
+						   get_curlang_option_value(BFWIN(user_data), self_close_singleton_tags) ? cap("<BASE TARGET=\"\" />") : cap("<BASE TARGET=\"\">"), NULL);
 }
 
 static void
@@ -814,18 +811,29 @@ htmlbar_toolbar_show(Thtmlbarwin * hbw, Thtmlbarsession *hbs, gboolean show)
 {
 	if (htmlbar_v.in_sidepanel)
 		return;
-	
+	DEBUG_MSG("htmlbar_toolbar_show, show=%d, handlebox=%p\n",show,hbw->handlebox);
 	if (show) {
 		if (hbw->handlebox) {
 			gtk_widget_show(hbw->handlebox);
 		} else {
 			GtkWidget *html_notebook;
 			html_notebook = htmlbar_toolbar_create(hbw, hbs);
+#if GTK_CHECK_VERSION(3,2,0)
+			/*gtk_notebook_set_tab_pos(GTK_NOTEBOOK(html_notebook), GTK_POS_TOP); this is the default, so why set it?*/ 
+ 			gtk_widget_set_hexpand(GTK_WIDGET(html_notebook),TRUE);
+ 			hbw->handlebox = gtk_event_box_new();
+ 			gtk_widget_set_name(GTK_WIDGET(hbw->handlebox), "html_notebook_event_box");
+			gtk_container_add(GTK_CONTAINER(hbw->handlebox),html_notebook); 
+			gtk_container_add(GTK_CONTAINER(hbw->bfwin->toolbarbox),hbw->handlebox);
+			gtk_widget_show_all(hbw->handlebox);
+#else			
 			hbw->handlebox = gtk_handle_box_new();
 			gtk_container_add(GTK_CONTAINER(hbw->handlebox), html_notebook);
 			gtk_notebook_set_tab_pos(GTK_NOTEBOOK(html_notebook), GTK_POS_TOP);
 			gtk_box_pack_start(GTK_BOX(hbw->bfwin->toolbarbox), hbw->handlebox, FALSE, FALSE, 0);
 			gtk_widget_show_all(hbw->handlebox);
+#endif
+		DEBUG_MSG("htmlbar_toolbar_show, created handlebox=%p\n",hbw->handlebox);
 		}
 	} else {
 		if (hbw->handlebox)
@@ -840,7 +848,7 @@ htmlbar_toolbar_show_toogle(GtkAction * action, gpointer user_data)
 	Thtmlbarsession *hbs;
 
 	hbs = g_hash_table_lookup(htmlbar_v.lookup, hbw->bfwin->session);
-	if (!hbs) 
+	if (!hbs)
 		return;
 	hbs->view_htmlbar = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
 	htmlbar_toolbar_show(hbw, hbs, gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action)));
@@ -1134,7 +1142,7 @@ static const GtkActionEntry htmlbar_actions[] = {
 	{"MiscStyle", BF_STOCK_CSSSTYLE, N_("_Style"), NULL, N_("Style"),
 	 G_CALLBACK(htmlbar_misc_style)},
 	{"Dialogscolumns", BF_STOCK_CSS_COLUMNS, N_("C_olumns..."), NULL, N_("Columns..."),
-	 G_CALLBACK(htmlbar_dialog_columns)},	 
+	 G_CALLBACK(htmlbar_dialog_columns)},
 	{"DialogsLinkStylesheet", BF_STOCK_LINK_STYLESHEET, N_("_Link to Stylesheet..."), NULL,
 	 N_("Link to Stylesheet..."),
 	 G_CALLBACK(htmlbar_dialog_style_link_to)},
@@ -1263,9 +1271,9 @@ htmlbar_quickbar_add_item(Thtmlbarwin * hbw, const gchar *actionname)
 		/*g_print("have child-child %p of type %s\n", tmplist2->data, G_OBJECT_TYPE_NAME(tmplist2->data));*/
 		g_signal_connect(tmplist2->data, "button-press-event", G_CALLBACK(quickbar_button_press_event_lcb), hbw);
 	}
-	g_list_free(children2);	
-	
-	gtk_toolbar_insert(GTK_TOOLBAR(hbw->quickbar_toolbar), GTK_TOOL_ITEM(toolitem), -1);	
+	g_list_free(children2);
+
+	gtk_toolbar_insert(GTK_TOOLBAR(hbw->quickbar_toolbar), GTK_TOOL_ITEM(toolitem), -1);
 }
 
 static void
@@ -1279,7 +1287,7 @@ add_to_quickbar_activate_lcb(GtkMenuItem *m, gpointer data)
 		return;
 	}
 	firstitem = (htmlbar_v.quickbar_items==NULL);
-	
+
 	DEBUG_MSG("add_to_quickbar_activate_lcb, adding %s to quickbar_items\n", (gchar *)data);
 	htmlbar_v.quickbar_items = g_list_append(htmlbar_v.quickbar_items, g_strdup(data));
 	/* now loop over all the windows that have a htmlbar, and add the item */
@@ -1292,7 +1300,7 @@ add_to_quickbar_activate_lcb(GtkMenuItem *m, gpointer data)
 				if (children)
 					gtk_container_remove(GTK_CONTAINER(hbw->quickbar_toolbar), children->data);
 			}
-			
+
 			htmlbar_quickbar_add_item(hbw, (gchar *)data);
 		}
 	}
@@ -1319,8 +1327,7 @@ static void
 remove_from_quickbar_activate_lcb(GtkMenuItem *m, gpointer data)
 {
 	GList *tmplist;
-	DEBUG_MSG("remove_from_quickbar_activate_lcb, removing %s from quickbar_items\n", (gchar *)data);
-	htmlbar_v.quickbar_items = remove_from_stringlist(htmlbar_v.quickbar_items, data);
+
 	/* now loop over all the windows that have a htmlbar, and remove the item */
 	for (tmplist=g_list_first(main_v->bfwinlist);tmplist;tmplist=g_list_next(tmplist)) {
 		Thtmlbarwin * hbw;
@@ -1329,6 +1336,8 @@ remove_from_quickbar_activate_lcb(GtkMenuItem *m, gpointer data)
 			htmlbar_quickbar_remove_item(hbw, (const gchar *)data);
 		}
 	}
+	DEBUG_MSG("remove_from_quickbar_activate_lcb, removing %s from quickbar_items\n", (gchar *)data);
+	htmlbar_v.quickbar_items = remove_from_stringlist(htmlbar_v.quickbar_items, data);
 }
 
 static GtkWidget *
@@ -1358,7 +1367,7 @@ toolbar_button_press_event_lcb(GtkWidget *widget,GdkEvent  *event,gpointer   use
 		if (!action)
 			return FALSE;
 		DEBUG_MSG("toolbar_button_press_event_lcb, add action %s \n", gtk_action_get_name(action));
-		
+
 		menu = quickbar_create_popup_menu(TRUE, gtk_action_get_name(action));
 		gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,3,event->button.time);
 /*		htmlbar_v.quickbar_items = g_list_prepend(htmlbar_v.quickbar_items, g_strdup(gtk_action_get_name(action)));*/
@@ -1375,7 +1384,7 @@ htmlbar_load_quickbar(Thtmlbarwin * hbw, GtkWidget *toolbar)
 		GtkWidget *label;
 		GtkToolItem *ti = gtk_tool_item_new();
 		label = gtk_label_new(_("Right click any html toolbar button to add it to the Quickbar."));
-		/*gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);*/
+		gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
 		gtk_container_add(GTK_CONTAINER(ti), label);
 		gtk_toolbar_insert(GTK_TOOLBAR(toolbar),ti,0);
 	}
@@ -1391,7 +1400,7 @@ setup_items_for_quickbar(Thtmlbarwin * hbw, GtkWidget *toolbar)
 	DEBUG_MSG("setup_items_for_quickbar, about to connect signals to toolbar buttons\n");
 	children = gtk_container_get_children(GTK_CONTAINER(toolbar));
 	for (tmplist=g_list_first(children);tmplist;tmplist=g_list_next(tmplist)) {
-/*		g_print("have child %p of type %s\n", tmplist->data, G_OBJECT_TYPE_NAME(tmplist->data));*/
+		/*g_print("have child %p of type %s\n", tmplist->data, G_OBJECT_TYPE_NAME(tmplist->data));*/
 		GtkAction *action = gtk_activatable_get_related_action(tmplist->data);
 		if (action) {
 			children2 = gtk_container_get_children(GTK_CONTAINER(tmplist->data));
@@ -1399,7 +1408,7 @@ setup_items_for_quickbar(Thtmlbarwin * hbw, GtkWidget *toolbar)
 				/*g_print("have child-child %p of type %s\n", tmplist2->data, G_OBJECT_TYPE_NAME(tmplist2->data));*/
 				g_signal_connect(tmplist2->data, "button-press-event", G_CALLBACK(toolbar_button_press_event_lcb), hbw);
 			}
-			g_list_free(children2);			
+			g_list_free(children2);
 		}
 		/*g_print("action=%p %s\n", action, action? gtk_action_get_name(action): "NULL");*/
 	}
@@ -1421,17 +1430,34 @@ notebook_switch_page_lcb(GtkNotebook *notebook,GtkWidget   *page,guint        pa
 static GtkWidget *new_html_subtoolbar(Thtmlbarwin * hbw, GtkWidget *html_notebook, GtkWidget *toolbar, const gchar *labeltext)
 {
 	GtkWidget *label;
+	GtkWidget *parent;
+	parent = gtk_widget_get_parent(toolbar);
+	DEBUG_MSG("new_html_subtoolbar, setup toolbar(parent=%p) %p for %s\n",parent, toolbar,labeltext);
+	if (parent) {
+		g_object_ref(toolbar);
+		gtk_container_remove(GTK_CONTAINER(parent), toolbar);
+	}
+	
 	
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
+#ifdef MAC_INTEGRATION
+#if GTK_CHECK_VERSION(3,4,0)
+	gtk_toolbar_set_icon_size (GTK_TOOLBAR(toolbar), GTK_ICON_SIZE_SMALL_TOOLBAR);
+#endif
+#endif
 	if (htmlbar_v.in_sidepanel) {
+		DEBUG_MSG("new_html_subtoolbar, setup vertical orientation\n");
 		gtk_orientable_set_orientation(GTK_ORIENTABLE(toolbar), GTK_ORIENTATION_VERTICAL);
 	}
 	label = gtk_label_new(labeltext);
-/*	gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);*/
+	DEBUG_MSG("new_html_subtoolbar, append toolbar to html_notebook\n");
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, label);
 	gtk_container_child_set(GTK_CONTAINER(html_notebook), label, "tab-fill", TRUE, "tab-expand", TRUE, NULL);
-
-	return toolbar;	
+	DEBUG_MSG("new_html_subtoolbar, return toolbar=%p for %s\n", toolbar, labeltext);
+	if (parent) {
+		g_object_unref(toolbar);
+	}
+	return toolbar;
 }
 
 GtkWidget *
@@ -1443,16 +1469,18 @@ htmlbar_toolbar_create(Thtmlbarwin * hbw, Thtmlbarsession *hbs)
 	GtkWidget *toolbar;
 
 	html_notebook = gtk_notebook_new();
-	
+	DEBUG_MSG("htmlbar_toolbar_create, created notebook %p\n",html_notebook);
 	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(html_notebook), TRUE);
 	gtk_notebook_set_show_border(GTK_NOTEBOOK(html_notebook), TRUE);
 	gtk_notebook_set_scrollable(GTK_NOTEBOOK(html_notebook), TRUE);
 
 	hbw->quickbar_toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_toolbar_new(), _("Quickbar"));
 	htmlbar_load_quickbar(hbw, hbw->quickbar_toolbar);
-
+	DEBUG_MSG("quickbar done\n");
 	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLStandardToolbar"), _("Standard"));
+	DEBUG_MSG("standard created\n");
 	setup_items_for_quickbar(hbw, toolbar);
+	DEBUG_MSG("standard setup done\n");
 
 	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLHTML5Toolbar"), _("HTML 5"));
 	setup_items_for_quickbar(hbw, toolbar);
@@ -1462,10 +1490,10 @@ htmlbar_toolbar_create(Thtmlbarwin * hbw, Thtmlbarsession *hbs)
 
 	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLTablesToolbar"), _("Tables"));
 	setup_items_for_quickbar(hbw, toolbar);
-	
+
 	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLListToolbar"), _("List"));
 	setup_items_for_quickbar(hbw, toolbar);
-	
+
 	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLCSSToolbar"), _("CSS"));
 	setup_items_for_quickbar(hbw, toolbar);
 
@@ -1477,11 +1505,11 @@ htmlbar_toolbar_create(Thtmlbarwin * hbw, Thtmlbarsession *hbs)
 
 	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFramesToolbar"), _("Frames"));
 	setup_items_for_quickbar(hbw, toolbar);
-	
+
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(html_notebook), hbs->notebooktab);
 	DEBUG_MSG("htmlbar htmlbar_toolbar_create, make page %d active\n", hbs->notebooktab);
 	g_signal_connect(G_OBJECT(html_notebook), "switch-page", G_CALLBACK(notebook_switch_page_lcb), hbw);
-	
+
 	return html_notebook;
 }
 
@@ -1618,7 +1646,7 @@ htmlbar_register_stock_icons(void)
 
 	for (i = 0; i < G_N_ELEMENTS(htmlbar_stock_icons); i++) {
 		GdkPixbuf *pixbuf;
-		
+
 		pixbuf = gdk_pixbuf_new_from_inline(-1,htmlbar_stock_icons[i].data,FALSE,NULL);
 		icon_set = gtk_icon_set_new_from_pixbuf(pixbuf);
 		g_object_unref(pixbuf);
