@@ -123,18 +123,29 @@ project_is_open(GFile * uri)
 	}
 	return NULL;
 }
-
+/*
 static void
 update_project_filelist(Tbfwin * bfwin, Tproject * prj)
 {
 	GList *tmplist;
 	DEBUG_MSG("update_project_filelist, started, bfwin=%p, prj=%p\n", bfwin, prj);
-	free_stringlist(prj->files);
+	free_arraylist(prj->files);
 	DEBUG_MSG("update_project_filelist, old list free'd, creating new list from documentlist %p (len=%d)\n",
 			  bfwin->documentlist, g_list_length(bfwin->documentlist));
 	tmplist = return_urilist_from_doclist(bfwin->documentlist);
 	prj->files = urilist_to_stringlist(tmplist);
 	free_urilist(tmplist);
+	
+}*/
+
+static void
+update_project_filearray_list(Tbfwin * bfwin, Tproject * prj)
+{
+	DEBUG_MSG("update_project_filearray_list, started, bfwin=%p, prj=%p\n", bfwin, prj);
+	free_arraylist(prj->files);
+	DEBUG_MSG("update_project_filearray_list, old list free'd, creating new list from documentlist %p (len=%d)\n",
+			  bfwin->documentlist, g_list_length(bfwin->documentlist));
+	prj->files = return_arraylist_from_doclist(bfwin->documentlist);
 }
 
 void
@@ -143,23 +154,57 @@ set_project_menu_actions(Tbfwin * bfwin, gboolean win_has_project)
 	gtk_action_group_set_sensitive(bfwin->projectGroup, win_has_project);
 }
 
-static void
-setup_bfwin_for_project(Tbfwin * bfwin)
+static gboolean
+project_grab_focus_lcb(gpointer data)
 {
-	DEBUG_MSG("setup_bfwin_for_project, bfwin=%p, bfwin->project=%p, bfwin->session=%p\n", bfwin,
-			  bfwin->project, bfwin->session);
-	DEBUG_MSG("setup_bfwin_for_project, bfwin->project->session=%p\n", bfwin->project->session);
+	Tdocument *doc = data;
+	if (doc && doc->view)
+		gtk_widget_grab_focus(doc->view);
+	return FALSE;
+}
+
+static gboolean
+project_document_load_finished_lcb(gpointer data)
+{
+	Tdocument *doc = data;
+	if (doc->load) {
+		return TRUE;
+	}
+	DEBUG_MSG("Callback project_document_load_finished_lcb , document=%p load finished", doc);
+	gint doc_index = gtk_notebook_get_current_page (GTK_NOTEBOOK(BFWIN(doc->bfwin)->notebook));
+	BFWIN(doc->bfwin)->last_activated_doc = NULL; /* Force document scanning, sometimes first tab document gets the same pointer as destroyed one */
+	bfwin_notebook_changed(BFWIN(doc->bfwin), doc_index);
+	/* get back focus once again, when all parts of bf are finally loaded; this forces cursor blinking; otherwise filebrowser hijacks focus and cursor disappears */
+	g_idle_add_full(G_PRIORITY_LOW+1, project_grab_focus_lcb, doc, NULL);
+	return FALSE;
+}
+
+static void
+setup_bfwin_for_project(Tbfwin * bfwin, Tdocument *active_doc)
+{
+	DEBUG_MSG("setup_bfwin_for_project, bfwin=%p, bfwin->project=%p, bfwin->session=%p\n, current_document=%p, active_doc=%p", bfwin,
+			  bfwin->project, bfwin->session, bfwin->current_document, active_doc);
+	if (active_doc && active_doc->uri) {
+		if(active_doc->load_first) {
+			g_idle_add_full(FILE2DOC_PRIORITY-1, project_document_load_finished_lcb, active_doc, NULL); 
+		} else {
+			g_idle_add_full(FILE2DOC_PRIORITY, project_document_load_finished_lcb, active_doc, NULL); 
+		}
+	} else {
+		/* if there is no document, the project title will not be shown in the window title unless we call this directly */
+		bfwin_set_title(bfwin, bfwin->current_document, 0);
+	}
 	bfwin->bmarkdata = bfwin->project->bmarkdata;
 	bmark_set_store(bfwin);
 	bmark_reload(bfwin);
 #ifdef HAVE_LIBENCHANT
 	reload_spell_dictionary(bfwin);
 #endif
-	bfwin_apply_session(bfwin);
+	bfwin_apply_session(bfwin, active_doc);
 	set_project_menu_actions(bfwin, TRUE);
 #ifdef MAC_INTEGRATION
 /*	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(doc->bfwin)->menubar));*/
-	gtk_osxapplication_sync_menubar(g_object_new(GTK_TYPE_OSX_APPLICATION, NULL));
+	gtkosx_application_sync_menubar(g_object_new(GTKOSX_TYPE_APPLICATION, NULL));
 #endif
 }
 
@@ -176,11 +221,11 @@ setup_bfwin_for_nonproject(Tbfwin * bfwin)
 	if (bfwin->current_document)
 		bfwin_set_title(bfwin, bfwin->current_document, 0);
 
-	bfwin_apply_session(bfwin);
+	bfwin_apply_session(bfwin, bfwin->current_document);
 	set_project_menu_actions(bfwin, FALSE);
 #ifdef MAC_INTEGRATION
 /*	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(bfwin)->menubar));*/
-	gtk_osxapplication_sync_menubar(g_object_new(GTK_TYPE_OSX_APPLICATION, NULL));
+	gtkosx_application_sync_menubar(g_object_new(GTKOSX_TYPE_APPLICATION, NULL));
 #endif
 }
 
@@ -190,7 +235,7 @@ project_destroy(Tproject * project)
 	DEBUG_MSG("project_destroy, project=%p, project->session=%p\n", project, project->session);
 	bookmark_data_cleanup(project->bmarkdata);
 	if (project->files)
-		free_stringlist(project->files);
+		free_arraylist(project->files);
 	if (project->session)
 		free_session(project->session);
 	if (project->uri)
@@ -206,49 +251,57 @@ create_new_project(Tbfwin * bfwin)
 {
 	Tproject *prj;
 	prj = g_new0(Tproject, 1);
+	DEBUG_MSG("create_new_project, project=%p, bfwin=%p\n", prj, bfwin);
 	prj->name = g_strdup(_("New project"));
 	prj->bmarkdata = bookmark_data_new();
-	DEBUG_MSG("create_new_project, project=%p, bfwin=%p\n", prj, bfwin);
-	if (bfwin) {
-		DEBUG_MSG("create_new_project, new project for bfwin %p\n", bfwin);
-		update_project_filelist(bfwin, prj);
-		bfwin->project = prj;
-	} else {
-		DEBUG_MSG("create_new_project, new project, no bfwin\n");
-	}
 	prj->session = g_new0(Tsessionvars, 1);
 	project_setup_initial_session(prj->session, FALSE);
-
-	if (bfwin && prj->files) {
-		GList *tmplist;
-		tmplist = g_list_first(bfwin->documentlist);
-		while (tmplist) {
-			bmark_clean_for_doc(DOCUMENT(tmplist->data));
-			tmplist = g_list_next(tmplist);
-		}
-
-		tmplist = g_list_first(bfwin->session->bmarks);
-		while (tmplist) {
-			gchar **entry = (gchar **) tmplist->data;
-			if (g_strv_length(entry) > 2) {
-				GList *tmplist2 = g_list_first(prj->files);
-				while (tmplist2) {
-					if (strcmp(tmplist2->data, entry[2]) == 0) {
-						/* move it out of the default session into this session */
-						bfwin->session->bmarks = g_list_remove_link(bfwin->session->bmarks, tmplist);
-						prj->session->bmarks = g_list_concat(prj->session->bmarks, tmplist);
-						/* no further filenames to check */
-						tmplist2 = g_list_last(tmplist2);
-					}
-					tmplist2 = g_list_next(tmplist2);
-				}
-			}
-			tmplist = g_list_next(tmplist);
-		}
+	/*Project MUST have recent_dirs*/
+	if (bfwin && bfwin->current_document && bfwin->current_document->uri) {/* We have document with uri */
+		GFile *dir_uri = g_file_get_parent(bfwin->current_document->uri);
+		prj->session->recent_dirs = g_list_append(prj->session->recent_dirs, g_file_get_uri(dir_uri));
+		g_object_unref(dir_uri);
+	} 
+		
+	if (prj->session->recent_dirs == NULL) { /* If still no recent_dirs, then set $HOME */
+		GFile *uri = g_file_new_for_path(g_get_home_dir());
+		prj->session->recent_dirs = g_list_append(prj->session->recent_dirs, g_file_get_uri(uri));
+		g_object_unref(uri);
 	}
+	
 	if (bfwin) {
-		bfwin->session = prj->session;
-		setup_bfwin_for_project(bfwin);
+		DEBUG_MSG("create_new_project, new project for bfwin %p\n", bfwin);
+		update_project_filearray_list(bfwin, prj);
+		if (prj->files) {
+			GList *tmplist;
+			tmplist = g_list_first(bfwin->documentlist);
+			while (tmplist) {
+				bmark_clean_for_doc(DOCUMENT(tmplist->data));
+				tmplist = g_list_next(tmplist);
+			}
+
+			tmplist = g_list_first(bfwin->session->bmarks);
+			while (tmplist) {
+				gchar **entry = (gchar **) tmplist->data;
+				if (g_strv_length(entry) > 2) {
+					GList *tmplist2 = g_list_first(prj->files);
+					while (tmplist2) {
+						if (strcmp(tmplist2->data, entry[2]) == 0) {
+							/* move it out of the default session into this session */
+							bfwin->session->bmarks = g_list_remove_link(bfwin->session->bmarks, tmplist);
+							prj->session->bmarks = g_list_concat(prj->session->bmarks, tmplist);
+							/* no further filenames to check */
+							tmplist2 = g_list_last(tmplist2);
+						}
+						tmplist2 = g_list_next(tmplist2);
+					}
+				}
+				tmplist = g_list_next(tmplist);
+			}
+		}
+		bfwin->project = prj;
+		bfwin->session = prj->session; 
+		setup_bfwin_for_project(bfwin, NULL);
 	}
 	return prj;
 }
@@ -266,7 +319,8 @@ project_save(Tbfwin * bfwin, gboolean save_as)
 	}
 	DEBUG_MSG("project_save, project=%p, num files was %d\n", bfwin->project,
 			  g_list_length(bfwin->project->files));
-	update_project_filelist(bfwin, bfwin->project);
+	/*update_project_filelist(bfwin, bfwin->project); */
+	update_project_filearray_list(bfwin, bfwin->project);
 
 	/*bfwin->project->session->searchlist = limit_stringlist(bfwin->project->session->searchlist, 10, TRUE);
 	bfwin->project->session->replacelist = limit_stringlist(bfwin->project->session->replacelist, 10, TRUE);
@@ -329,7 +383,8 @@ project_save(Tbfwin * bfwin, gboolean save_as)
 	return retval;
 }
 
-void
+/* Returns 0 on fail and 1 on success */
+gint
 project_open_from_file(Tbfwin * bfwin, GFile * fromuri)
 {
 	Tbfwin *prwin;
@@ -343,7 +398,7 @@ project_open_from_file(Tbfwin * bfwin, GFile * fromuri)
 	if (prwin != NULL) {
 		DEBUG_MSG("project_open_from_file, project is open in bfwin=%p\n", prwin);
 		gtk_window_present(GTK_WINDOW(prwin->main_window));
-		return;
+		return 1;
 	}
 
 	prj = g_new0(Tproject, 1);
@@ -359,12 +414,16 @@ project_open_from_file(Tbfwin * bfwin, GFile * fromuri)
 		   bookmark_data_cleanup(prj->bmarkdata);
 		   g_free(prj->session);
 		   g_free(prj); */
-		return;
+		return 0;
 	}
 	curi = g_file_get_uri(fromuri);
 	main_v->globses.recent_projects =
-				add_to_history_stringlist(main_v->globses.recent_projects, curi, FALSE, TRUE);
-	bfwin_recent_menu_remove(bfwin, TRUE, curi);
+				add_to_history_stringlist(main_v->globses.recent_projects, curi, TRUE);
+	if (main_v->props.recent_means_recently_closed) {
+		bfwin_recent_menu_remove(bfwin, TRUE, curi);
+	} else {
+		bfwin_recent_menu_add(bfwin,TRUE, curi);
+	}
 	if (main_v->props.register_recent_mode != 0) {
 		gtk_recent_manager_add_item(main_v->recentm, curi);
 	}
@@ -379,31 +438,62 @@ project_open_from_file(Tbfwin * bfwin, GFile * fromuri)
 		prwin->session = prj->session;
 		DEBUG_MSG("project_open_from_file, project %p will be in existing prwin=%p\n", prj, bfwin);
 		/* destroy the current empty document, it should use settings from the new session */
-		if (bfwin->current_document)
-			doc_destroy(bfwin->current_document, TRUE);
+		if (prwin->current_document)
+			doc_destroy(prwin->current_document, TRUE);
 	} else {
 		/* we will open a new Bluefish window for this project */
 		DEBUG_MSG("project_open_from_file, we need a new window\n");
 		prwin = bfwin_window_new_with_project(prj);
+		if (prwin->current_document)
+			doc_destroy(prwin->current_document, TRUE); /*new window is created with empty doc, so we destroy it TODO move empty tab creation from bfwin_create_main() */
 	}
 	tmplist = g_list_last(prj->files);
+	gint doc_index = 0;
+	gint i =0;
+	gint cursor_offset = -1;
+	gint goto_offset = -1; 
+	gint is_active = 0;
 	if (tmplist) {
+		bfwin_notebook_block_signals(prwin); /* Block switch-page signal while we are creating doc backends */
 		while (tmplist) {
 			GFile *uri;
-			if (strstr((gchar *) tmplist->data, "://") == NULL)
-				uri = g_file_new_for_path((gchar *) tmplist->data);
-			else
-				uri = g_file_new_for_uri((gchar *) tmplist->data);
-			doc_new_from_uri(prwin, uri, NULL, !(prj->files->prev == NULL), TRUE, -1, -1);
+			gchar **tmparr = (gchar **) tmplist->data;
+			if (strstr(tmparr[0], "://") == NULL) {
+				uri = g_file_new_for_path(tmparr[0]);
+			} else {
+				uri = g_file_new_for_uri(tmparr[0]);
+			}
+			
+			if (tmparr[1] && tmparr[1]!='\0') {
+				cursor_offset = atoi(tmparr[1]);
+				if (tmparr[2] && tmparr[2]!='\0') {
+					goto_offset = atoi(tmparr[2]);
+					if (tmparr[3] && tmparr[3]!='\0') {
+						is_active = atoi(tmparr[3]);
+					}
+				}
+			}
+			if (is_active) {
+				doc_index = i;
+				doc_new_from_uri(prwin, uri, NULL, TRUE, TRUE, -1, goto_offset, cursor_offset, FALSE, TRUE);
+			} else {
+				doc_new_from_uri(prwin, uri, NULL, TRUE, TRUE, -1, goto_offset, cursor_offset, FALSE, FALSE);
+			}
 			g_object_unref(uri);
 			tmplist = g_list_previous(tmplist);
+			i++;
 		}
+		/* Now switch to tab that holds last active document from previous session */
+		bfwin_switch_to_document_by_index(prwin, doc_index);
+		bfwin_notebook_unblock_signals(prwin);  /* Unblock signals, doc will be activated in setup_bfwin_for_project*/
 	} else {
-		doc_new(bfwin, FALSE);
+		doc_new(prwin, FALSE);
 	}
+	Tdocument * active_doc = documentlist_return_document_from_index(prwin->documentlist, doc_index);
 	DEBUG_MSG("project_open_from_file, new window with files ready at prwin=%p\n", prwin);
-	setup_bfwin_for_project(prwin);
+	setup_bfwin_for_project(prwin, active_doc);
 	DEBUG_MSG("project_open_from_file, done\n");
+	return 1;
 }
 
 static void
@@ -435,7 +525,7 @@ project_save_and_mark_closed(Tbfwin * bfwin)
 	if (bfwin->project) {
 		project_save(bfwin, FALSE);
 
-		if (bfwin->project->uri) {
+		if (bfwin->project->uri && main_v->props.recent_means_recently_closed) { /* Add menu entry only if we use "recently_closed" logic otherwise it is already there*/
 			gchar *curi = g_file_get_uri(bfwin->project->uri);
 			bfwin_recent_menu_add(bfwin, TRUE, curi);
 			g_free(curi);
@@ -545,8 +635,15 @@ project_edit_ok_clicked_lcb(GtkWidget * widget, Tprojecteditor * pred)
 	if (pred->bfwin == NULL) {
 		pred->bfwin = bfwin_window_new_with_project(pred->project);
 		pred->bfwin->session = pred->project->session;
-		setup_bfwin_for_project(pred->bfwin);
+		setup_bfwin_for_project(pred->bfwin, NULL);
 	} else {
+#ifdef MAC_INTEGRATION
+	if (main_v->osx_status == 2){
+		gtk_widget_show(pred->bfwin->main_window);
+		bfwin_action_groups_set_sensitive(pred->bfwin, TRUE);
+		main_v->osx_status = 0;
+	}
+#endif	
 		bfwin_set_title(pred->bfwin, pred->bfwin->current_document, 0);
 	}
 /* set_project_menu_widgets(pred->bfwin, TRUE);*/
@@ -560,7 +657,7 @@ if NULL is passed as bfwin, a new window will be created for this project once O
 void
 project_edit(Tbfwin * bfwin)
 {
-	GtkWidget *vbox, *but, *hbox, *label, *table;
+	GtkWidget *vbox, *but, *hbox, *label, *table, *parent_window=NULL;
 	gchar *wintitle = NULL;
 	Tprojecteditor *pred;
 
@@ -575,6 +672,7 @@ project_edit(Tbfwin * bfwin)
 		pred->project = create_new_project(bfwin);
 		if (bfwin) {
 			bfwin->project = pred->project;
+			parent_window = bfwin->main_window;
 		}
 		/* id the user does not press OK, we destroy the project */
 		pred->destroy_project_on_close = TRUE;
@@ -582,17 +680,15 @@ project_edit(Tbfwin * bfwin)
 	} else {
 		pred->destroy_project_on_close = FALSE;
 		wintitle = g_strdup(_("Edit Project"));
-		if (bfwin) {
-			pred->project = bfwin->project;
-		}
+		pred->project = bfwin->project;
+		parent_window = bfwin->main_window;
 	}
 	DEBUG_MSG("project_edit, Tproject at %p\n", pred->project);
 	pred->bfwin = bfwin;
 	pred->project->editor = pred;
-
 	pred->win =
-		window_full2(wintitle, GTK_WIN_POS_CENTER_ALWAYS, 5, G_CALLBACK(project_edit_destroy_lcb), pred, TRUE,
-					 NULL);
+		window_full2(wintitle, GTK_WIN_POS_CENTER, 5, G_CALLBACK(project_edit_destroy_lcb), pred, TRUE,
+					parent_window);
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(pred->win), vbox);
 
@@ -605,10 +701,11 @@ project_edit(Tbfwin * bfwin)
 	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
 	{
 		gchar *message;
+		gint numdocs = g_list_length(pred->project->files);
 		message =
 			g_strdup_printf(ngettext
 							("This project contains %d file", "This project contains %d files",
-							 g_list_length(pred->project->files)), g_list_length(pred->project->files));
+							 numdocs), numdocs);
 		gtk_label_set_markup(GTK_LABEL(label), message);
 		g_free(message);
 	}
@@ -691,8 +788,8 @@ project_create_gui(Tbfwin * bfwin)
 	pc = g_new(Tpc, 1);
 	pc->bfwin = bfwin;
 	pc->win =
-		window_full2(_("Create project"), GTK_WIN_POS_NONE, 5, G_CALLBACK(project_create_destroy_lcb), pc,
-					 TRUE, NULL);
+		window_full2(_("Create project"), GTK_WIN_POS_CENTER, 5, G_CALLBACK(project_create_destroy_lcb), pc,
+					 TRUE, bfwin->main_window);
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(pc->win), vbox);
 	pc->rad[0] =
