@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * rcfile.c - loading and parsing of the configfiles
  *
- * Copyright (C) 2000-2012 Olivier Sessink
+ * Copyright (C) 2000-2013 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -216,7 +216,7 @@ save_config_file(GHashTable * config_list, GFile * file)
 
 			DEBUG_MSG("save_config_file, adding %s\n", tmpstring);
 
-			rclist = g_list_append(rclist, tmpstring);
+			rclist = g_list_prepend(rclist, tmpstring);
 			break;
 		case 's':
 			DEBUG_MSG("save_config_file, converting \"%p\" to string\n", tmpitem);
@@ -228,7 +228,7 @@ save_config_file(GHashTable * config_list, GFile * file)
 
 				DEBUG_MSG("save_config_file, adding %s\n", tmpstring);
 
-				rclist = g_list_append(rclist, tmpstring);
+				rclist = g_list_prepend(rclist, tmpstring);
 			}
 			break;
 		case 'e':
@@ -241,7 +241,7 @@ save_config_file(GHashTable * config_list, GFile * file)
 
 				DEBUG_MSG("save_config_file, adding %s\n", tmpstring);
 
-				rclist = g_list_append(rclist, tmpstring);
+				rclist = g_list_prepend(rclist, tmpstring);
 				g_free(tmpstring2);
 			}
 			break;
@@ -252,19 +252,19 @@ save_config_file(GHashTable * config_list, GFile * file)
 						  &tmpitem);
 				if (tmpitem->type == 'm')
 					max = tmpitem->len;
-				tmplist2 = g_list_last((GList *) * (void **) tmpitem->pointer);
+				tmplist2 = g_list_first((GList *) * (void **) tmpitem->pointer);
 				while (tmplist2 != NULL && max != 0) {
 					tmpstring2 = escape_string((char *) tmplist2->data, FALSE);
 					tmpstring = g_strconcat(tmpitem->identifier, " ", tmpstring2, NULL);
 					g_free(tmpstring2);
 					DEBUG_MSG("save_config_file, tmpstring(%p)=%s, orig was %s\n", tmpstring, tmpstring,
 							  (gchar *) tmplist2->data);
-					rclist = g_list_append(rclist, tmpstring);
-					tmplist2 = g_list_previous(tmplist2);
+					rclist = g_list_prepend(rclist, tmpstring);
+					tmplist2 = g_list_next(tmplist2);
 					max--;
 #ifdef DEBUG
 					if (max == 0 && tmplist2 != NULL)
-						g_print("save_config_file, limit reached!, next item would have been %s\n",
+						DEBUG_MSG("save_config_file, limit reached!, next item would have been %s\n",
 								(gchar *) tmplist2->data);
 #endif
 				}
@@ -281,7 +281,7 @@ save_config_file(GHashTable * config_list, GFile * file)
 				else
 					tmpstring = g_strdup_printf("%s", tmpstring2);
 				DEBUG_MSG("save_config_file, tmpstring(%p)=%s\n", tmpstring, tmpstring);
-				rclist = g_list_append(rclist, tmpstring);
+				rclist = g_list_prepend(rclist, tmpstring);
 				tmplist2 = g_list_previous(tmplist2);
 				g_free(tmpstring2);
 			}
@@ -404,13 +404,20 @@ parse_config_file(GHashTable * config_list, GFile * file)
 static GHashTable *
 props_init_main(GHashTable * config_rc)
 {
+	/* if there is no config version set, assume it is a config file from a version before the config version was introduced
+	this was introduced in revision r7372 in april 2012, which was released in june 2012 in the 2.2.3 release */
 	init_prop_string(&config_rc, &main_v->props.config_version, "config_version:", "2.2.2");
 	init_prop_integer(&config_rc, &main_v->props.do_periodic_check, "do_periodic_check:", 1, TRUE);
+#ifdef MAC_INTEGRATION
+	init_prop_string(&config_rc, &main_v->props.editor_font_string, "editor_font_string:", "Lucida Grande 13");
+#else
 	init_prop_string(&config_rc, &main_v->props.editor_font_string, "editor_font_string:", "monospace 10");
+#endif
 	init_prop_integer(&config_rc, &main_v->props.editor_smart_cursor, "editor_smart_cursor:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.editor_tab_indent_sel, "editor_tab_indent_sel:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.editor_auto_close_brackets, "editor_auto_close_brackets:", 2, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.use_system_tab_font, "use_system_tab_font:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.max_shown_filename_len, "max_shown_filename_len:", 0, TRUE);
 	init_prop_string(&config_rc, &main_v->props.tab_font_string, "tab_font_string:", "");
 	init_prop_string(&config_rc, &main_v->props.tab_color_modified, "tab_color_modified:", "#0000FF");
 	init_prop_string(&config_rc, &main_v->props.tab_color_loading, "tab_color_loading:", "#C7C7C7");
@@ -436,6 +443,7 @@ props_init_main(GHashTable * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.max_recent_files, "max_recent_files:", 15, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.max_dir_history, "max_dir_history:", 10, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.backup_file, "backup_file:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.show_long_line_warning, "show_long_line_warning:", 1, TRUE);
 /*	init_prop_string    (&config_rc, &main_v->props.backup_suffix,"backup_suffix:","~");
 	init_prop_string    (&config_rc, &main_v->props.backup_prefix,"backup_prefix:","");*/
 	init_prop_integer(&config_rc, &main_v->props.backup_abort_action, "backup_abort_action:",
@@ -478,6 +486,7 @@ props_init_main(GHashTable * config_rc)
 					  TRUE);
 	init_prop_integer(&config_rc, &main_v->props.open_in_new_window, "open_in_new_window:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.register_recent_mode, "register_recent_mode:", 2, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.recent_means_recently_closed, "recent_means_recently_closed:", 1, TRUE);
 	init_prop_arraylist(&config_rc, &main_v->props.plugin_config, "plugin_config:", 3, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.use_system_colors, "use_system_colors:", 1, TRUE);
 	init_prop_string(&config_rc, &main_v->props.btv_color_str[BTV_COLOR_ED_FG], "editor_fg:", "");
@@ -504,6 +513,7 @@ props_init_main(GHashTable * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.delay_full_scan, "delay_full_scan:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.delay_scan_time, "delay_scan_time:", 900, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.autocomp_popup_mode, "autocomp_popup_mode:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.autocomp_min_prefix_len, "autocomp_min_prefix_len:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.reduced_scan_triggers, "reduce_scan_triggers:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.autosave, "autosave:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.autosave_time, "autosave_time:", 60, TRUE);
@@ -514,6 +524,8 @@ props_init_main(GHashTable * config_rc)
 
 	init_prop_integer(&config_rc, &main_v->props.adv_textview_right_margin, "adv_textview_right_margin:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.adv_textview_left_margin, "adv_textview_left_margin:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.wrap_on_right_margin, "wrap_on_right_margin:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.adv_smart_indent_mode, "adv_smart_indent_mode:", 1, TRUE);
 	/*init_prop_arraylist(&config_rc, &main_v->props.templates, "templates:", 2, TRUE);*/
 	return config_rc;
 }
@@ -618,9 +630,11 @@ static GList *update_externals(GList *current, GList *defaults, gboolean overwri
 {
 	GList *tmplist;
 	GHashTable *ht=NULL;
-
-	if (current == NULL)
+	DEBUG_MSG("update_externals, started\n");
+	if (current == NULL) {
 		overwrite = TRUE;
+		DEBUG_MSG("update_externals, current == NULL, set overwrite=TRUE\n");
+	}
 
 	if (overwrite) {
 		free_arraylist(current);
@@ -639,7 +653,7 @@ static GList *update_externals(GList *current, GList *defaults, gboolean overwri
 
 			if (len == (newlen-1)) { /* convert the old (2.2.2 or older) format into the new format */
 				gchar **oldarr = arr;
-				/*g_print("prepend %s in front of %s\n",USER_DEFINED_ENABLED,arr[0]);*/
+				/*DEBUG_MSG("prepend %s in front of %s\n",USER_DEFINED_ENABLED,arr[0]);*/
 				cur->data = arr = prepend_array(USER_DEFINED_ENABLED,arr);
 				g_strfreev(oldarr);
 				/* arr[2] contains the commandstring. now replace %I and %O (previously fifo in and
@@ -649,7 +663,7 @@ static GList *update_externals(GList *current, GList *defaults, gboolean overwri
 			}
 			if (len == newlen && arr[0][0]!='0' && arr[0][0]!='1') {
 				/* keep all the user defined options */
-				/*g_print("update_externals, insert into hash: %s\n",arr[1]);*/
+				DEBUG_MSG("update_externals, insert into hash: %s\n",arr[1]);
 				g_hash_table_insert(ht, arr[1], arr);
 			} else {
 				/* remove all others */
@@ -662,7 +676,7 @@ static GList *update_externals(GList *current, GList *defaults, gboolean overwri
 	for (tmplist=g_list_first(defaults);tmplist;tmplist = g_list_next(tmplist)) {
 		gchar **arr=tmplist->data;
 		if (overwrite || g_hash_table_lookup(ht, arr[0])==NULL) {
-			/*g_print("update_externals, prepend a missing or removed-default value %s\n",arr[0]);*/
+			DEBUG_MSG("update_externals, prepend a missing or removed-default value %s\n",arr[0]);
 			current = g_list_prepend(current, prepend_array(BLUEFISH_DEFINED_ENABLED, arr));
 		}
 	}
@@ -892,6 +906,7 @@ rcfile_parse_main(void)
 		main_v->props.encoding_search_Nbytes = 2048;
 
 	/* do some default configuration for the lists */
+	DEBUG_MSG("config_version=%s\n",main_v->props.config_version);
 	if (!main_v->props.config_version || strlen(main_v->props.config_version)<5 ||main_v->props.config_version[0] < '2' || main_v->props.config_version[2] < '2' || main_v->props.config_version[4] < '4') {
 		main_v->props.rcfile_from_old_version = 1;
 		main_v->props.external_command = update_commands(main_v->props.external_command, FALSE);
@@ -1004,11 +1019,11 @@ static GHashTable *
 return_globalsession_configlist(gboolean init_values)
 {
 	GHashTable *config_rc = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, free_config_list_item);
-	init_prop_integer(&config_rc, &main_v->globses.main_window_h, "main_window_height:", 400, init_values);
-	init_prop_integer(&config_rc, &main_v->globses.main_window_w, "main_window_width:", 600, init_values);	/* negative width means maximized */
+	init_prop_integer(&config_rc, &main_v->globses.main_window_h, "main_window_height:", 800, init_values);
+	init_prop_integer(&config_rc, &main_v->globses.main_window_w, "main_window_width:", 1200, init_values);	/* negative width means maximized */
 	init_prop_integer(&config_rc, &main_v->globses.two_pane_filebrowser_height,
-					  "two_pane_filebrowser_height:", 250, init_values);
-	init_prop_integer(&config_rc, &main_v->globses.left_panel_width, "left_panel_width:", 150, init_values);
+					  "two_pane_filebrowser_height:", 300, init_values);
+	init_prop_integer(&config_rc, &main_v->globses.left_panel_width, "left_panel_width:", 250, init_values);
 	/*init_prop_integer   (&config_rc, &main_v->globses.lasttime_filetypes, "lasttime_filetypes:", 0, init_values);
 	   init_prop_integer   (&config_rc, &main_v->globses.lasttime_encodings, "lasttime_encodings:", 0, init_values); */
 	init_prop_integer(&config_rc, &main_v->globses.bookmarks_default_store, "bookmarks_default_store:", 1,
@@ -1030,7 +1045,8 @@ return_globalsession_configlist(gboolean init_values)
 	init_prop_arraylist(&config_rc, &main_v->globses.reference_files, "reference_files:", 2, init_values);
 	init_prop_limitedstringlist(&config_rc, &main_v->globses.recent_projects, "recent_projects:",
 								main_v->props.max_recent_files, init_values);
-	init_prop_arraylist(&config_rc, &main_v->globses.encodings, "encodings:", 3, FALSE);
+	init_prop_arraylist(&config_rc, &main_v->globses.encodings, "encodings:", 3, init_values);
+	init_prop_arraylist(&config_rc, &main_v->globses.custombflangmime, "custom_bflang_mime:", 2, init_values);
 #ifdef WITH_MSG_QUEUE
 	init_prop_integer(&config_rc, &main_v->globses.msg_queue_poll_time, "msg_queue_poll_time:", 500,
 					  init_values);
@@ -1091,7 +1107,11 @@ return_session_configlist(GHashTable * configlist, Tsessionvars * session)
 	init_prop_stringlist(&configlist, &session->fontlist, "fontlist:", FALSE);
 	init_prop_arraylist(&configlist, &session->bmarks, "bmarks:", 6, FALSE);	/* what is the lenght for a bookmark array? */
 	init_prop_limitedstringlist(&configlist, &session->recent_files, "recent_files:",
-								main_v->props.max_recent_files, FALSE);
+								main_v->props.max_recent_files*2, FALSE);
+	/* because the recentlist probably contains both the files that will be opened in a project AND the
+	files that were recently opened/closed, we store more than then number of files that are shown to the
+	user. Otherwise the recent list might be empty if the user has a project list with more files than
+	the length of the recent_files */
 	init_prop_limitedstringlist(&configlist, &session->recent_dirs, "recent_dirs:",
 								main_v->props.max_dir_history, FALSE);
 	init_prop_string_with_escape(&configlist, &session->opendir, "opendir:", NULL);
@@ -1165,11 +1185,14 @@ setup_session_after_parse(Tsessionvars * session)
 }
 
 static GHashTable *
-return_project_configlist(Tproject * project)
+return_project_configlist(Tproject * project, GList **oldfiles)
 {
 	GHashTable *configlist = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, free_config_list_item);
 	init_prop_string(&configlist, &project->name, "name:", _("Untitled Project"));
-	init_prop_stringlist(&configlist, &project->files, "files:", FALSE);
+	init_prop_arraylist(&configlist, &project->files, "openfiles:", 0, FALSE); /* Zero allows any length of array, array lenght should be 5 after first save */
+	if (oldfiles) {
+		init_prop_stringlist(&configlist, oldfiles, "files:", FALSE);
+	}
 /*	init_prop_stringlist(&configlist, &project->recentfiles, "recentfiles:", FALSE); / * should be changed to use the session->recent_files */
 	configlist = return_session_configlist(configlist, project->session);
 	return configlist;
@@ -1179,9 +1202,21 @@ gboolean
 rcfile_parse_project(Tproject * project, GFile * file)
 {
 	gboolean retval;
-	GHashTable *configlist = return_project_configlist(project);
+	GList *oldfiles=NULL;
+	GHashTable *configlist = return_project_configlist(project, &oldfiles);
 	retval = parse_config_file(configlist, file);
 	free_configlist(configlist);
+	if (project->files == NULL && oldfiles != NULL) {
+		/* convert old format to new format! */
+		GList *tmplist = g_list_last(oldfiles);
+		while (tmplist) {
+			g_print("convert old format project file to new, for file %s\n",(char *)tmplist->data);
+			project->files = g_list_prepend(project->files, array_from_arglist(tmplist->data, "0", "0", "0", NULL));
+			tmplist = g_list_previous(tmplist);
+		}
+		free_stringlist(oldfiles);
+	}
+
 	setup_session_after_parse(project->session);
 	return retval;
 }
@@ -1190,7 +1225,7 @@ gboolean
 rcfile_save_project(Tproject * project, GFile * file)
 {
 	gboolean retval;
-	GHashTable *configlist = return_project_configlist(project);
+	GHashTable *configlist = return_project_configlist(project, NULL);
 	DEBUG_MSG("rcfile_save_project, project %p, name='%s'\n", project, project->name);
 	DEBUG_MSG("rcfile_save_project, bmarks=%p, list length=%d\n", project->session->bmarks,
 			  g_list_length(project->session->bmarks));
@@ -1221,7 +1256,7 @@ static void
 add_new_encodings(GList *defaults, GList *current)
 {
 	GHashTable *ht;
-	GList *tmplist, *last;
+	GList *tmplist, *last=NULL;
 	ht = g_hash_table_new(g_str_hash, g_str_equal);
 
 	for (tmplist=g_list_first(current);tmplist;tmplist=tmplist->next) {
@@ -1325,13 +1360,13 @@ load_templates_from_dir(GFile *uri, gboolean create_ifnexist)
 	while(finfo) {
 		gchar *tmp, **arr;
 		child = g_file_get_child(uri, g_file_info_get_name(finfo));
-    	arr = g_malloc(sizeof(char *)* 3);
-    	arr[0] = g_strdup(g_file_info_get_name(finfo));
+		arr = g_malloc(sizeof(char *)* 3);
+		arr[0] = g_strdup(g_file_info_get_name(finfo));
 		arr[1] = g_file_get_path(child);
 		arr[2] = NULL;
 		while ((tmp = strchr(arr[0], '_')) != NULL) {
-    		*tmp = ' ';
-    	}
+			*tmp = ' ';
+		}
 		retlist = g_list_prepend(retlist, arr);
 		g_object_unref(child);
 		g_object_unref(finfo);
