@@ -299,12 +299,12 @@ s3result_replace(Tsnr3run *s3run, Tsnr3result *s3result, GMatchInfo *matchinfo)
 		doc_replace_text_backend(s3result->doc, tmp2, s3result->so, s3result->eo);
 		g_free(tmp1);
 		g_free(tmp2);
-	} else if (s3run->replacetype == snr3replace_string) {
+	} else {
 		if (s3run->type == snr3type_string) {
 			DEBUG_MSG("s3result_replace, replace %d:%d with %s\n", s3result->so, s3result->eo, s3run->replacereal);
 			doc_replace_text_backend(s3result->doc, s3run->replacereal, s3result->so, s3result->eo);
 			offsetupdate.offset = g_utf8_strlen(s3run->replacereal, -1)-(s3result->eo - s3result->so);
-		} else if (s3run->type == snr3type_pcre) {
+		} else if (s3run->type == snr3type_pcre && s3run->replacetype == snr3replace_string) {
 			gchar *newstr = retrieve_pcre_replace_string(s3run, s3result, matchinfo);
 			if (newstr) {
 				DEBUG_MSG("s3result_replace, replace from  %d:%d\n",s3result->so,s3result->eo);
@@ -766,7 +766,7 @@ snr3run_resultcleanup(Tsnr3run *s3run)
 
 /* called from bfwin.c for simplesearch */
 void
-snr3run_free(Tsnr3run *s3run) {
+snr3run_free(Tsnr3run *s3run, gboolean remove_highlights) {
 	DEBUG_MSG("snr3run_free, started for %p\n",s3run);
 	snr3_cancel_run(s3run);
 	if (s3run->curbuf)
@@ -787,8 +787,10 @@ snr3run_free(Tsnr3run *s3run) {
 	DEBUG_MSG("snr3run_free, basedir\n");
 	if (s3run->basedir)
 		g_object_unref(s3run->basedir);
-	DEBUG_MSG("snr3run_free, remove all highlights\n");
-	remove_all_highlights_in_doc(s3run->bfwin->current_document);
+	if (remove_highlights) {
+		DEBUG_MSG("snr3run_free, remove all highlights\n");
+		remove_all_highlights_in_doc(s3run->bfwin->current_document);
+	}
 	DEBUG_MSG("snr3run_free, resultcleanup\n");
 	snr3run_resultcleanup(s3run);
 	g_slice_free(Tsnr3run, s3run);
@@ -1113,7 +1115,7 @@ activate_simple_search(void *data) {
 	DEBUG_MSG("activate_simple_search, s3run=%p\n", s3run);
 	s3run->curdoc = NULL;
 	highlight_run_in_doc(s3run, s3run->bfwin->current_document);
-	snr3_run_go(s3run, TRUE);
+	/*snr3_run_go(s3run, TRUE);*/
 }
 
 static void
@@ -1138,7 +1140,7 @@ simple_search_add_to_history(Tbfwin *bfwin, const gchar *string)
 		}
 	}
 	DEBUG_MSG("prepend %s to searchlist\n", string);
-	bfwin->session->searchlist = add_to_history_stringlist(bfwin->session->searchlist, string, TRUE, TRUE);
+	bfwin->session->searchlist = add_to_history_stringlist(bfwin->session->searchlist, string, TRUE);
 }
 
 gpointer simple_search_run(Tbfwin *bfwin, const gchar *string, Tsnr3type type
@@ -1374,11 +1376,11 @@ snr3_advanced_response(GtkDialog * dialog, gint response, TSNRWin * snrwin)
 	if (guichange == -1)
 		return;
 
-	snrwin->bfwin->session->searchlist = add_to_history_stringlist(snrwin->bfwin->session->searchlist, s3run->query, TRUE,TRUE);
+	snrwin->bfwin->session->searchlist = add_to_history_stringlist(snrwin->bfwin->session->searchlist, s3run->query, TRUE);
 	if (s3run->replace && s3run->replace[0] != '\0')
-		snrwin->bfwin->session->replacelist = add_to_history_stringlist(snrwin->bfwin->session->replacelist, s3run->replace,TRUE, TRUE);
+		snrwin->bfwin->session->replacelist = add_to_history_stringlist(snrwin->bfwin->session->replacelist, s3run->replace,TRUE);
 	if (s3run->scope == snr3scope_files && s3run->filepattern && s3run->filepattern[0] != '\0')
-		snrwin->bfwin->session->filegloblist = add_to_history_stringlist(snrwin->bfwin->session->filegloblist, s3run->filepattern,TRUE, TRUE);
+		snrwin->bfwin->session->filegloblist = add_to_history_stringlist(snrwin->bfwin->session->filegloblist, s3run->filepattern,TRUE);
 
 	switch(response) {
 		case SNR_RESPONSE_FIND:
@@ -1492,7 +1494,7 @@ snr3win_destroy_cb(GtkWidget *widget, gpointer user_data)
 {
 	TSNRWin *snrwin=user_data;
 	DEBUG_MSG("snr3win_destroy_cb, user_data=%p\n",user_data);
-	snr3run_free(snrwin->s3run);
+	snr3run_free(snrwin->s3run, TRUE);
 	g_slice_free(TSNRWin, snrwin);
 }
 
@@ -1825,6 +1827,6 @@ snr3_run_extern_replace(Tdocument * doc, const gchar * search_pattern, Tsnr3scop
 			g_warning("snr3_run_extern_replace does not support replace in files\n");
 		break;
 	}
-	snr3run_free(s3run);
+	snr3run_free(s3run, FALSE); /* We do not need to remove hightlights in current_document, so we pass FALSE */
 }
 
