@@ -93,7 +93,7 @@ static gchar *line_from_buffer(gchar *buffer, guint offset) {
 
 static Tfileresult *new_result(guint line, gchar *buffer, guint offset) {
 	Tfileresult *fr;
-	
+
 	fr = g_slice_new(Tfileresult);
 	fr->line = line;
 	fr->text = line_from_buffer(buffer, offset);
@@ -104,7 +104,7 @@ static GList *snr3_find_pcre(Tsnr3run *s3run, gchar *buffer) {
 	Tlineinbuffer lib = {0,1};
 	GList *results=NULL;
 	GMatchInfo *match_info;
-	
+
 	g_regex_match(s3run->regex, buffer, 0, &match_info);
 	while(g_match_info_matches(match_info)) {
 		gint so, eo;
@@ -115,8 +115,8 @@ static GList *snr3_find_pcre(Tsnr3run *s3run, gchar *buffer) {
 		g_match_info_next(match_info, NULL);
 	}
 	g_match_info_free(match_info);
-	
-	return results; 
+
+	return results;
 }
 
 
@@ -129,10 +129,10 @@ static GList *snr3_replace_pcre(Tsnr3run *s3run, gchar *buffer, gchar **replaced
 	GList *results=NULL;
 	GMatchInfo *match_info;
 	gsize prevpos=0;
-	
+
 	DEBUG_MSG("snr3_replace_pcre, search for %s, replace with %s\n",s3run->query, s3run->replace);
 	buflen = strlen(buffer);
-	
+
 	alloced = MAX(buflen*2,4096);
 	newbuf = g_malloc0(alloced);
 
@@ -150,17 +150,17 @@ static GList *snr3_replace_pcre(Tsnr3run *s3run, gchar *buffer, gchar **replaced
 		newbufpos += (so-prevpos);
 		bufferpos = buffer+eo;
 		prevpos = eo;
-		
+
 		line = calculate_line_in_buffer(&lib, newbuf, (newbufpos-newbuf));
 		results = g_list_prepend(results, new_result(line, newbuf, (newbufpos-newbuf)));
-		
+
 		replacestring = g_match_info_expand_references(match_info, s3run->replace, &gerror);
 		if (gerror) {
 			g_print("replace error %s\n",gerror->message);
 			g_error_free(gerror);
 		}
-		replacelen = strlen(replacestring);	
-		
+		replacelen = strlen(replacestring);
+
 		/* now check if we have enough memory */
 		if (alloced < (1+ newbufpos-newbuf + replacelen + buflen-prevpos)) {
 			gchar *tmp;
@@ -169,18 +169,18 @@ static GList *snr3_replace_pcre(Tsnr3run *s3run, gchar *buffer, gchar **replaced
 			newbufpos = tmp + (newbufpos-newbuf);
 			newbuf=tmp;
 		}
-			
+
 		memcpy(newbufpos, replacestring, replacelen);
 		newbufpos += replacelen;
 		g_free(replacestring);
-		
+
 		g_match_info_next(match_info, NULL);
 	}
 	g_match_info_free(match_info);
-	
+
 	memcpy(newbufpos, buffer+prevpos, buflen-prevpos+1);
 	*replacedbuffer = newbuf;
-	return results; 
+	return results;
 }
 
 static GList *snr3_find_string(Tsnr3run *s3run, gchar *buffer) {
@@ -189,7 +189,7 @@ static GList *snr3_find_string(Tsnr3run *s3run, gchar *buffer) {
 	Tlineinbuffer lib = {0,1};
 	GList *results=NULL;
 	char *(*f) ();
-	
+
 	if (s3run->is_case_sens) {
 		f = strstr;
 	} else {
@@ -219,44 +219,44 @@ static GList *snr3_replace_string(Tsnr3run *s3run, gchar *buffer, gchar **replac
 	Tlineinbuffer lib = {0,1};
 	GList *results=NULL;
 	char *(*f) ();
-	
+
 	DEBUG_MSG("snr3_replace_string, search for %s, replace with %s\n",s3run->query, s3run->replace);
 	querylen = strlen(s3run->query);
 	replacelen = strlen(s3run->replace);
 	buflen = strlen(buffer);
-	
-	
+
+
 	alloced = (replacelen > querylen)? MAX(1+replacelen+buflen*2,4096):MAX(buflen+querylen+1, 4096);
 	newbuf = g_malloc0(alloced);
 
 	bufferpos = buffer;
 	newbufpos = newbuf;
-	
+
 	if (s3run->is_case_sens) {
 		f = strstr;
 	} else {
 		f = strcasestr;
 	}
-	
+
 	result = buffer;
-	
+
 	do {
 		result = f(result, s3run->query);
 		if (result) {
 			guint line;
-			
+
 			memcpy(newbufpos, bufferpos, result-bufferpos);
 			newbufpos += (result-bufferpos);
-			
+
 			line = calculate_line_in_buffer(&lib, newbuf, (newbufpos-newbuf));
 
 			memcpy(newbufpos, s3run->replace, replacelen);
 			newbufpos += replacelen;
 			result += querylen;
 			bufferpos = result;
-			
+
 			results = g_list_prepend(results, new_result(line, newbuf, newbufpos-newbuf));
-			
+
 			if (alloced <= (1+replacelen+(newbufpos-newbuf)+replacelen+(buflen-(bufferpos-buffer)))) {
 				gchar *tmp;
 				alloced += MAX(buflen, 4096);
@@ -264,12 +264,12 @@ static GList *snr3_replace_string(Tsnr3run *s3run, gchar *buffer, gchar **replac
 				newbufpos = tmp + (newbufpos-newbuf);
 				newbuf=tmp;
 			}
-		}		
+		}
 	} while (result);
-	
+
 	memcpy(newbufpos, bufferpos, strlen(bufferpos)+1);
 	*replacedbuffer = newbuf;
-	return results; 
+	return results;
 }
 
 typedef struct {
@@ -282,7 +282,7 @@ static gboolean replace_files_in_thread_finished(gpointer data) {
 	Treplaceinthread *rit = data;
 	gchar *curi;
 	GList *tmplist;
-	
+
 	DEBUG_MSG("replace_files_in_thread_finished, add %d results to outputbox\n",g_list_length(rit->results));
 	if (g_atomic_int_get(&rit->s3run->cancelled)==0) {
 		curi = g_file_get_uri(rit->uri);
@@ -309,9 +309,14 @@ static gpointer files_replace_run(gpointer data) {
 	gchar *inbuf=NULL, *encoding=NULL, *outbuf, *utf8buf;
 	gsize inbuflen=0, outbuflen=0;
 	Tasyncqueue *tmpqueue;
-	
+
+	if (!data) {
+		g_warning("problem detected in files_replace_run, data is NULL, please report a bug\n");
+		return NULL;
+	}
+
 	DEBUG_MSG("thread %p: files_replace_run, started rit %p\n", g_thread_self(), rit);
-	
+
 	g_file_load_contents(rit->uri,NULL,&inbuf,&inbuflen,NULL,&gerror);
 	if (g_atomic_int_get(&rit->s3run->cancelled)!=0) {
 		g_free(inbuf);
@@ -329,7 +334,7 @@ static gpointer files_replace_run(gpointer data) {
 		/* is the following function thread safe ?? */
 		utf8buf = buffer_find_encoding(inbuf, inbuflen, &encoding, rit->s3run->bfwin->session->encoding);
 		g_free(inbuf);
-		
+
 		if (utf8buf) {
 			gchar *replacedbuf=NULL;
 			DEBUG_MSG("starting threaded search/replace\n");
@@ -354,7 +359,7 @@ static gpointer files_replace_run(gpointer data) {
 			if ((g_atomic_int_get(&rit->s3run->cancelled)==0) && rit->results && replacedbuf) {
 				DEBUG_MSG("replaced %d entries\n",g_list_length(rit->results));
 				outbuf = g_convert(replacedbuf, -1, encoding, "UTF-8", NULL, &outbuflen, NULL);
-				
+
 				g_file_replace_contents(rit->uri,outbuf,outbuflen,NULL,TRUE,G_FILE_CREATE_NONE,NULL,NULL,&gerror);
 				if (gerror) {
 					g_print("failed to save file: %s\n",gerror->message);
@@ -363,7 +368,7 @@ static gpointer files_replace_run(gpointer data) {
 				g_free(outbuf);
 			}
 			g_free(replacedbuf);
-			
+
 		}
 	}
 	rit->results = g_list_reverse(rit->results);
@@ -393,7 +398,7 @@ static void filematch_cb(Tsnr3run *s3run, GFile *uri, GFileInfo *finfo) {
 		DEBUG_MSG("filematch_cb, cancelled, do nothing\n");
 		return;
 	}
-	/* TODO: first check if we have this file open, in that case we have to run the 
+	/* TODO: first check if we have this file open, in that case we have to run the
 	function that replaces in the document */
 	doc = documentlist_return_document_from_uri(s3run->bfwin->documentlist, uri);
 	if (doc) {
@@ -425,9 +430,9 @@ void snr3_run_in_files_cancel(Tsnr3run *s3run) {
 	if (s3run->findfiles) {
 		findfiles_cancel(s3run->findfiles);
 	}
-	queue_cancel(&s3run->threadqueue, queue_cancel_freefunc, NULL);	
+	queue_cancel(&s3run->threadqueue, queue_cancel_freefunc, NULL);
 	/* hmm but what if there are still idle callbacks registered ???
-	the s3run structure can only be free'd after the fw refcount 
+	the s3run structure can only be free'd after the fw refcount
 	is 0 */
 }
 
